//

#include "camera.h"
#include "main.h"
#include "hardware.h"

//------------------------------------------------------------------------

Camera::Camera(){
	lastmanpos.set_xyz(0,0,0);
	lastmanorient.set_xyz(0,0,0);
	reset();
};
Camera::~Camera(){};

//------------------------------------------------------------------------

void Camera::reset(void){
	fov=90;
	eyex=0; eyey=2; eyez=0;

	place.reset();
	place.pos=lastmanpos;
	place.orient=lastmanorient;
}

//------------------------------------------------------------------------

void Camera::get_matrix(void) {
	glGetFloatv(GL_MODELVIEW_MATRIX,model);
	rightx  = model[0];	righty  = model[4];	rightz  = model[ 8];
	upx     = model[1];	upy     = model[5];	upz     = model[ 9];
	forwardx= model[2];	forwardy= model[6];	forwardz= model[10];

	irightx  = model[0];	irighty  = model[1];	irightz  = model[ 2];
	iupx     = model[4];	iupy     = model[5];	iupz     = model[ 6];
	iforwardx= model[8];	iforwardy= model[9];	iforwardz= model[10];

	translatex=model[12];
	translatey=model[13];
	translatez=model[14];
/*
	worldSizeNear= .025 *hw->sys->worldSize; // for custom culling
	worldSizeFar =-.8   *hw->sys->worldSize;
	worldSizeOff = .0375*hw->sys->worldSize;

	frustumrange2=pow((hw->screen->farz-hw->screen->nearz)*.2,2);//.5125w/.5x// for spherical frustum culling
	frustumcenterz=-((hw->screen->farz-hw->screen->nearz)*.19+hw->screen->nearz);
	frustumx=x+forwardx*frustumcenterz;
	frustumy=y+forwardy*frustumcenterz;
	frustumz=z+forwardz*frustumcenterz;

	if (hw->camera->cull(x,y,z)==0) continue;

*/
}

//------------------------------------------------------------------------

void Camera::orient(void) {
	float t=Hw::get()->time.time;
	gluLookAt(eyex,eyey,eyez, eyex,eyey,eyez-1,  0.,1.,0.);
	glRotatef(-place.orient.x+place.shakeampl*50*sinf(t*place.shakefreqx*.4),1,0,0);
	glRotatef(-place.orient.y+place.shakeampl*50*sinf(t*place.shakefreqy*.4),0,1,0);
	glTranslatef(-(place.pos.x+place.shake.x),
		          -(place.pos.y+place.shake.y),//-eyey,
				    -(place.pos.z+place.shake.z));
	get_matrix();
}

//------------------------------------------------------------------------

void Camera::matmult(float *inx, float *iny, float *inz) {
	float x,y;

	   x=irightx * *inx + iupx * *iny + iforwardx * *inz;
	   y=irighty * *inx + iupy * *iny + iforwardy * *inz;
	*inz=irightz * *inx + iupz * *iny + iforwardz * *inz;
	*inx=x;
	*iny=y;
}

//------------------------------------------------------------------------

float Camera::cull(float inx, float iny, float inz, float size) {// return 0 if out, else inDistance
	float slope,cx=inx-place.pos.x, cy=iny-place.pos.y, cz=inz-place.pos.z;

	matmult(&cx,&cy,&cz); // rotate
	cz-=size;
	if (eyez<=0) {
		if (cz>-Hw::get()->screen.nearz) return -1; // behind?
		if (cz<-Hw::get()->screen.farz)  return -2; // too far?
		slope=(-cz+Hw::get()->screen.nearz*1.5)*2;
		if (fabs(cx)>slope) return -3; // outside x/z view angle?
		slope*=1.5;
		if (fabs(cy)>slope) return -4; // outside y/z view angle?
	}
	return (cx*cx+cy*cy+cz*cz);
}

//------------------------------------------------------------------------
/*
float Camera::frustum_check(Vector v) {
	if ( (pow(frustumx-v.x,2)+ // distance from frustrum center
		  pow(frustumy-v.y,2)+
		  pow(frustumz-v.z,2))>frustumrange2) return 0; // outside
	else return (pow(x-v.x,2)+  // return sqr distance from camera
				 pow(y-v.y,2)+
				 pow(z-v.z,2));
}

//----------------------------------------------------------------------------

float Camera::frustum_check(float inx, float iny, float inz) {
	if ( (pow(frustumx-inx,2)+ // distance from frustrum center
		  pow(frustumy-iny,2)+
		  pow(frustumz-inz,2))>frustumrange2) return 0; // outside
	else return (pow(x-inx,2)+  // return sqr distance from camera
				 pow(y-iny,2)+
				 pow(z-inz,2));
}
*/
//------------------------------------------------------------------------

void Camera::billboard(float sizex, float sizey) {
	float rightxs,rightys,rightzs, upxs,upys,upzs;

	rightxs=rightx*sizex;
	rightys=righty*sizex;
	rightzs=rightz*sizex;
	upxs   =upx   *sizey;
	upys   =upy   *sizey;
	upzs   =upz   *sizey;
	glBegin(GL_QUADS);
		glTexCoord2f(0,0); glVertex3f(-rightxs-upxs,-rightys-upys,-rightzs-upzs);
		glTexCoord2f(1,0); glVertex3f( rightxs-upxs, rightys-upys, rightzs-upzs);
		glTexCoord2f(1,1); glVertex3f( rightxs+upxs, rightys+upys, rightzs+upzs);
		glTexCoord2f(0,1); glVertex3f(-rightxs+upxs,-rightys+upys,-rightzs+upzs);
		glEnd();
}

//------------------------------------------------------------------------

