//-------------------------------------------------------------------
//	File:		   config.cpp
//	Comments:   by George Ziniewicz
//-------------------------------------------------------------------

#include "config.h"
#include "hardware.h"

//-------------------------------------------------------------------

Config::Config() {};
Config::~Config() {};

//--------------------------------------------------------------------

#define NCONFIGITEMS 37
int Config::load(const char* title) {
	int match;
	//char *field0=new char[80],
	//	  *field1=new char[80],
	//	  *line  =new char[80], s[80];
	char *field0, *field1, line[80], s[80];
	Hw *hw=Hw::get();
	struct item{
		int type;
		float min,max;
		void *ptr;
		char name[22];
	} items[NCONFIGITEMS];
	int nitems=NCONFIGITEMS;

	item a[]={
//  f/i min    max var                     label
		1, 0,     1, &hw->pref.clock,        "clock",
		1, 0,     1, &hw->useMultitexture,   "useMultitexture",
		1, 0,     1, &hw->pref.demosequence, "demoSequence",
		1, 0,     1, &hw->pref.fullscreen,   "fullscreen",
		1, 0,     1, &hw->screen.lightmap.on,"lightmap",
		0, 0,     1, &hw->pref.letterbox,    "letterbox",
		1, 0,    20, &hw->pref.n,            "numericItem",
		1, 1, 10000, &hw->pref.randseed,     "randSeed",
		1, 0, 50000, &hw->pref.nstars,       "nstars",
		1, 0,     1, &hw->pref.user,         "user",
		1, 0,     1, &hw->skipSplash,        "skipSplash",
		0, 0,     1, &hw->pref.doubleClickInterval,"doubleClickInterval",
		0, 0,  1000, &hw->pref.spurt,              "spurt",
		0, 0,    10, &hw->pref.moveSens,           "moveSens",
		0, 0,    10, &hw->pref.mouseSens,          "mouseSens",
		0, 0,    10, &hw->pref.annotateDuration,   "annotateDuration",
		0,10,    100, &hw->pref.sceneduration,     "sceneDuration",
		1, 0,     1, &hw->pref.scrollInfo ,        "scrollInfo",
		0, 0,    10, &hw->pref.scrollSpeed,        "scrollSpeed",
		0, 0,  1000, &hw->pref.scrollInterval,     "scrollInterval",
		0, 0,     1, &hw->pref.ambient,   "ambient",
		0, 0,     1, &hw->pref.diffuse,   "diffuse",
		0, 0,     1, &hw->pref.specular,  "specular",
		0, 0,     1, &hw->pref.reflective,"reflective",
		0, 0,     1, &hw->pref.exponent,  "exponent",
		0, 0,   180, &hw->pref.cutoff,    "cutoff",
		1, 0,   128, &hw->pref.shininess, "shininess",
		1, 0,     9, &hw->pref.useSoundDriver,  "useSoundDriver",
		0, 0,   100, &hw->pref.minSoundInterval,"minSoundInterval",
		0, 0,   100, &hw->pref.maxSoundInterval,"maxSoundInterval",
		0, 0,     1, &hw->pref.stepVolume,      "stepVolume",
		0, 0,     1, &hw->pref.ambientvolume,   "ambientVolume",
		0, 0,     1, &hw->pref.songvolume,      "songVolume",
		0, 0,     1, &hw->pref.stepPan,         "stepPan",
		0, 1,    100,&hw->pref.gruntMPS,        "gruntMPS",
		0, 0,    100,&hw->pref.gravityScaler,   "gravityScaler",
		0, 0,    100,&hw->pref.dragScaler,      "dragScaler"
	};

	for (int i=0; i<nitems; i++) { // build list for proper addressing
		items[i]=a[i];
	}

	strcpy(s,title); // build filename to open
	strcat(s,".config");
	ifstream file(s,ios::nocreate);
	if (file.bad()||(file==0)) return 1;

	while (file.eof()==0) {
		match=-1;
		file.getline(line,80);                       // read line
		if ((line[0]==0)||(line[0]=='/')) continue;  // empty or comment line?
		field0=strtok(line," =\0");                  // extract name field
		if (field0[0]==0) continue;                  // empty field?
		field1=strtok(NULL," \0");                   // extract value field
		if (field1[0]!=0) {                          // not empty field?
			for (i=0; i<nitems; i++) {                // scan table for name
				if (strcmp(field0,items[i].name)==0) { // match?
					match=i;
					switch(items[i].type) {
						case 0: *((float*)items[i].ptr)=clampf((float)atof(field1),items[i].min,items[i].max); break;
						case 1: *((  int*)items[i].ptr)=clampi(       atoi(field1),items[i].min,items[i].max); break;
					}
					break;
				}
			}
			if (match>-1) { // log if found valid name
				sprintf(s,"Config: %s %s",field0,field1); // log
				log_message(s);
			} else {        // log error
				sprintf(s,"Config: %s %s cannot be processed ************",field0,field1); // log
				log_message(s);
			}
		}
		field0[0]=0;
		field1[0]=0;
	}
	file.close();
	return 0;
}

//--------------------------------------------------------------------


