// display manager w/
//   displayable objects w/
//      pointer to geom obj (mesh,billboard,sphere...)
//      pos/orient controller
//      visual attribute handler

#include "disp.h"
#include "hardware.h"
#include "physics.h"

extern PFNGLMULTITEXCOORD2FARBPROC     glMultiTexCoord2fARB;
extern PFNGLACTIVETEXTUREARBPROC       glActiveTextureARB;
//extern PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB;

//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------

DispMan::DispMan(){
	nobjects=0;
	selObject=0;
	for (int i=0; i<MAXDISPLAYOBJECTS; i++) {
		//sizes[i]=1;
		ons[i]=1;
	}
};
DispMan::~DispMan(){};

//----------------------------------------------------------------------------

void DispMan::reset(void){
	Vis::reset();
	TexAttrib::reset();
}

//----------------------------------------------------------------------------

void DispMan::unset(void){
	Vis::unset();
	for (int i=0; i<Hw::get()->nTextureUnits; i++) TexAttrib::unset(i);
}

//----------------------------------------------------------------------------

int DispMan::add(BaseObject *base){
	if (nobjects>=MAXDISPLAYOBJECTS) return -1;
	dos[nobjects].object=base;
	return nobjects++;
}

//----------------------------------------------------------------------------

void DispMan::draw(int n){
	if (n>=nobjects) return;
	if (dos[n].vis.cull>1) {
		if (Hw::get()->screen.viewport->camera.cull(dos[n].place.pos.x,
			                                         dos[n].place.pos.y,
																  dos[n].place.pos.z,
																  sizes[n])<0) return;
																  //dos[n].geomattrib.size)<0) return;
	}
	dos[n].draw(sizes[n]);
	//dos[n].draw(dos[n].geomattrib.size);
}

//----------------------------------------------------------------------------

void DispMan::update(void){
	int editflag;

	for(int i=0; i<nobjects; i++) {
		editflag=(((Hw::get()->screen.viewport->camera.place.movemode!=MANUAL)|| // not in manual cam mode?
		           (Hw::get()->pause==1))&&   // | paused?
		           (Hw::get()->dialogs==0)&&  // | not dialog ?
					  (i==selObject-1));              // & this 1 selected?
		dos[i].place.update_pos(editflag); // move w/ possible manual control
		dos[i].place.physics->damp(&dos[i].place); // damp
		if (i==selObject-1) pref2object(i); // selected obj?
	}
}

//----------------------------------------------------------------------------

void DispMan::pref2object(int i) {
	Hw *hw=Hw::get();

	dos[i].vis.alphafunc =hw->pref.alphafunc; // update prefs->objects
	dos[i].vis.blendFunc0=hw->pref.blendFunc0;
	dos[i].vis.blendFunc1=hw->pref.blendFunc1;
	dos[i].vis.r         =hw->pref.r;
	dos[i].vis.g         =hw->pref.g;
	dos[i].vis.b         =hw->pref.b;
	dos[i].vis.a         =hw->pref.a;

	dos[i].texattribs[hw->pref.seltexlayer].image     =hw->pref.image;
	dos[i].texattribs[hw->pref.seltexlayer].texmagx   =hw->pref.texmagx;
	dos[i].texattribs[hw->pref.seltexlayer].texmagy   =hw->pref.texmagy;
	dos[i].texattribs[hw->pref.seltexlayer].texoffx   =hw->pref.texoffx;
	dos[i].texattribs[hw->pref.seltexlayer].texoffy   =hw->pref.texoffy;
	dos[i].texattribs[hw->pref.seltexlayer].blendFunc0=hw->pref.texBlendFunc0;
	dos[i].texattribs[hw->pref.seltexlayer].blendFunc1=hw->pref.texBlendFunc1;

	dos[i].material.ambient   =hw->pref.ambient;
	dos[i].material.diffuse   =hw->pref.diffuse;
	dos[i].material.specular  =hw->pref.specular;
	dos[i].material.reflective=hw->pref.reflective;
	dos[i].material.exponent  =hw->pref.exponent;
	dos[i].material.cutoff    =hw->pref.cutoff;
	dos[i].material.shininess =hw->pref.shininess;
	dos[i].material.r         =hw->pref.mat_r;
	dos[i].material.g         =hw->pref.mat_g;
	dos[i].material.b         =hw->pref.mat_b;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

DispObj::DispObj(){
	object=0;
	label[0]=0;
};
DispObj::~DispObj(){};

//----------------------------------------------------------------------------

void DispObj::draw(float size){
	int i, unitn=0, multi=0, layern=0, layers[5],tas[5];
	int useTextureUnits=Hw::get()->useTextureUnits;

	vis.set();
	glPushMatrix();
		place.set(); // place object
		do{
			while((unitn<useTextureUnits)&&(layern<vis.ntexlayers)) { // dole out layers to units
				texattribs[layern].set(unitn);        // set texture attribs
				//texattribs[layern].set_matrix(unitn); // tweak texture matrix
				TexAttrib::layerunits[layern]=unitn;  // correlate layer-unit for texcoord gen
				layers[unitn]=layern;                 // save for matrix pop
				tas[unitn]=unitn;                     //  "
				unitn++;                              // next unit
				layern++;                             // next layer
				if (Hw::get()->do2ndlayer==0) break;
			}
			if (multi==1) {
				glDepthFunc(GL_EQUAL);     // enable multipass depth matching
				if (texattribs[layern].blend) {
					glEnable(GL_BLEND);
					set_blend(texattribs[layern].blendFunc0,texattribs[layern].blendFunc1);
				}
			}
			material.set();
			if (object!=0) object->draw(size);       // render
			for (i=0; i<unitn; i++) texattribs[layers[i]].unset_matrix(tas[i]); // pop all texture matrices
			unitn=0;                                 // reset unit
			multi++;                                 // count multipass layers
		} while ((layern<vis.ntexlayers)&&(Hw::get()->do2ndlayer=1));            // more to do?
		glPopMatrix();
	if (multi>1) {
		glDepthFunc(GL_LESS);             // reset depth calc
		if (texattribs[layern].blend) glDisable(GL_BLEND);
	}
}

//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------

