//
//  "this" main graphic effect class
//
//   standard functions:
//      init()
//      reset();
//      start_frame();
//      key();
//      draw();
//      annot();
//      end_frame();
//      close();
//

#include "effect.h"
#include "text.h"
#include "geom.h"
#include "camerascript.h"

extern Texture fontT,font2T;  // clean this up ///////////////////
extern int demoend;
extern Fog fog;
extern CameraScript camscript;

extern PFNGLMULTITEXCOORD2FARBPROC     glMultiTexCoord2fARB;
extern PFNGLACTIVETEXTUREARBPROC       glActiveTextureARB;


//------------------------------------------------------------------------

Effect::Effect(){
	sphereT=0; // external sphere texture ptr
	nobjects=12;
	effectn=0;
	neffects=7;
	collide=0;
	physics=0;
	totalscenes=0;
	init_script();
	nite=0;
	soundindex1=soundindex2=soundindex3=songindex=0;
};
Effect::~Effect(){};

//------------------------------------------------------------------------

void Effect::annot(Text *ann){
	if (hw->screen.capture) return;
	//char s[80];
	//sprintf(s,"si: %d",scriptindex); ann->hold(s);
}

//------------------------------------------------------------------------

void Effect::close(void){
	columns.dl.destroy();
	lightning.deinit();
}

//------------------------------------------------------------------------

void Effect::draw(Text *ann){
	proc_effect(1);
	//draw_eye();
}

//------------------------------------------------------------------------

void Effect::end_frame(Text *ann){
	Place *place=&hw->screen.viewport->camera.place;
	float camy=hw->screen.viewport->camera.eyey;

   if (hw->time.time>10) {
		if ( (sign((place->    pos.y+camy)-dispman.dos[1].place.pos.y) != // just thru cloud layer 2?
				sign((place->lastpos.y+camy)-dispman.dos[1].place.pos.y)) ) {	
			hw->sound.play_type("sounds/whoosh2811.wav",FSOUND_MONO);
			hw->sound.set_volume(.2);
			hw->sound.set_frequency(120000);
		}
		if ( (sign((place->    pos.y+camy)-dispman.dos[2].place.pos.y) != // just thru cloud layer 2?
				sign((place->lastpos.y+camy)-dispman.dos[2].place.pos.y)) ) {	
			hw->sound.play_type("sounds/whoosh2811.wav",FSOUND_MONO);
			hw->sound.set_volume(.2);
			hw->sound.set_frequency(140000);
		}
	}
	hw->   screen.viewport->camera.place.lastpos=
	   hw->screen.viewport->camera.place.pos;
}

//------------------------------------------------------------------------

void Effect::init(Text *ann){
	hw=Hw::get();
	int i=0;
	float ws=hw->worldSize, meter=hw->meter;
	char s[80]={0},log[80]={0};

	log_message("Init effect...");

	reset(ann);

	for (i=0; i<4; i++) { // program all cameras
		hw->screen.viewports[i].camera.place.moveRange. set_xyz(200,40,200);  // set camera movement
		hw->screen.viewports[i].camera.place.moveCenter.set_xyz(5,50,10);
		hw->screen.viewports[i].camera.place.spinRange. set_xyz(70,720,0);
		hw->screen.viewports[i].camera.place.moverate=1;
		hw->screen.viewports[i].camera.place.pos.set_xyz(0,0,0);//.2*ws,0);
		switch(i) {
			case 0:                     // 1st main viewport/camera
				//if (hw->skipSplash) 
				//hw->screen.viewports[i].camera.place.movemode=MANUAL;
				//else
				hw->screen.viewports[i].camera.place.movemode=LOOKAT;
				break;
			case 1: hw->screen.viewports[i].camera.place.movemode=ORBIT; break;
			case 2:
				hw->screen.viewports[i].camera.place.movemode=LOOKAT;
				hw->screen.viewports[i].camera.place.moveCenter.set_xyz(-40,50,10);
				hw->screen.viewports[i].camera.place.to.set_xyz(0,40,0);
				break;
			case 3: hw->screen.viewports[i].camera.place.movemode=LOOKFROM; break;
		}
		hw->screen.viewports[i].camera.place.otherPlace=&dispman.dos[3].place; // lookfrom
	}

	spacePH.gravity.   set_xyz(        0,  0*meter,        0); // zone physics ----
	spacePH.movedrag.  set_xyz( .4*meter, .4*meter, .4*meter); // auto-thruster drag
	spacePH.friction.  set_xyz(  0*meter,  0*meter,  0*meter);
	spacePH.rotatedrag.set_xyz(  7*meter,  7*meter,  7*meter);
	spacePH.spdLimit=300;

	airPH.gravity.   set_xyz(        0,-9.8*meter,       0); // 32 f/s
	airPH.movedrag.  set_xyz(  1*meter,  1*meter,  1*meter);
	airPH.friction.  set_xyz(  0*meter,  0*meter,  0*meter);
	airPH.rotatedrag.set_xyz(  7*meter,  7*meter,  7*meter);
	airPH.spdLimit=200;

	groundPH.gravity.   set_xyz(        0,  0*meter,        0);
	groundPH.movedrag.  set_xyz(1.5*meter,1.5*meter,1.5*meter);
	groundPH.friction.  set_xyz(  .5*meter,  .5*meter,  .5*meter);
	groundPH.rotatedrag.set_xyz(  7*meter,  7*meter,  7*meter);
	groundPH.spdLimit=50;

	log_message("Init effect meshes...");  // meshes ---------------------------

	groundM.cullStrip=0;
	groundM.newmesh(20,20, ws,ws);

	volcanoM.cullStrip=0;
	volcanoM.newmesh(60,60, ws,ws);

	fallM.cullStrip=0;
	fallM.newmesh(20,20, ws*.1,ws*.1);

	ceilingM.cullStrip=0;
	ceilingM.newmesh(21,21, ws,ws);
	ceilingM.normal.set_xyz(0,-1,0);

	bb.sizex=32;
	bb.sizey=bb.sizex;
	bb.tx0=.05;
	bb.tx1=.95;

	cloud1M.newmesh(4,4, ws*1.5,ws*1.5);

	globe1.size=40;
	globe2.size=40.1;
	globe1.tessdistmin=100; globe1.mindisttess=70; // close up
	globe2.tessdistmin=100; globe2.mindisttess=70; // faraway
	globe1.tessdistmax=400; globe1.maxdisttess=30; // close up
	globe2.tessdistmax=400; globe2.maxdisttess=30; // faraway

	torus.tessdistmin=100; torus.mindisttess=160; // close up
	torus.tessdistmax=400; torus.maxdisttess=30; // faraway
	torus.innerclip=-.1; // flatten inner ring surface
	torus.outerpoint=.2; // round outer ring surface

	particle_init();  // volcano smoke in p1P

	log_message("Init effect sound...");
	hw->sound.set_rolloff(.9);
	hw->sound.set_master_volume(0);

	soundindex1=hw->sound.place_loop("sounds/wo11.wav",-50,50,0);
	soundindex2=hw->sound.place_loop("sounds/ho7.wav",  50,50,0);
	soundindex3=hw->sound.place_loop("sounds/wind.wav",  0,70,0);

	hw->sound.playex=1; // delay play
	hw->sound.play2d("sounds/1i_16_32_2.mp3");
	songindex=hw->sound.samplen;
	hw->sound.set_reserved();
	hw->sound.set_volume(0);
	hw->sound.playex=0;

	/*
	strcpy(s,"sounds/1i_16_32_2.mp3");
	//hw->sound.set_reverb(13,hw->ann);
	samplesong=FSOUND_Sample_Load(FSOUND_FREE,s,FSOUND_2D,0);
	if (samplesong==0)	{
		sprintf(log,"Can't load sample as 2d **********", s); log_message(log);
		samplesong=FSOUND_Sample_Load(FSOUND_FREE,s,FSOUND_STEREO,0);
	}
	if (samplesong==0) {
		sprintf(log,"Can't load sample: %s as stereo **********", s); log_message(log);	
		samplesong   =FSOUND_Sample_Load(FSOUND_FREE,s,FSOUND_MONO,0);
	}
	if (samplesong==0) {
		sprintf(log,"Can't load sample: %s as mono **********", s); log_message(log);	
	}
	if (samplesong!=0) {
		hw->sound.finish_2d(samplesong);
		hw->sound.set_reserved();
		hw->sound.set_volume(hw->pref.songvolume);
	}
*/

	log_message("Init effect textures..."); // textures -------------------------------
	if (imageT[9].load_tex("images/cloudbw2128.bmp", GL_LUMINANCE_ALPHA, -1, 0, 0)) log_message("Cant make cloud texture **********");
	if (imageT[0].load_tex("images/monacolor128.bmp",GL_RGBA, 255, 0, 1))log_message("Cant make mona **********");
	if (imageT[1].load_tex("images/waterpool256.bmp",GL_RGBA, -1, 0, 1)) log_message("Cant make  **********");
	if (imageT[2].load_tex("images/fisheye.bmp",     GL_RGBA, -1, 0, 1)) log_message("Cant make  **********");
	if (imageT[3].load_tex("images/earth_256.bmp",   GL_RGBA, -1, 0, 1)) log_message("Cant make earth **********");
	//if (imageT[]. load_tex("images/.bmp",             GL_RGBA, -1, 0, 1)) log_message("Cant make  **********");

	if (imageT[8].load_tex("images/cloudsphere256.bmp", GL_LUMINANCE_ALPHA, -1, 0, 0)) log_message("Cant make cloudsphere texture **********");
	if (imageT[7].load_tex("images/cloudsand.bmp",      GL_LUMINANCE_ALPHA, -1, 0, 1)) log_message("Cant make cloudsand **********");
	//if (imageT[4].load_tex("images/mur023_256.bmp", GL_RGBA, -1, 0, 1)) log_message("Cant make mur023 **********");
	if (imageT[4].load_tex("images/mur079_256.bmp", GL_RGBA, -1, 0, 1)) log_message("Cant make mur079 **********");
	if (imageT[5].load_tex("images/mot240_256.bmp", GL_RGBA, -1, 0, 1)) log_message("Cant make mot240 **********");
	if (imageT[6].load_tex("images/sup074_256.bmp", GL_RGBA, -1, 0, 1)) log_message("Cant make sup74 **********");

	if (smokeT.make_tex(SMOKE,256,256,GL_LUMINANCE_ALPHA,1)) log_message("Cant make smoke texture **********");
	if (lavaT.load_tex("images/sue074_256.bmp", GL_RGBA, 255, 0, 1)) log_message("Cant make lava texture **********");
	if (imageT[10].load_tex("images/sup076_256.bmp",   GL_RGBA, -1, 0, 1)) log_message("Cant make sup076 **********");
	if (imageT[11].load_tex("images/marblegood128.bmp",GL_RGBA, -1, 0, 1)) log_message("Cant make sup076 **********");
	if (eyeT.make_tex(BLANK,256,256,GL_RGBA,0)) log_message("Cant make eyet **********");
	if (tmpT.make_tex(BLANK,256,256,GL_RGBA,0)) log_message("Cant make tmpt **********");
	if (imageT[12].load_tex("images/moon128.bmp",GL_RGBA, -1, 0, 1)) log_message("Cant make moon **********");
	if (arcT.make_tex(SPARK,128,128,GL_LUMINANCE_ALPHA,0)) log_message("Cant make arc **********");

	log_message("Init effect objects...");
	load_data();
	effect0(-1); //pre-init effect
	effect1(-1);
	effect2(-1);
	effect3(-1);
	effect4(-1);
	effect5(-1);
	effect6(-1);
	lastscenetime=0;

	lightning.arcT=&arcT;
	lightning.sphereT=sphereT;
	lightning.init();
	lightning.width=6;
	lightning.crag=10;
	lightning.ballDuration=.3;
	lightning.ballsize=100;
	lightning.flashInterval=1.2;//1
}

//------------------------------------------------------------------------

void Effect::key(Text *ann){
	if ((hw->key.alt==0)&&(hw->key.control==0)&&(hw->key.key!=0)) {
		switch(hw->key.key){

			case 'e': dispman.selObject=(dispman.selObject>=dispman.nobjects)?               0:dispman.selObject+1; break;
			case 'E': dispman.selObject=(dispman.selObject<=               0)?dispman.nobjects:dispman.selObject-1; break;

			case 'i': if (dispman.selObject!=0)
							 dispman.dos[dispman.selObject-1].texattribs[hw->pref.seltexlayer].image=hw->pref.image;
				break;

			case 'm': hw->useMultitexture=1-hw->useMultitexture;
				ann->hold_multi(hw->useMultitexture,"useMultitexture Off","useMultitexture On");
				if (hw->useMultitexture) Hw::get()->useTextureUnits=Hw::get()->nTextureUnits;
				else                     Hw::get()->useTextureUnits=1;
				break;
			case 'M': hw->do2ndlayer     =1-hw->do2ndlayer;
				ann->hold_multi(hw->do2ndlayer,"do2ndlayer Off","do2ndlayer On");
				break;

			case '@': case 'r': object2pref(); load_data(); break;

			case 'o': set_option(ann); break;

			case '%':  // step effect qqq
				proc_effect(2);
				effectn=(effectn==neffects-1)? 0:effectn+1;
				proc_effect(0);
				break;

			case 127: // jump
				hw->screen.viewport->camera.place.vel.y+=15*hw->meter;
				hw->screen.viewport->camera.place.pos.y +=.1; // take off ground
				break;
		}
	}
	if ((hw->key.key=='e')||(hw->key.key=='E')) {
		if (dispman.selObject>0) Hw::get()->ann->hold(dispman.dos[dispman.selObject-1].label);
		else Hw::get()->ann->hold("No object selected");
		object2pref();
	}

	if (hw->key.skey) {
		if ((hw->key.alt==1)&&(hw->key.control==0)&&(hw->key.shift==0)) { // alt bank
			switch(hw->key.skey){
				case GLUT_KEY_F1:  dispman.ons[ 0]=!dispman.ons[ 0]; ann->hold("0: %d", dispman.ons[ 0]); break;
				case GLUT_KEY_F2:  dispman.ons[ 1]=!dispman.ons[ 1]; ann->hold("1: %d", dispman.ons[ 1]); break;
				case GLUT_KEY_F3:  dispman.ons[ 2]=!dispman.ons[ 2]; ann->hold("2: %d", dispman.ons[ 2]); break;
				case GLUT_KEY_F4:  dispman.ons[ 3]=!dispman.ons[ 3]; ann->hold("3: %d", dispman.ons[ 3]); break;
				case GLUT_KEY_F5:  dispman.ons[ 4]=!dispman.ons[ 4]; ann->hold("4: %d", dispman.ons[ 4]); break;
				//case GLUT_KEY_F6:  dispman.ons[ 5]=!dispman.ons[ 5]; ann->hold("5: %d", dispman.ons[ 5]); break;
				case GLUT_KEY_F7:  dispman.ons[ 5]=!dispman.ons[ 5]; ann->hold("5: %d", dispman.ons[ 5]); break;
				case GLUT_KEY_F8:  dispman.ons[ 6]=!dispman.ons[ 6]; ann->hold("6: %d", dispman.ons[ 6]); break;
				case GLUT_KEY_F9:  dispman.ons[ 7]=!dispman.ons[ 7]; ann->hold("7: %d", dispman.ons[ 7]); break;
				case GLUT_KEY_F10: dispman.ons[ 8]=!dispman.ons[ 8]; ann->hold("8: %d", dispman.ons[ 8]); break;
				case GLUT_KEY_F11: dispman.ons[ 9]=!dispman.ons[ 9]; ann->hold("9: %d", dispman.ons[ 9]); break;
				case GLUT_KEY_F12: dispman.ons[10]=!dispman.ons[10]; ann->hold("10: %d",dispman.ons[10]); break;
			}
		}
		if ((hw->key.alt==0)&&(hw->key.control==1)&&(hw->key.shift==0)) { // control bank
			switch(hw->key.skey){
				case GLUT_KEY_F1: vars[0]*=0.95   ; ann->hold("Var0: %.3f",vars[0]); break;
				case GLUT_KEY_F2: vars[0]*=1.05263; ann->hold("Var0: %.3f",vars[0]); break;
				case GLUT_KEY_F3: vars[1]*=0.95   ; ann->hold("Var1: %.3f",vars[1]); break;
				case GLUT_KEY_F4: vars[1]*=1.05263; ann->hold("Var1: %.3f",vars[1]); break;
				case GLUT_KEY_F5: vars[2]*=0.95   ; ann->hold("Var2: %.3f",vars[2]); break;
				case GLUT_KEY_F6: vars[2]*=1.05263; ann->hold("Var2: %.3f",vars[2]); break;
				case GLUT_KEY_F7: vars[3]*=0.95   ; ann->hold("Var3: %.3f",vars[3]); break;
				case GLUT_KEY_F8: vars[3]*=1.05263; ann->hold("Var3: %.3f",vars[3]); break;
				case GLUT_KEY_F9: vars[4]*=0.95   ; ann->hold("Var4: %.3f",vars[4]); break;
				case GLUT_KEY_F10:vars[4]*=1.05263; ann->hold("Var4: %.3f",vars[4]); break;
				case GLUT_KEY_F11:vars[5]*=0.95   ; ann->hold("Var5: %.3f",vars[5]); break;
				case GLUT_KEY_F12:vars[5]*=1.05263; ann->hold("Var5: %.3f",vars[5]); break;
			}
		}
		if ((hw->key.alt==0)&&(hw->key.control==0)&&(hw->key.shift==0)) { // plain
			switch(hw->key.skey){
				case GLUT_KEY_PAGE_UP: case GLUT_KEY_PAGE_DOWN:
					break;
			}
		}
	}
}

//------------------------------------------------------------------------

void Effect::load_data(void){
	BaseObject *os  []={&groundM,&cloud1M,&cloud1M,&globe1,&globe2,&torus,&bb,&ceilingM,&rays,&columns,&volcanoM, &fallM};

	log_message("Load object data...");
	if (dispman.nobjects==0) {
		for (int i=0; i<nobjects; i++)
			dispman.add(os[i]); //create object pak (object+vis+texattr+mat+place)
	}
	load_shader(hw->title);
	load_script(hw->title);
}

//------------------------------------------------------------------------------------

int Effect::load_shader(const char* infile) {
	char s[200],*lastStr, logstr[80];
	int i,n, objectn, nfields[]={16,11,11,7}, section=0; // vis/tex/mat/place
	float array[20];
	Place *p;
	Vis *v;
	Material *m;
	TexAttrib *t;

	strcpy(s,infile);
	strcat(s,".shader"); // build filename to open
	ifstream file(s,ios::nocreate);
	if (file.bad()||(file==0)) {
		log_message("************** cant open shader file ***************");
		return 1;
	}

	nobjects=0;
	file.getline(s,180); // skip 1st comment line
	while (file.eof()==0) { // more to read?
		file.getline(s,180); // read line
		if ((s[0]==0)||(s[0]=='/')||(strlen(s)<5)) continue; // skip?
		if (strcmp(s,"[visuals]")   ==0) {section=0; objectn=0; log_message("   Visuals...");    continue;}
		if (strcmp(s,"[texattribs]")==0) {section=1; objectn=0; log_message("   Texattribs..."); continue;}
		if (strcmp(s,"[materials]") ==0) {section=2; objectn=0; log_message("   Materials...");  continue;}
		if (strcmp(s,"[placement]") ==0) {section=3; objectn=0; log_message("   Placements..."); continue;}
		for (i=0; i<nfields[section]; i++) { // read all fields to array
			if      (i==0)                 array[i]=atof(strtok(s,   ", ")); // 1st field
			else if (i<nfields[section]-1) array[i]=atof(strtok(NULL,", ")); // other fields
			else {
				if (section>1)              array[i]=atof(strtok(NULL,", \n")); // last field
				else 					           lastStr=     strtok(NULL,", \n");  // last string field
			}
		}
		n=0;
		switch(section) {
			case 0:
				sprintf(logstr,"      Object %2d, %s",nobjects, lastStr); log_message(logstr);
				v=&dispman.dos[nobjects].vis; // stuff vis attribs
				v->alphatest =array[n++];
				v->alphafunc =array[n++];
				v->blend     =array[n++];
				v->blendFunc0=array[n++];
				v->blendFunc1=array[n++];
				v->r         =array[n++];
				v->g         =array[n++];
				v->b         =array[n++];
				v->a         =array[n++];
				v->cull      =array[n++];
				v->depthtest =array[n++];
				v->depthwrite=array[n++];
				v->fog       =array[n++];
				v->lights    =array[n++];
				v->ntexlayers=array[n++];
				strcpy(dispman.dos[nobjects].label,lastStr);
				nobjects++;
				break;
			case 1:
				//sprintf(logstr,"      tex object: %d   layer: %d",objectn, (int)array[0]); log_message(logstr);
				t=&dispman.dos[objectn].texattribs[(int)array[n++]];
				t->textures  =array[n++];
				t->texmagx   =array[n++];
				t->texmagy   =array[n++];
				t->texoffx   =array[n++];
				t->texoffy   =array[n++];
				t->image     =array[n++];
				t->blend     =array[n++];
				t->blendFunc0=array[n++];
				t->blendFunc1=array[n++];
				if      (strcmp(lastStr,"ADD"        )==0) {t->function=GL_ADD;            }
				else if (strcmp(lastStr,"ADD_SIGNED" )==0) {t->function=GL_ADD_SIGNED_EXT; }
				else if (strcmp(lastStr,"MODULATE"   )==0) {t->function=GL_MODULATE;       }
				else if (strcmp(lastStr,"INTERPOLATE")==0) {t->function=GL_INTERPOLATE_EXT;}
				else                                 {      t->function=GL_ADD;}

				if ((int)array[0]==dispman.dos[objectn].vis.ntexlayers-1) objectn++; // fin all layers?
				break;
			case 2:
				//sprintf(logstr,"      material object: %d",objectn); log_message(logstr);
				m=&dispman.dos[objectn].material;
				m->on        =array[n++]; // stuff material attribs
				m->ambient   =array[n++];
				m->diffuse   =array[n++];
				m->specular  =array[n++];
				m->reflective=array[n++];
				m->shininess =array[n++];
				m->exponent  =array[n++];
				m->cutoff    =array[n++];
				m->r         =array[n++];
				m->g         =array[n++];
				m->b         =array[n++];
				objectn++;
				break;
			case 3:
				//sprintf(logstr,"      placement object: %d",objectn); log_message(logstr);
				p=&dispman.dos[objectn].place;
				dispman.sizes[objectn]=array[n++]; // stuff placement attribs
				p->pos.x        =array[n++];
				p->pos.y        =array[n++];
				p->pos.z        =array[n++];
				p->orient.x     =array[n++];
				p->orient.y     =array[n++];
				p->orient.z     =array[n++];
				p->physics      =&airPH;
				objectn++;
				break;
		}
	}
	file.close();
	return 0;
}

//-------------------------------------------------------------------

void Effect::object2pref(void) {
	Vis *vis;
	TexAttrib *ta;
	Material *mat;

	if (dispman.selObject) { // map object attr to pref 
		vis=&dispman.dos[dispman.selObject-1].vis;
		ta =&dispman.dos[dispman.selObject-1].texattribs[hw->pref.seltexlayer];
		mat=&dispman.dos[dispman.selObject-1].material;

		hw->pref.alphafunc =vis->alphafunc;
		hw->pref.blendFunc0=vis->blendFunc0;
		hw->pref.blendFunc1=vis->blendFunc1;
		hw->pref.r         =vis->r;
		hw->pref.g         =vis->g;
		hw->pref.b         =vis->b;
		hw->pref.a         =vis->a;

		hw->pref.image        =ta->image;
		hw->pref.texmagx      =ta->texmagx;
		hw->pref.texmagy      =ta->texmagy;
		hw->pref.texoffx      =ta->texoffx;
		hw->pref.texoffy      =ta->texoffy;
		hw->pref.texBlendFunc0=ta->blendFunc0;
		hw->pref.texBlendFunc1=ta->blendFunc1;

		hw->pref.mat_r     =mat->r;
		hw->pref.mat_g     =mat->g;
		hw->pref.mat_b     =mat->b;
		hw->pref.ambient   =mat->ambient;
		hw->pref.diffuse   =mat->diffuse;
		hw->pref.specular  =mat->specular;
		hw->pref.reflective=mat->reflective;
		hw->pref.exponent  =mat->exponent;
		hw->pref.cutoff    =mat->cutoff;
		hw->pref.shininess =mat->shininess;
	}
}

//-----------------------------------------------------------------------

void Effect::particle_init(void) {
	Particle p;
	float meter=hw->meter;

	log_message("Init effect particles...");

	//------------------------------------------------
	p.interval=.05;     p.lifeTime=5;       p.lifeTimeRnd=1;
	p.size0=meter*.01;
   p.size1=meter*20;   p.sizey=1;          p.length=0*meter;   p.sizeRnd=0; 
	p.r0=1;             p.g0=.125;          p.b0=.125;          p.alpha0=1;   p.colorRnd=.1;
	p.r1=1;             p.g1=.125;          p.b1=.125;          p.alpha1=0;
	
	p.windx=0*meter; p.dispersex=0*meter; p.xi=0*meter;
	p.windy=0*meter; p.dispersey=0*meter; p.yi=5*meter;
	p.windz=0*meter; p.dispersez=0*meter; p.zi=0*meter;

	p.rollspeed=0;

	p.alphatest=1;      p.alphafunc=.05;        p.depthtest=1;    p.depthwrite=0;
	p.blend=1;          p.blendFunc0=6;         p.blendFunc1=1;
	p.maxparticles=900;//	p.lifeTime   / p.interval;
	p.init=1;           p.tex=&smokeT;
   p1P=p;

	//------------------------------------------------ water

	p.interval=.05;     p.lifeTime=2;       p.lifeTimeRnd=1;
	p.size0=meter*.01;
   p.size1=meter*10;   p.sizey=1;          p.length=0*meter; p.sizeRnd=.1; 
	p.r0=1;             p.g0=1;          p.b0=1;          p.alpha0=1;   p.colorRnd=.1;
	p.r1=1;             p.g1=1;          p.b1=1;          p.alpha1=0;
	
	p.windx=0*meter; p.dispersex=0*meter; p.xi=2*meter;
	p.windy=0*meter; p.dispersey=0*meter; p.yi=0*meter;
	p.windz=0*meter; p.dispersez=0*meter; p.zi=0*meter;

	p.rollspeed=0;

	p.alphatest=1;      p.alphafunc=.05;       p.depthtest=1;    p.depthwrite=0;
	p.blend=1;          p.blendFunc0=6;        p.blendFunc1=1;
	p.maxparticles=900;//	p.lifeTime / p.interval;
	//p.init=1;         p.tex=sphereT;//&smokeT;
	p.init=1;           p.tex=&smokeT;
	p2P=p;

	//------------------------------------------------ eye

	p.interval=0;       p.lifeTime=1.2;       p.lifeTimeRnd=.3;
	p.size0=meter*10;
   p.size1=meter*.1;   p.sizey=1;          p.sizeRnd=5;      p.length=0*meter; 

	p.r0=1;             p.g0=0;          p.b0=0;          p.alpha0=1;   p.colorRnd=.25;//3,4
	p.r1=1;             p.g1=.5;         p.b1=.25;        p.alpha1=0;
	
	p.windx=0*meter; p.dispersex=.5*meter; p.xi=0*meter;
	p.windy=0*meter; p.dispersey=.5*meter; p.yi=0*meter;
	p.windz=0*meter; p.dispersez=.5*meter; p.zi=0*meter;

	p.rollspeed=50;

	p.alphatest=1;      p.alphafunc=.01;       p.depthtest=0;    p.depthwrite=0;
	p.blend=1;          p.blendFunc0=1;        p.blendFunc1=1;
	p.maxparticles=900; //	p.lifeTime / p.interval;
	p.init=1;           p.tex=&smokeT;//sphereT
	p3P=p;
}

//-----------------------------------------------------------------------

void Effect::proc_effect(int mode){
	//effect4(mode);	return;
	switch(effectn) {
		case 0: effect0(mode); break;
		case 1: effect1(mode); break;
		case 2: effect2(mode); break;
		case 3: effect3(mode); break;
		case 4: effect4(mode); break;
		case 5: effect5(mode); break;
		case 6: effect6(mode); break;
	}
}

//------------------------------------------------------------------------

void Effect::reset(Text *ann){
	static int init=0;

	log_message("Reset effect...");
	hw->pref.blendFunc0=7;//6
	hw->pref.blendFunc1=1;//1
	hw->pref.b1=.01*hw->worldSize;
	hw->pref.b2=.15*hw->worldSize;
	hw->pref.tess=10;
	hw->pref.image=0;
	hw->pref.option=0;
	hw->pref.universe=1;
	hw->pref.spd1=1.5;
	hw->pref.n=0;
	//hw->pref.demosequence=1;

	for (int i=0; i<11; i++)    dispman.ons[i]=1;
	for (    i=0; i< 6; i++)           vars[i]=1;
	for (    i=0; i<10; i++) hw->pref.value[i]=1;
	hw->pref.value[6]=.6;
	hw->pref.value[7]=.8;
	hw->pref.value[8]=.9;
	hw->pref.value[9]=.5;
	hw->pref.value[10]=.5;
	hw->pref.value[11]=.5;

	hw->screen.lightmap.on=1;
	hw->screen.lightmap.type=4;
	hw->screen.lightmap.ampl=30;
	hw->screen.lightmap.size=70;
	hw->screen.lightmap.texmagx=1;
	hw->screen.lightmap.texmagy=3;

	hw->screen.viewport->camera.place.reset();
	hw->screen.viewport->camera.place.movemode=LOOKAT;
	dispman.selObject=0;
	if (init>0) {
		hw->pref.ambientvolume=.33;
		hw->pref.songvolume=.5;
		hw->pref.fullscreen=1;
		hw->pref.letterbox=.12;
		hw->pref.clock=0;
		hw->pref.randseed=hw->time.msTime;
	} else init=1;
}

//------------------------------------------------------------------------

void Effect::set_option(Text *ann) {
}

//-----------------------------------------------------------------------

void Effect::start_frame(Text *ann){
	Viewport *viewport=hw->screen.viewport;
	Place    *place=&viewport->camera.place;
	float lookdrag;
	//char s[80];

	if (hw->ex) return;

	// adjust ambient volumes to current spec
	hw->sound.lastChannel=hw->sound.samples[soundindex1].channel; hw->sound.set_volume(hw->pref.ambientvolume);
	hw->sound.lastChannel=hw->sound.samples[soundindex2].channel; hw->sound.set_volume(hw->pref.ambientvolume);
	hw->sound.lastChannel=hw->sound.samples[soundindex3].channel; hw->sound.set_volume(hw->pref.ambientvolume);
	hw->sound.lastChannel=hw->sound.samples[songindex  ].channel; hw->sound.set_volume(hw->pref.songvolume);

   if (hw->pref.demosequence==1) {
		if      ((runscript==1)&&(nscriptitems>0)) run_script();// sequence through scenes?
		else //if  (runscript==0)
			run_auto();// auto-sequence through scenes?
	}
	
	torus.size=40*hw->pref.value[0]; // -----------------tweaks --------------
	torus.thickness=torus.size*.25*hw->pref.value[1];
	torus.outermag=.5*hw->pref.value[2];

	//place->to=dispman.dos[3].place.pos; // lookat sphere

	//generalPH=airPH; // ----------------objects -------------------------------
	dispman.update(); // edits and orbits
	globe1.tessdist=length(dispman.dos[3].place.pos-place->pos); // sphere-camera delta
	globe2.tessdist=length(dispman.dos[4].place.pos-place->pos); // sphere-camera delta
	 torus.tessdist=length(dispman.dos[5].place.pos-place->pos); // ring-camera delta

	globe1.mindisttess=maxi(3,mini(80*hw->pref.tessf,120)); // close up hi detail
	globe2.mindisttess=maxi(3,mini(80*hw->pref.tessf,120));
	globe1.maxdisttess=maxi(3,mini(30*hw->pref.tessf, 60));   // faraway lo res
	globe2.maxdisttess=maxi(3,mini(30*hw->pref.tessf, 60));
	 torus.mindisttess=maxi(3,mini(130*hw->pref.tessf,200)); // close up
	 torus.maxdisttess=maxi(3,mini( 20*hw->pref.tessf, 40)); // faraway

	place->physics=&airPH;
	lookdrag=5;

	if (place->movemode==MANUAL) { // geom effects if manual cam
		if (groundM.inMesh(place->pos.x,place->pos.z)) test_mesh_zone(ann); // mesh zone effects
		if (place->pos.y < 250) { // adjust grav by height
			float ht;

			ht=hw->screen.viewport->camera.place.pos.y / 250;         //map camera y of 0->350 as 0 to 1
			ht=1-clampf(ht,0,1);                                    // clamp and invert 1->0
			if (physics==0) ht=0;                                   // overide grav
			airPH.gravity.set_xyz(0, -9.80665*hw->meter*ht, 0);// grav func(height)

			ht=hw->screen.viewport->camera.place.pos.y / 100;        //map camera y of 0->100 as 0 to 1
			ht=1-clampf(ht,0,1);                                   // clamp and invert 1->0
			lookdrag=ht*6+1;                                       // spindrag varies 1->7 as y 0->100
			place->physics->rotatedrag.set_xyz(lookdrag,lookdrag,lookdrag); // camera spindrag func(height) map 
		}
	}

	place->physics->rotatedrag.set_xyz(lookdrag,lookdrag,lookdrag);
	if ((hw->help==0)&&(hw->dialogs==0)) place->physics->damp(place);// camera damping
	place->lastPhysics=place->physics;
}

//------------------------------------------------------------------------

void Effect::test_mesh_zone(Text *ann) {
	Viewport *viewport=hw->screen.viewport;
	Place    *place=&viewport->camera.place;
	static float lastunsplashtime=0, lastlandtime=0;

	//   if manual cam              in main viewport              do camera/mesh physics
	if ((place->movemode==MANUAL)&&(viewport==&hw->screen.viewports[0]) ) {
		if (collide) {
			if ((groundM.collide(place, &groundPH))&&  // test mesh
				((hw->time.time-lastlandtime)>2)){  // hit & hard?
				lastlandtime=hw->time.time;
				hw->screen.viewport->camera.place.acc.x+=rndn(10)-5;
				hw->screen.viewport->camera.place.acc.z+=rndn(10)-5;
				hw->screen.viewport->camera.place.shakeampl=1; // shake camera
				hw->sound.play2d("sounds/land.wav");
				hw->sound.set_frequency(rndn(6000)+8000);
				hw->screen.set_dim(DIMOFF, .9, 1,0,0,.5); // fade full->0, over .9 sec, in transparent red
			}
		}
		if ((place->physics==&groundPH)&&(hw->pref.stepVolume>0)&&
			(hw->pause==0)&&(hw->dialogs==0)) track_steps();
	}
}

//---------------------------------------------------------------------------

void Effect::track_steps(void) {
	static float disttravelled=0, laststeptime=0;
	static int stepside=1;
	Viewport *viewport=hw->screen.viewport;
	Place    *place=&viewport->camera.place;

	disttravelled*=1-(.1*hw->time.dt);
	disttravelled+=place->vel.length()*hw->time.dt;
	if ((disttravelled>2)&&((hw->time.time-laststeptime)>.25)) { // step sound?// 1.5
		laststeptime=hw->time.time;
		disttravelled=0;
		stepside=-stepside; // toggle left/right
		hw->sound.play2d("sounds/step-dirt.wav");
		hw->sound.set_volume(hw->pref.stepVolume);
		hw->sound.set_pan(.5+hw->pref.stepPan*stepside);
		hw->sound.set_frequency(rndn(4000)+8000);
	}
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
// effects, each with a 1-time init -1, a start 0, a draw 1, and a close mode 2.
//
void Effect::effect0(int mode) { // ring
	static char s[280];
	Text ann;

	switch(mode) {
		case -1: // init effect
			strcpy(s,"ash nazg durbatulk, ash nazg gimbatul, ash nazg thrakatulk agh burzum-ishi krimpatul ");
			strcat(s,"ash nazg durbatulk, ash nazg gimbatul, ash nazg thrakatulk agh burzum-ishi krimpatul ");
			strcat(s,"ash nazg durbatulk, ash nazg gimbatul, ash nazg thrakatulk agh burzum-ishi krimpatul ");
			break;
		case 0: // start effect
			hw->screen.viewports[0].camera.place.moveRange. set_xyz(55,22,55); // camera
			hw->screen.viewports[0].camera.place.moveCenter.set_xyz(1,0,2);
			hw->screen.viewports[0].camera.place.to.set_xyz(0,0,0);
			dispman.dos[1].place.pos.set_xyz(0,-100,0); // clouds
			dispman.dos[2].place.pos.set_xyz(0, 100,0);
			dispman.dos[1].vis.set_color( 1, 1,.75);
			dispman.dos[2].vis.set_color(.75, 1, 1);
			hw->pref.b1=100;// fog
			hw->pref.b2=160;
			hw->pref.c2=rndn(.2)+.1; // fade
			hw->pref.blank=0;
			hw->screen.viewports[0].camera.fov=80;
			fog.set_color(0,0,0);
			hw->screen.set_color(0,0,0);
			//if (totalscenes==nscriptitems-1) dispman.dos[5].vis.ntexlayers=1;
			//else                             dispman.dos[5].vis.ntexlayers=2;
			break;
		case 1: // draw effect
			dispman.reset();
			//dispman.dos[5].place.orient.z+=11*hw->time.dt;
			//dispman.dos[5].place.orient.y+=.2*sinf(hw->time.time*.3);
			//if (dispman.ons[1]) dispman.dos[1].draw(1); // cloud
			//if (dispman.ons[2]) dispman.dos[2].draw(1); // cloud
			if(hw->useMultitexture) dispman.dos[5].vis.set_color(.95,.85,.3);
			else                    dispman.dos[5].vis.set_color(.75,.6,.225);
			if (dispman.ons[5]) dispman.dos[5].draw(1); // ring
			dispman.unset();
			
			glPushMatrix(); // ----------- draw ring inscription ------------
				Text::charInX=12; Text::charInY=8; Text::tex=&font2T;
				float sine=sin1(hw->time.time)*.25+.375;
				ann.set_color(sine,sine,0);//.25//.5 .5 0
				glEnable(GL_CULL_FACE);
				glEnable(GL_DEPTH_TEST);
				ann.print_sphere(s, // outer ring
					dispman.dos[5].place.pos.x,
					dispman.dos[5].place.pos.y,
					dispman.dos[5].place.pos.z,
					torus.size+torus.innerclip*torus.thickness,
					dispman.dos[5].place.orient.x,
					dispman.dos[5].place.orient.y,-1);
				glDisable(GL_DEPTH_TEST);

				ann.print_sphere(s, // inner ring
					dispman.dos[5].place.pos.x,
					dispman.dos[5].place.pos.y,
					dispman.dos[5].place.pos.z,
					torus.size+(torus.outerpoint+(1-torus.outerpoint)*torus.outermag)*torus.thickness,
					dispman.dos[5].place.orient.x,
					dispman.dos[5].place.orient.y,1);
				glDisable(GL_CULL_FACE);
				Text::charInX=13; Text::charInY=8; Text::tex=&fontT;
				glPopMatrix();
			break;
	}
}

//-----------------------------------------------------------------------

void Effect::effect1(int mode) { // hall of columns
	int i;
	static int seed=23;
	float light;

	switch(mode) {
		case -1:
			seed=rndn(9999);
			columns.init();
			break;
		case 0:
			srand(seed);
			groundM.init();
			for (i=0; i<50; i++) { // make mountains
				groundM.impress(rndn(.85)+ .1, rndn(.15)+.05, rndn(.15)+.1,rndn(25)); //s
				groundM.impress(rndn(.85)+ .1, rndn(.15)+.85, rndn(.15)+.1,rndn(25)); //n
				groundM.impress(rndn(.15)+.05, rndn(.85)+ .1, rndn(.15)+.1,rndn(25)); //w
				groundM.impress(rndn(.15)+.85, rndn(.85)+ .1, rndn(.15)+.1,rndn(25)); //e
			}
			hw->screen.viewports[0].camera.place.moveRange. set_xyz(300,47,300); // camera
			hw->screen.viewports[0].camera.place.moveCenter.set_xyz( 1,46, 2);
			hw->screen.viewports[0].camera.place.to.set_xyz(0,40,0);
			dispman.dos[1].place.pos.set_xyz(0,.5,0); // clouds
			dispman.dos[2].place.pos.set_xyz(0,99.5,0);
			dispman.dos[1].vis.set_color(0,.2,.2);
			dispman.dos[2].vis.set_color(.125,.083,.025);
			hw->pref.b1=100; // fog
			hw->pref.b2=1000;//+rndn(1000);
			hw->pref.c2=.5; // fade
			hw->pref.blank=0;
			//hw->screen.viewports[0].camera.fov=lin_interp(10,70,1-sqrt(rndn(1)));
			if (nite==0) {
				hw->screen.set_color(.2,.2,.1);
				fog.set_color(.25,.166,.05);
			} else {
				hw->screen.set_color(0,0,0);
				fog.set_color(0,0,0);
			}
			//hw->screen.lightmap.type=8;
			//hw->screen.lightmap.size=70;
			if (nite==1) hw->screen.lightmap.ampl=80;
			else         hw->screen.lightmap.ampl=50;
			hw->screen.lightmap.on=1;
			hw->screen.lightmap.texmagx=1;
			hw->screen.lightmap.texmagy=4;

			if (scriptindex==8) {
				hw->pref.viewportset=5;
				hw->screen.set_nviewports(hw->pref.viewportset);
				hw->pref.letterbox=0;
				hw->screen.letterbox=hw->pref.letterbox;
			}
			break;
		case 1:
			//if (scriptindex==9) {
			//	hw->pref.letterbox=0;
			//	hw->screen.letterbox=hw->pref.letterbox;
			//}
			//light=(sinf(hw->time.time*8)*
			//	    sinf(hw->time.time*13))*.5+.5;
			if (nite==1) {
				light=perlin.get1(1,hw->time.time*10)*.8+.2; // light flicker
				dispman.dos[9].material.r=1;
				dispman.dos[9].material.g=.8;
				dispman.dos[9].material.b=.6;
			} else {
				light=1;
				dispman.dos[9].material.r=1;
				dispman.dos[9].material.g=.75;
				dispman.dos[9].material.b=.5;
			}
			dispman.dos[9].material.diffuse=light;
			dispman.dos[0].material.diffuse=light;
			dispman.dos[7].material.diffuse=light;
			hw->screen.viewports[0].camera.fov=lin_interp(5,65,sin1(hw->time.time*.28));
			dispman.reset();
			if (dispman.ons[9])dispman.dos[9].draw(1); //columns
			if (dispman.ons[0]) dispman.dos[0].draw(1); // ground
			if (dispman.ons[7]) dispman.dos[7].draw(1); // ceiling
			if (dispman.ons[1]) dispman.dos[1].draw(1); // cloud1
			if (dispman.ons[2]) dispman.dos[2].draw(1); // cloud2
			dispman.unset();
			break;
		case 2:
			if (hw->pref.viewportset!=1) {
				hw->pref.viewportset=1;
				hw->screen.set_nviewports(hw->pref.viewportset);
			}
			nite=1-nite;
			hw->screen.lightmap.type=4;
			hw->screen.lightmap.on=1;
			hw->screen.lightmap.ampl=30;
			hw->screen.lightmap.texmagx=1;
			hw->screen.lightmap.texmagy=3;
			break;
	}
}

//-----------------------------------------------------------------------

void Effect::effect2(int mode) { // globe
	static int once=0;
	static float af=rndn(pi2);
	float f;

	switch(mode) {
		case -1:
			break;
		case 0:
			hw->screen.viewports[0].camera.place.moveRange. set_xyz(80,80,80); // camera
			hw->screen.viewports[0].camera.place.moveCenter.set_xyz(1,0,2);
			hw->screen.viewports[0].camera.place.to.set_xyz(0,0,0);
			dispman.dos[1].place.pos.set_xyz(0,-100,0); // clouds
			dispman.dos[2].place.pos.set_xyz(0, 100,0);
			if (once==0) {
				dispman.dos[1].vis.set_color(0,0,1);
				dispman.dos[2].vis.set_color(1,0,0);
				once=1;
			} else {
				switch(rndi(6)) {
					case 0: dispman.dos[1].vis.set_color(1,0,0); break;
					case 1: dispman.dos[1].vis.set_color(0,1,0); break;
					case 2: dispman.dos[1].vis.set_color(0,0,1); break;
					case 3: dispman.dos[1].vis.set_color(1,1,0); break;
					case 4: dispman.dos[1].vis.set_color(1,0,1); break;
					case 5: dispman.dos[1].vis.set_color(0,1,1); break;
				}
				switch(rndi(6)) {
					case 0: dispman.dos[2].vis.set_color(1,0,0); break;
					case 1: dispman.dos[2].vis.set_color(0,1,0); break;
					case 2: dispman.dos[2].vis.set_color(0,0,1); break;
					case 3: dispman.dos[2].vis.set_color(1,1,0); break;
					case 4: dispman.dos[2].vis.set_color(1,0,1); break;
					case 5: dispman.dos[2].vis.set_color(0,1,1); break;
				}
			}
			lightning.ballr0=dispman.dos[1].vis.r;
			lightning.ballg0=dispman.dos[1].vis.g;
			lightning.ballb0=dispman.dos[1].vis.b;
			lightning.ballr1=dispman.dos[2].vis.r;
			lightning.ballg1=dispman.dos[2].vis.g;
			lightning.ballb1=dispman.dos[2].vis.b;
			lightning.bottom=dispman.dos[1].place.pos.y;
			lightning.top   =dispman.dos[2].place.pos.y;
			hw->pref.b1=100; // fog
			hw->pref.b2=600;
			hw->pref.c2=.2; // fade
			hw->pref.blank=1;
			hw->screen.viewports[0].camera.fov=85;
			fog.set_color(0,0,0);
			hw->screen.set_color(0,0,0);
			break;
		case 1:
			globe1.size=40  *hw->pref.value[3];
			globe2.size=40.1*hw->pref.value[3];
			bb.sizex=32*hw->pref.value[3];
			bb.sizey=bb.sizex;

			//f=sinf(hw->time.time)*.125+.375;
			f=sin1(hw->time.time*.66+af)*.85+.05;
			dispman.dos[3].vis.alphafunc=f;

			dispman.reset();
			if (dispman.ons[1]) dispman.dos[1].draw(1); // cloud
			if (dispman.ons[2]) dispman.dos[2].draw(1); // cloud
			dispman.unset();

			lightning.update();
			lightning.draw();

			dispman.reset();
			if (dispman.ons[3]) dispman.dos[3].draw(1); // cloud sphere
			if (dispman.ons[6]) dispman.dos[6].draw(1); // bb
			if (dispman.ons[4]) dispman.dos[4].draw(1); // glass sphere
			dispman.unset();
			break;
		case 2:
			bb.sizex=32;
			bb.sizey=bb.sizex;
			break;
	}
}

//-----------------------------------------------------------------------

void Effect::effect3(int mode) { // underwater
	switch(mode) {
		case -1:
			break;
		case 0:
			hw->screen.viewports[0].camera.place.moveRange. set_xyz(45,20,45); // camera
			hw->screen.viewports[0].camera.place.moveCenter.set_xyz(1,-60,2);
			hw->screen.viewports[0].camera.place.to.set_xyz(0,-10,0);
			//dispman.dos[1].place.pos.set_xyz(0,0,0); // clouds
			//dispman.dos[1].vis.set_color(0,.5,.6);
			hw->pref.b1=100; // fog
			hw->pref.b2=260;
			hw->screen.viewports[0].camera.fov=90;
			fog.set_color(0,0,0);
			hw->screen.set_color(0,0,.15);
			hw->screen.viewports[0].camera.place.moverate=rndn(1)+1; 
			hw->screen.viewports[0].camera.place.spinrate=rndn(1)+1; 
			break;
		case 1:
			hw->pref.c2=.2; // fade
			hw->pref.blank=0;
			dispman.reset();
			if (dispman.ons[8]) dispman.dos[8].draw(1);//rays
			dispman.unset();
			break;
		case 2: 
			hw->screen.viewports[0].camera.place.moverate=1;
			hw->screen.viewports[0].camera.place.spinrate=1; 
			break;
	}
}

//-----------------------------------------------------------------------

void Effect::effect4(int mode) { // volcano
	float lavaheight=135;
	static int seed=23, lavatexgen=0;
	Geom geom;
	geom.q=1;

	switch(mode) {
		case -1:
			//seed=rndn(hw->time.msTime);
			seed=hw->pref.randseed;
			break;
		case 0:
			srand(seed);
			volcanoM.init();
			volcanoM.impress(.5,  .5,  .8,  300); // cone
			volcanoM.impress(.52, .52, .4, -230); // dent
			volcanoM.noise(12); // +- rndn(10)
			hw->screen.viewports[0].camera.place.moveRange. set_xyz(120,  0,120); // camera 140,30,140
			hw->screen.viewports[0].camera.place.moveCenter.set_xyz(  1,210,  2);
			hw->screen.viewports[0].camera.place.to.set_xyz(.0275*volcanoM.sizex,lavaheight-20,
				                                              -.03*volcanoM.sizey);
			dispman.dos[1].place.pos.set_xyz(0,90,0); // clouds
			dispman.dos[2].place.pos.set_xyz(0,275,0);
			dispman.dos[1].vis.set_color(.1,.1,.1);
			dispman.dos[2].vis.set_color(.3,.2,.1);
			lightning.ballr0=dispman.dos[1].vis.r;
			lightning.ballg0=dispman.dos[1].vis.g;
			lightning.ballb0=dispman.dos[1].vis.b;
			lightning.ballr1=dispman.dos[2].vis.r;
			lightning.ballg1=dispman.dos[2].vis.g;
			lightning.ballb1=dispman.dos[2].vis.b;
			lightning.bottom=dispman.dos[1].place.pos.y;
			lightning.top   =dispman.dos[2].place.pos.y;

			hw->pref.b1=100; // fog
			hw->pref.b2=1400;
			hw->pref.c2=.5; // fade
			hw->pref.blank=1;
			hw->screen.viewports[0].camera.fov=lin_interp(60,80,rndn(1));
			fog.set_color(0,0,0);
			hw->screen.set_color(.125,.125,0);
			if (lavatexgen==0) hw->pref.option=5;
			else               hw->pref.option=0;
			lavatexgen=1-lavatexgen;
			break;
		case 1:
			lightning.update();
			lightning.draw();

			if (lightning.flashOn) dispman.dos[10].material.ambient=1;
			else                   dispman.dos[10].material.ambient=.5;

			dispman.reset();
			if (dispman.ons[10]) dispman.dos[10].draw(1); // ground
			dispman.unset();

			zColor3f(1,1,1);  // lava pool
			glEnable(GL_DEPTH_TEST);
			glEnable(GL_TEXTURE_2D);
			lavaT.bind();

			if (hw->pref.option!=5) hw->pref.paren=sin1(hw->time.time*.1)*.3+.8;//.4
			else                    hw->pref.paren=sin1(hw->time.time*.1)*1+1;
			glMatrixMode(GL_TEXTURE);
			glPushMatrix();
				//glTranslatef(hw->time.time*.05*hw->pref.page,
				//	          hw->time.time*.05*hw->pref.page, 0);
				//glScalef(3*hw->pref.home,3*hw->pref.home,1);
				glTranslatef(hw->time.time*.05,
					          hw->time.time*.05, 0);
				glScalef(3,3,1);
				glMatrixMode(GL_MODELVIEW);
				glPushMatrix(); // lava
					glTranslatef(.0275*volcanoM.sizex,lavaheight,
						           -.03*volcanoM.sizey);
					glRotatef(-90,1,0,0); // tilt since disk drawn down z
					geom.disk(130,130, 30,30);
					glPopMatrix();

				glMatrixMode(GL_TEXTURE);
				glPopMatrix();
			glMatrixMode(GL_MODELVIEW);

			glDisable(GL_DEPTH_TEST);
			glDisable(GL_TEXTURE_2D);

			p1P.blendFunc0=6;
			p1P.blendFunc1=1;
			p1P.r0= 1;			p1P.g0=.4;			p1P.b0=.1;
			p1P.r1= 0;			p1P.g1= 0;			p1P.b1= 0;
			p1P.size1=hw->meter*lin_interp(120,180,sin1(hw->time.time));
			p1P.windx=  3*hw->meter;
			p1P.windy=  9*hw->meter;
			p1P.windz=  0*hw->meter;
			p1P.dispersex=4*hw->meter;
			p1P.dispersey=4*hw->meter;
			p1P.dispersez=4*hw->meter;
			p1P.rollspeed=30;
			p1P.interval= .2;

			p1P.create(rnd_gaussian(27)+.0275*volcanoM.sizex, lavaheight+1,
				        rnd_gaussian(27)-.0275*volcanoM.sizey);
			p1P.update();
			p1P.draw();

			dispman.reset();
			if (dispman.ons[ 1]) dispman.dos[1].draw(1); // cloud1
			if (dispman.ons[ 2]) dispman.dos[2].draw(1); // cloud2
			dispman.unset();
			break;
	}
}

//-----------------------------------------------------------------------

void Effect::effect5(int mode) { // falls
	float waterheight=0;
	static int i, seed=23, watertexgen=0;
	static int side=2;
	float cyl[7]={0,.7,.4,.9,.5,.1,.6}; // rnd cylinder texy
	Geom geom;
	geom.q=0;//1
	static Material mat;

	switch(mode) {
		case -1:
			//seed=rndn(hw->time.msTime);
			seed=hw->pref.randseed;
			break;
		case 0:
			mat.on=1;
			mat.ambient=.25;
			mat.diffuse=.7;
			mat.specular=.99;
			mat.reflective=.99;
			mat.shininess=10;//qqq
			mat.exponent=1;
			mat.cutoff=180;
			mat.r=1;
			mat.g=.75;
			mat.b=.25;

			srand(seed);
			fallM.texgenmode=1;
			fallM.init();
			for (i=0; i<50; i++) { // make mountains
				fallM.impress(rndn(.85)+ .1, rndn(.15)+.05, rndn(.15)+.1,rndn(6)); //s 4
				fallM.impress(rndn(.85)+ .1, rndn(.15)+.85, rndn(.15)+.1,rndn(6)); //n
				fallM.impress(rndn(.15)+.05, rndn(.85)+ .1, rndn(.15)+.1,rndn(6)); //w
				fallM.impress(rndn(.15)+.85, rndn(.85)+ .1, rndn(.15)+.1,rndn(6)); //e
			}
			fallM.noise(1.5); // +- rndn(1)
			fallM.foldx(.5);
			p2P.kill();
			//fallM.box(.1,.5,.5,.9, waterheight);
			//fallM.slope(-.2,-.2,0,0); // +- rndn(10)

			//fallM.impress(.5,  .5,  .8,  300); // cone
			//fallM.impress(.9, .5, .5, -200); // dent

			hw->screen.viewports[0].camera.place.moveRange. set_xyz(8,12,12); // camera 140,30,140
			hw->screen.viewports[0].camera.place.moveCenter.set_xyz(12, -1, 1);
			hw->screen.viewports[0].camera.place.to.set_xyz(0,waterheight-5,0);
			dispman.dos[1].place.pos.set_xyz(0, 20,0); // clouds
			dispman.dos[2].place.pos.set_xyz(0,-80,0);
			dispman.dos[1].vis.set_color(.9,.9,.5);
			dispman.dos[2].vis.set_color(.5,.5,.5);

			lightning.ballr0=dispman.dos[1].vis.r;
			lightning.ballg0=dispman.dos[1].vis.g;
			lightning.ballb0=dispman.dos[1].vis.b;
			lightning.ballr1=dispman.dos[2].vis.r;
			lightning.ballg1=dispman.dos[2].vis.g;
			lightning.ballb1=dispman.dos[2].vis.b;
			lightning.bottom=dispman.dos[1].place.pos.y;
			lightning.top   =dispman.dos[2].place.pos.y;

			hw->pref.b1=10; // fog
			hw->pref.b2=140;
			hw->pref.c2=.5; // fade
			hw->pref.blank=1;
			hw->pref.letterbox=hw->screen.letterbox=0;
			hw->screen.viewports[0].camera.fov=70;//lin_interp(60,80,rndn(1));
			fog.set_color(0,0,0);
			hw->screen.set_color(0,0,0);
			if (watertexgen==0) hw->pref.option=5;
			else                hw->pref.option=0;
			watertexgen=1-watertexgen;
			hw->screen.lightmap.type=8;
			hw->screen.lightmap.size=70;
			hw->screen.lightmap.ampl=30;
			hw->screen.lightmap.on=1;
			hw->screen.lightmap.texmagx=1;
			hw->screen.lightmap.texmagy=1;

			if (hw->pref.demosequence==1) {
				camscript.play=1;
				//camscript.itemn=rndn(camscript.nitems);
				//camscript.pos=0;
			}
			break;
		case 2:
			camscript.play=0;
			hw->screen.lightmap.type=4;
			hw->screen.lightmap.on=1;
			hw->screen.lightmap.ampl=30;
			hw->screen.lightmap.texmagx=1;
			hw->screen.lightmap.texmagy=3;
			break;
		case 1:
			p2P.blendFunc0=hw->pref.blendFunc0;
			p2P.blendFunc1=hw->pref.blendFunc1;
			p2P.r0=1;			p2P.g0=1;			p2P.b0=1;
			p2P.r1= 0;			p2P.g1= 0;			p2P.b1= 0;
			p2P.size1=hw->meter*lin_interp(1,2,sin1(hw->time.time));
			p2P.sizey=1.5; /////////////////
			p2P.windx=    0*hw->meter;
			p2P.windy=-5.25*hw->meter;
			p2P.windz=    0*hw->meter;
			p2P.dispersex=.5*hw->meter;
			p2P.dispersey=.5*hw->meter;
			p2P.dispersez=.5*hw->meter;
			p2P.rollspeed=80;
			p2P.interval=0;

			p2P.xi=2.5*hw->meter;
			p2P.yi=0;
			p2P.zi=0;//rndnn(3)*.2;
			for (int i=0; i<(250./hw->time.fps); i++) {
				p2P.create( -.17, waterheight-1+rnd_gaussian(.05),//.1
					                             rnd_gaussian(.1)+side);//.125
				side=-side;
			}
			lightning.update();
			lightning.draw();

			dispman.reset();
			if (dispman.ons[11]) dispman.dos[11].draw(1); // falls
			if (dispman.ons[ 1]) dispman.dos[1].draw(1); // cloud2
			//if (dispman.ons[ 2]) dispman.dos[2].draw(1); // cloud2
			dispman.unset();

			//if (lightning.flashOn==1) mat.ambient=.4;
			//else                      mat.ambient=.2;

			mat.set();////////////////
			zColor3f(hw->pref.value[9],hw->pref.value[10],hw->pref.value[11]);//1,.9,.8);//.8,.6);
			glEnable(GL_DEPTH_TEST);
			glEnable(GL_FOG);
			glEnable(GL_LIGHTING);
			glEnable(GL_TEXTURE_2D);
			imageT[11].bind();

			geom.texmagx=.1; // object linear mapped
			geom.texmagy=.1;
			glPushMatrix(); //floor
				glTranslatef(-5.5,0,0);
				geom.texbasex=0;
				geom.texbasey=0;
				geom.slab(10,1,17,8,4,8);

				glTranslatef(0,.5,0);
				geom.texbasex=.3;
				geom.texbasey=.7;
				geom.slab(9,1,16,8,4,8);
				glPopMatrix();

			geom.texmagx=1;
			geom.texmagy=1;

			for (i=0; i<7; i++) {
				glPushMatrix();
					glTranslatef(-sin(i*(pi1/6.))*7-2,1,
						          -cos(i*(pi1/6.))*7);
					switch(i) {
						default: glRotatef(-90, 1,0,0); // standing column
							break;
						case 1:
							glTranslatef(0,.3,0); // fallen inside
							glRotatef( 10, 0,1,0);
							break;
						case 5:
							glTranslatef(0,.65,0); // fallen outside
							glRotatef( 15, 1,0,0);
							glRotatef(-30, 0,1,0);
							break;
					}
					glRotatef(45, 0,0,1);
					geom.disk(.5,.5,3,4); // base cap
					geom.cylinder(.5,.5, .5,.5, .1, 0, .375, 4,4); // base
					glTranslatef(0,0,.1);
					geom.disk(.5,.5,3,4); // top of base
					glRotatef(-45, 0,0,1);

					geom.cylinder(.3,.3, .3,.3, 6, cyl[i],  1,   4,16); // main column

					glRotatef(45, 0,0,1);
					glTranslatef(0,0,6);
					geom.disk(.5,.5,3,4); // bottom of crown
					geom.cylinder(.5,.5, .5,.5, .1, 0, .0125, 4,4); // crown
					glTranslatef(0,0,.1);
					geom.disk(.5,.5,3,4); // top
					glRotatef(-45, 0,0,1);
					glPopMatrix();
			}

			geom.texmagx=.1; // object linear mapped
			geom.texmagy=.1;
			glPushMatrix(); // roof
				glTranslatef(-5.5,7.325,0);
				geom.slab(8,.25,15,8,4,8);
				glPopMatrix();

			glDisable(GL_DEPTH_TEST);
			glDisable(GL_FOG);
			glDisable(GL_LIGHTING);
			glDisable(GL_TEXTURE_2D);

			p2P.update();
			p2P.draw();
			break;
	}
}

//-----------------------------------------------------------------------

void Effect::effect6(int mode) { // eye
	static int once=0;

	switch(mode) {
		case -1:
			break;
		case 0:
			hw->screen.viewports[0].camera.place.moveRange. set_xyz(30,30,30); // camera
			hw->screen.viewports[0].camera.place.moveCenter.set_xyz(2,1,32);
			hw->screen.viewports[0].camera.place.to.set_xyz(0,0,0);
			hw->screen.viewports[0].camera.place.moverate=1.5;
			dispman.dos[1].place.pos.set_xyz(0,-50,0); // clouds
			dispman.dos[2].place.pos.set_xyz(0, 50,0);
			if (once==0) {
				dispman.dos[1].vis.set_color(1,0,0);
				dispman.dos[2].vis.set_color(1,0,0);
				once=1;
			} else {
				switch(rndi(6)) {
					case 0: dispman.dos[1].vis.set_color(1,0,0); break;
					case 1: dispman.dos[1].vis.set_color(0,1,0); break;
					case 2: dispman.dos[1].vis.set_color(0,0,1); break;
					case 3: dispman.dos[1].vis.set_color(1,1,0); break;
					case 4: dispman.dos[1].vis.set_color(1,0,1); break;
					case 5: dispman.dos[1].vis.set_color(0,1,1); break;
				}
				switch(rndi(6)) {
					case 0: dispman.dos[2].vis.set_color(1,0,0); break;
					case 1: dispman.dos[2].vis.set_color(0,1,0); break;
					case 2: dispman.dos[2].vis.set_color(0,0,1); break;
					case 3: dispman.dos[2].vis.set_color(1,1,0); break;
					case 4: dispman.dos[2].vis.set_color(1,0,1); break;
					case 5: dispman.dos[2].vis.set_color(0,1,1); break;
				}
			}
			lightning.ballr0=dispman.dos[1].vis.r;
			lightning.ballg0=dispman.dos[1].vis.g;
			lightning.ballb0=dispman.dos[1].vis.b;
			lightning.ballr1=dispman.dos[2].vis.r;
			lightning.ballg1=dispman.dos[2].vis.g;
			lightning.ballb1=dispman.dos[2].vis.b;
			lightning.bottom=dispman.dos[1].place.pos.y;
			lightning.top   =dispman.dos[2].place.pos.y;
			hw->pref.b1=100; // fog
			hw->pref.b2=600;
			hw->pref.c2=.2; // fade
			hw->pref.blank=1;
			hw->screen.viewports[0].camera.fov=60;
			fog.set_color(0,0,0);
			hw->screen.set_color(0,0,0);
			break;
		case 1:
			dispman.reset();
			if (dispman.ons[1]) dispman.dos[1].draw(1); // cloud
			if (dispman.ons[2]) dispman.dos[2].draw(1); // cloud
			dispman.unset();

			lightning.update();
			lightning.draw();

			dispman.reset();
			if (dispman.ons[6]) dispman.dos[6].draw(1); // bb
			dispman.unset();
			break;
		case 2:
			hw->screen.viewports[0].camera.place.moverate=1;
			break;

	}
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

Rays::Rays(){
	nrays=200;
	for (int i=0; i<nrays; i++) uray[i].age=rndn(1);
};
Rays::~Rays(){};

//----------------------------------------------------------------------------

void Rays::draw(float size){
	int i;
	float width=1.6*Hw::get()->pref.value[4], depth=85, pitch, heading, alpha;
	Vector ray;
	ray.set_xyz(-1500,900,0); //////// never finished code!
	ray=ray-Hw::get()->screen.viewport->camera.place.pos;//////////////
	heading=xy2r(ray.x,ray.z);///////////////
	pitch  =xy2r(ray.y,sqrt(pow(ray.x,2)+pow(ray.z,2)));/////////////
	ray.set_xyz(0,0,0); //should be interp////////////////

	for (i=0; i<nrays; i++) {
		float agef;
		uray[i].age+=Hw::get()->time.dt; // update rays
		if (uray[i].age>=.6) { // rebirth?
			uray[i].age=rndn(.3);
			uray[i].x=ray.x+rnd_gaussian(22*Hw::get()->pref.value[5]);
			uray[i].y=ray.y;
			uray[i].z=ray.z+rnd_gaussian(22*Hw::get()->pref.value[5]);
			uray[i].xi=rndn(4)-2;
			uray[i].yi=0;
			uray[i].zi=rndn(4)-2;
			uray[i].pitch  =pitch;
			uray[i].heading=heading;
			uray[i].roll=rndn(360);
			uray[i].pitchi  =rndn(24)-12;
			uray[i].headingi=rndn(24)-12;
			uray[i].rolli=rndn(24)-12;
			uray[i].depth=(rndn(.4)+.8)*depth;
		} else {
			uray[i].x+=uray[i].xi*Hw::get()->time.dt;
			uray[i].y+=uray[i].yi*Hw::get()->time.dt;
			uray[i].z+=uray[i].zi*Hw::get()->time.dt;
			uray[i].pitch  +=uray[i].pitchi  *Hw::get()->time.dt;
			uray[i].heading+=uray[i].headingi*Hw::get()->time.dt;
			uray[i].roll   +=uray[i].rolli   *Hw::get()->time.dt;
		}
		agef=uray[i].age / .5;
		alpha=1-fabs(agef*2-1);
		
		glPushMatrix();// draw
			glRotatef(uray[i].heading,0,0,1);
			glRotatef(uray[i].pitch,  1,0,0);
			glRotatef(uray[i].roll,0,1,0);
			glBegin(GL_QUADS);
				zColor4f(Hw::get()->pref.value[6],Hw::get()->pref.value[7]*1.1,Hw::get()->pref.value[8]*.9,alpha);
				glTexCoord2f(0,0); glVertex3f(uray[i].x-width,uray[i].y,uray[i].z);
				glTexCoord2f(1,0); glVertex3f(uray[i].x+width,uray[i].y,uray[i].z);
				//glColor4f(0,.5,.6,alpha);
				zColor4f(Hw::get()->pref.value[6],Hw::get()->pref.value[7]*.9,Hw::get()->pref.value[8]*1.1,alpha);
				glTexCoord2f(1,1); glVertex3f(uray[i].x+width*2,uray[i].y-uray[i].depth,uray[i].z);
				glTexCoord2f(0,1); glVertex3f(uray[i].x-width*2,uray[i].y-uray[i].depth,uray[i].z);
				glEnd();
			glPopMatrix();
	}
	Hw::get()->ntris+=nrays*2;
}

//----------------------------------------------------------------------------

int Rays::collide(Place *place, Physics *physicson){return 0;};

//----------------------------------------------------------------------------

void Rays::init(void){};

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

Columns::Columns(){
};
Columns::~Columns(){};

//----------------------------------------------------------------------------

void Columns::draw(float size){
	int ix,iy;

	for    (ix=0; ix<13; ix++) { //12 280
		for (iy=0; iy<13; iy++) {
		glPushMatrix();
			glTranslatef(ix*66-400,0, // place column
							 iy*66-400);
			dl.draw();
			glPopMatrix();
		}
	}
	Hw::get()->ntris+=13*13*8*14*2;
}

//----------------------------------------------------------------------------

int Columns::collide(Place *place, Physics *physicson){return 0;};

//----------------------------------------------------------------------------

void Columns::init(void){
	int nrings=1, nslices=8,cyl;
	float width=18, height=100, texy=0;
	Geom geom;

	geom.texmagy=10;
	struct cyl{
		int type;
		float height, width, diskwidth, disknormal, nslices;
	} data[]={ // cylinder/disk profiles / top-down
		-1,   1,.9,    0, 0, 8,
		 1, .93,.4,    0, 0, 8,
		 1, .85,.2,    0, 0, 8,
		 0, .85,.4,   .4, 1, 8,
		 1, .74,.3,    0, 0, 8,
		 0, .74,.18,  .3,-1, 8,
		 1, .56,.18,   0, 0, 8,
		 1, .38,.18,   0, 0, 8,
		 1,  .2,.18,   0, 0, 8,
		 0,  .2,.21,   .21, 1, 8,
		 1,  .15,.21,   0, 0, 8,
		 0,  .15,.24,.24, 1, 8,
		 1,  .1,.24,   0, 0, 8,
		 0,  .1,.3,   .3, 1, 8,
		 1,   0,.4,    0, 0, 8,
	};
	int ncyl=14;

	glPushMatrix();
		dl.create(COMPILE);
		dl.open();
		glRotatef(-90,1,0,0); // tilt since generic columns drawn down z
		//glRotatef(30,0,0,1); // square

		glEnable(GL_CULL_FACE); //         cylinders
		for (cyl=0; cyl<ncyl; cyl++) {
			nslices=data[ncyl-cyl].nslices;
			float cylheight=data[ncyl-(cyl+1)].height-
								 data[ncyl- cyl   ].height;

			if (data[ncyl-cyl].type==1) {
				geom.cylinder(width*data[ncyl-cyl].width,
								  width*data[ncyl-cyl].width,
								  width*data[ncyl-(cyl+1)].width,
								  width*data[ncyl-(cyl+1)].width,
								  height*cylheight,
								  texy,
								  cylheight,
								  nrings,nslices);
			}
			glTranslatef(0,0,height*cylheight);
			texy+=cylheight;
		}
		glDisable(GL_CULL_FACE);
		glTranslatef(0,0,-height);
		for (cyl=0; cyl<ncyl; cyl++) {  //      disks
			nslices=data[ncyl-cyl].nslices;
			float cylheight=data[ncyl-(cyl+1)].height-
								 data[ncyl-cyl  ].height;

			if (data[ncyl-cyl].type==0) {
				geom.normal.z =   data[ncyl-cyl].disknormal;
				geom.disk(width * data[ncyl-cyl].diskwidth,
								width*data[ncyl-cyl].diskwidth,
								nrings,nslices);
			}
			glTranslatef(0,0,height*cylheight);
		}
		dl.close();
		glPopMatrix();
};

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

int Effect::load_script(const char* infile) {
	char s[200], *str;//, logstr[80];

	nscriptitems=0;
	scriptindex=0;
	strcpy(s,infile);
	strcat(s,".script.txt"); // build filename to open
	ifstream file(s,ios::nocreate);
	if (file.bad()||(file==0)) {
		log_message("************** cant open script file ***************");
		return 1;
	}
	log_message("Loading scene script...");
	file.getline(s,180); // skip 1st comment line
	while (file.eof()==0) { // more to read?
		file.getline(s,180); // read line
		if ((s[0]==0)||(s[0]=='/')||(strlen(s)<5)) continue; // skip?
		scriptitems[nscriptitems].time=atof(strtok(s,   ", ")); // 1st field
		str=                                strtok(NULL,", \n");  // string field
		if      (strcmp(str,"ring")   ==0) scriptitems[nscriptitems].effectn=0;
		else if (strcmp(str,"columns")==0) scriptitems[nscriptitems].effectn=1;
		else if (strcmp(str,"sphere") ==0) scriptitems[nscriptitems].effectn=2;
		else if (strcmp(str,"rays")   ==0) scriptitems[nscriptitems].effectn=3;
		else if (strcmp(str,"volcano")==0) scriptitems[nscriptitems].effectn=4;
		else if (strcmp(str,"falls")  ==0) scriptitems[nscriptitems].effectn=5;
		else if (strcmp(str,"eye")    ==0) scriptitems[nscriptitems].effectn=6;
		nscriptitems++;
	}
	file.close();
	effectn=scriptitems[0].effectn;
	proc_effect(0);
	return 0;
}

//-------------------------------------------------------------------

void Effect::check_script(void) {
	//char s[80];

	newScene=0;
	effectf=0;
	if ((nscriptitems>0)&&(runscript)) {  // script?
		effectf= (hw->time.time-lastscenetime)   // how far into scene
			/ (scriptitems[scriptindex].time-lastscenetime); // duration, 0->1
		if (effectf>=1) { // switch time?
			proc_effect(2);  // shutdown current scene
			effectf=0;
			newScene=1;
			totalscenes++;
			lastscenetime=hw->time.time;
			scriptindex++;   // next item
			if (scriptindex==nscriptitems) { // end of script?
				runscript=0;
				effectn=1; // set scene
			} else effectn=scriptitems[scriptindex].effectn; // set scene
			proc_effect(0);  // init scene
		} 
	}
}

//-------------------------------------------------------------------

void Effect::init_script(void) {
	effectf=0;
	scriptindex=0;
	nscriptitems=0;
	runscript=1;
	newScene=0;
	lastscenetime=0;
	for (int i=0; i<MAXSCRIPTITEMS; i++) {
		scriptitems[i].time=0;
		scriptitems[i].effectn=0;
	}
}

//-------------------------------------------------------------------

void Effect::draw_eye(void) { // ortho
	if ((effectn!=2)&&(effectn!=6)) return;
	static float lasttime=0;
	int sizeh=eyeT.sizex*.5, i;
	float cy=hw->screen.viewport->camera.place.orient.y;
	float cx=hw->screen.viewport->camera.place.orient.x;
	float x,y, r, dt=hw->time.dt, interval;
	float speed;//(hw->time.fps/100.); 14

	for (i=0; i<800./hw->time.fps; i++) {
		//x=rnd_gaussian(3)+sizeh; if (x==sizeh) x=rndi(2)*2-1;
		//y=rnd_gaussian(9)+sizeh; if (y==sizeh) y=rndi(2)*2-1;
		r=rndn(pi2);
		x=sin(r)*14+sizeh;
		y=cos(r)*80+sizeh;
		p3P.create(x,y,0);
	}

	if (effectn==2) interval=.08;
	else            interval=.06;
	speed=.1*hw->pref.page;
	if ((hw->time.time-lasttime)<interval) return;
	lasttime=hw->time.time;

	if (hw->pref.blank==0) copy_image(0,0, &tmpT);

	hw->screen.viewport->camera.place.orient.y=0;
	hw->screen.viewport->camera.place.orient.x=0;

	p3P.size0=10*(sin1(hw->time.time*1.5)*.45+.8);
	p3P.size1=p3P.size0;

	for (i=0; i<p3P.maxparticles; i++) {
		if (p3P.p[i].on==0) continue;
		p3P.p[i].xi=(p3P.p[i].x-sizeh)*15.8 *speed;
		p3P.p[i].yi=(p3P.p[i].y-sizeh)*3.17*speed;
	}
	hw->time.dt=interval;
	p3P.update();
	hw->time.dt=dt;
	for (i=0; i<p3P.maxparticles; i++) {
		if (p3P.p[i].on==0) continue;
		if      ((p3P.p[i].x<0)||(p3P.p[i].x>eyeT.sizex)) p3P.p[i].on=0;
		else if ((p3P.p[i].y<0)||(p3P.p[i].y>eyeT.sizey)) p3P.p[i].on=0;
	}
	glColor4f(0,0,0,0);
	glRectf(0,0,256,256);
	p3P.draw();
	eyeT.bind();
	glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 0,0, eyeT.sizex,eyeT.sizey, 0);
	hw->screen.viewport->camera.place.orient.y=cy;
	hw->screen.viewport->camera.place.orient.x=cx;//////
	if (hw->pref.blank==0) paste_image(0,0, 1, &tmpT);
}

//-------------------------------------------------------------------

void Effect::run_script(void) {
	static int setdim=0, imagen=-1;

	check_script(); // check sync, set effectf //qqqqqqq
	if ( (effectf>(1-(hw->shutdownduration / 
		(scriptitems[scriptindex].time-lastscenetime)))) && (setdim==0) ) { // inter effect dim
		if ((demoend) && (totalscenes>=(nscriptitems-1))) { // time to end? (done all effects?)
			//hw->sound.stop_sound();
			hw->ex=hw->time.time;
			hw->screen.set_dim(DIMON, hw->shutdownduration*1.1, 0,0,0,1); // fade 0->full, over n sec, in black
			hw->screen.capture=1; hw->pref.blank=0; hw->pref.c2=.2;
			hw->pref.user=1;
			hw->ann->hold("Keep the fellowship...");
		}
		else {
			hw->screen.capture=1;//////////////////
			//if (scriptindex==11) hw->screen.set_dim(DIMA, hw->shutdownduration*2, 1,1,1,1); // "A" shape over _ sec in white (rays)
			//else
			hw->screen.set_dim(DIMA, hw->shutdownduration*2, 0,0,0,1); // "A" shape over _ sec in black
		}
		setdim=1;
	}
	if (newScene) {// (effectf<.5)&&(setdim==1)) { // just turned 0/started new effect?
		doc_scene();
		hw->screen.capture=0;//////////
		if (effectn==0) { // ready for new ring reflection?
			imagen=(imagen+1+hw->pref.image)%4;
			dispman.dos[5].texattribs[0].image=8+imagen; // reflection image /////////////////
		}
		if      (effectn==0)     hw->pref.letterbox=.05;//+rndnn(1)*.10; ring
		else if (effectn==5)     hw->pref.letterbox=0;  // falls
		else if (scriptindex==8) hw->pref.letterbox=0;  // multiviewport columns
		else                     hw->pref.letterbox=.05+rndnn(1)*.20;// random letterbox
		hw->screen.letterbox=hw->pref.letterbox;
		setdim=0;
	}
}

//-------------------------------------------------------------------------

void Effect::run_auto(void) {
	static int setdim=0, imagen=-1;
	int effecti;

	effecti=((int)(hw->time.time / hw->pref.sceneduration))%neffects; // int 0->neffects-1, chg effect every 10 seconds
	effectf=wrap(  hw->time.time / hw->pref.sceneduration,0,1);       // float 0->1 over duration
	if (effectn!=effecti) { // change scene, f just wrapped around
		proc_effect(2); // close old
		effectn=effecti;
		proc_effect(0); // open new
	}
	if ( (effectf> (1-(hw->shutdownduration / hw->pref.sceneduration) ) ) &&
		  (setdim==0) ) { // inter effect dim
		hw->screen.capture=1;///////////////
		hw->screen.set_dim(DIMA, hw->shutdownduration*2, 0,0,0,1); // "A" shape over _ sec in black
		setdim=1;
	}
	if ((effectf<.5)&&(setdim==1)) { // just turned 0/started new effect?
		doc_scene();
		hw->screen.capture=0;
		if (effectn==0) { // ready for new ring reflection?
			imagen=(imagen+1+hw->pref.image)%4;
			dispman.dos[5].texattribs[0].image=8+imagen; // reflection image /////////////////
		}

		if      (effectn==0)     hw->pref.letterbox=.05;//+rndnn(1)*.10; ring
		else if (effectn==5)     hw->pref.letterbox=0;  // falls
		else                     hw->pref.letterbox=.05+rndnn(1)*.20;// random letterbox

		hw->screen.letterbox=hw->pref.letterbox;
		setdim=0;
	}
}

//-----------------------------------------------------------------------------

void Effect::doc_scene(void) {
	switch(effectn) {
		case 0: hw->ann->hold("The Ring of the Lords");break;
		case 1: hw->ann->hold("Moria/Khazad-dum");break;
		case 2: hw->ann->hold("The Sphere of Saruman - The Palantiri");break;
		case 3: hw->ann->hold("Sam Gamgee-cam");break;
		case 4: hw->ann->hold("Mount Doom volcano, Mordor");break;
		case 5: hw->ann->hold("Rivendell");break;
		case 6: hw->ann->hold("The Eye of Sauron");break;
	}
}

//-----------------------------------------------------------------------------

void Effect::copy_image(int x, int y, Texture *texture) {
	texture->bind();
	glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, x,y, texture->sizex,texture->sizey, 0);
}

//-----------------------------------------------------------------------------

void Effect::paste_image(int x, int y, float scale, Texture *texture) {
	glColor3f(1,1,1);
	glEnable(GL_TEXTURE_2D);
	texture->bind();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0); glVertex2f(x                     ,y);
		glTexCoord2f(1,0); glVertex2f(x+texture->sizex*scale,y);
		glTexCoord2f(1,1); glVertex2f(x+texture->sizex*scale,y+texture->sizey*scale);
		glTexCoord2f(0,1); glVertex2f(x                     ,y+texture->sizey*scale);
		glEnd();
	glDisable(GL_TEXTURE_2D);
}

//-----------------------------------------------------------------------------


