//
#include "geom.h"
#include "hardware.h"

//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------

Geom::Geom() {
	normals=1;
	texmagx=1;
	texmagy=1;
	texbasex=0;
	texbasey=0;
	normal.set_xyz(0,0,1);
	q=0;
};
Geom::~Geom() {};

//-----------------------------------------------------------------------------------

void Geom::disk(float widthx, float widthy, int nrings, int nslices, int slice0, int slice1) {
	int i,ai, ring; // draws a disk, of rings and slices, in x/y plane, clockwise from 12 oclock
	float x,y, angle, angleinc=pi2 / nslices, sinangle, cosangle;
	float r0,r1,slicef;

	if (slice1==-1) slice1=nslices;
	glNormal3f(normal.x,normal.y,normal.z);
	for (ring=0; ring<nrings; ring++) {
		r0=((float) ring   / nrings);
		r1=((float)(ring+1) / nrings);
		if (r0==0) { // fan
			glBegin(GL_TRIANGLE_FAN);
				if (q==1) glTexCoord4f(.5*texmagx, .5*texmagy,0,xy2q(.5,.5));
				else      glTexCoord2f(.5*texmagx, .5*texmagy);
				glVertex2f(0,0);
				for (i=slice0; i<=slice1; i++) {
					ai=(i==nslices)?0:i; // avoid holes
					angle=ai*angleinc;
					sinangle=sinf(angle);
					cosangle=cosf(angle);
					x=sinangle;
					y=cosangle;
					if (q==1) glTexCoord4f(x*r1*.5*texmagx+.5,
						                    y*r1*.5*texmagy+.5, 0, xy2q(x*r1+.5,y*r1+.5));
					else      glTexCoord2f(x*r1*.5*texmagx+.5,
						                    y*r1*.5*texmagy+.5);
					glVertex2f(x*widthx*r1,
								  y*widthy*r1);
				}
				glEnd();
		} else {
			glBegin(GL_TRIANGLE_STRIP);
				for (i=slice0; i<=slice1; i++) {
					slicef=(float)i / nslices;
					ai=(i==nslices)?0:i; // avoid holes
					angle=ai*angleinc;
					sinangle=sinf(angle);
					cosangle=cosf(angle);
					x=sinangle;
					y=cosangle;
					if (q==1) glTexCoord4f(x*r0*.5*texmagx+.5,
						                    y*r0*.5*texmagy+.5, 0, xy2q(x*r0+.5,y*r0+.5));
					else      glTexCoord2f(x*r0*.5*texmagx+.5,
						                    y*r0*.5*texmagy+.5);
					glVertex2f(x*widthx*r0,y*widthy*r0);
					if (q==1) glTexCoord4f(x*r1*.5*texmagx+.5,
						                    y*r1*.5*texmagy+.5, 0, xy2q(x*r1+.5,y*r1+.5));
					else      glTexCoord2f(x*r1*.5*texmagx+.5,
						                    y*r1*.5*texmagy+.5);
					glVertex2f(x*widthx*r1,y*widthy*r1);
				}
				glEnd();
		}
	}
	Hw::get()->ntris+=nslices*nrings*2;
}

//-------------------------------------------------------------------------

float xy2q(float x, float y) {
	Hw *hw=Hw::get();
	//return (sinf(x*30*hw->pref.paren)+
	//	     sinf(y*30*hw->pref.paren) )*.04*hw->pref.home+hw->pref.page;
	switch(hw->pref.option) {
	case 0: default: 
	return (sinf(x*30*hw->pref.paren)+
		     sinf(y*30*hw->pref.paren) )*.04*hw->pref.bracket+1;
		break;
	case 1: 
	return (sinf(x*30*hw->pref.paren)+
		     cosf(y*30*hw->pref.paren) )*.04*hw->pref.bracket+1;
		break;
	case 2: 
	return (sinf(x*30*hw->pref.paren)*
		     sinf(y*30*hw->pref.paren) )*.04*hw->pref.bracket+1;
		break;
	case 3: 
	return (sinf(x*30*hw->pref.paren)*
		     cosf(y*30*hw->pref.paren) )*.04*hw->pref.bracket+1;
		break;
	case 4: 
	return (sinf((x+y)*30*hw->pref.paren))*.04*hw->pref.bracket+1;
		break;
	case 5: 
	return (sinf(((x-.5)*(y-.5))*30*hw->pref.paren))*.04*hw->pref.bracket+1;
		break;
	}
}

//-------------------------------------------------------------------------

void Geom::cylinder(float widthx0, float widthy0, float widthx1, float widthy1,
						  float length, float texy, float texlen,
						  int nstacks, int nslices, int slice0, int slice1) {
	int i,ai, stack; // draws a cylinder, of slices on x/y plane, clockwise from 12, and in stacks on +z;
	float x,y, angle, angleinc=pi2 / nslices, sinangle, cosangle;
	float slicef, stackf0, stackf1, widthx, widthy;

	if (slice1==-1) slice1=nslices;
	for (stack=0; stack<nstacks; stack++) {
		stackf0=(float) stack    / nstacks;
		stackf1=(float)(stack+1) / nstacks;
		glBegin(GL_TRIANGLE_STRIP);
			for (i=slice0; i<=slice1; i++) {
				slicef=(float)i / nslices;
				ai=(i==nslices)?0:i; // avoid holes
				angle=ai*angleinc;
				sinangle=sinf(angle);
				cosangle=cosf(angle);
				x=sinangle;
				y=cosangle;
				if (normals) glNormal3f(x,y,0);
				widthx=lin_interp(widthx0,widthx1, stackf0);
				widthy=lin_interp(widthy0,widthy1, stackf0);
				glTexCoord2f(                            slicef  *texmagx+texbasex,
					          lin_interp(texy,texy+texlen,stackf0)*texmagy+texbasey);
				glVertex3f(x*widthx,y*widthy,stackf0*length);

				widthx=lin_interp(widthx0,widthx1, stackf1);
				widthy=lin_interp(widthy0,widthy1, stackf1);
				glTexCoord2f(                            slicef  *texmagx+texbasex,
					          lin_interp(texy,texy+texlen,stackf1)*texmagy+texbasey);
				glVertex3f(x*widthx,y*widthy,stackf1*length);
			}
			glEnd();
	}
	Hw::get()->ntris+=nslices*nstacks*2;
}

//-------------------------------------------------------------

void Geom::pill(float widthx, float widthy, float height, int nrings=5, int nslices=10, int flag=7) {
	if (flag&1) disk(widthx, widthy, nrings, nslices);
	if (flag&2) cylinder(widthx, widthy, widthx, widthy, height,0,height, nrings, nslices);
	if (flag&4) {
		glTranslatef(0,0,height);
		disk(widthx, widthy, nrings, nslices);
		glTranslatef(0,0,-height);
	}
}

//-------------------------------------------------------------
/*
void Geom::slab(float sizex, float sizey, float sizez, int nx, int ny, int nz) {
	int i,j;
	float fx=1./nx, 
		   fy=1./ny, 
			fz=1./nz;
	float sizexh=.5*sizex, 
		   sizeyh=.5*sizey, 
			sizezh=.5*sizez,k;

	for (k=-sizezh; k<=sizezh; k+=sizez) {
		for    (i=0; i<ny; i++) { // front/back  l/r?
			for (j=0; j<nx; j++) {
				glBegin(GL_QUADS);
					if (k<0) glNormal3f(0,0,-1);
					else     glNormal3f(0,0, 1);
					glTexCoord2f( j   *fx*texmagx, i   *fy*texmagy); glVertex3f( (j   *fx)*sizex-sizexh, ( i   *fy)*sizey-sizeyh, k);
					glTexCoord2f((j+1)*fx*texmagx, i   *fy*texmagy); glVertex3f(((j+1)*fx)*sizex-sizexh, ( i   *fy)*sizey-sizeyh, k);
					glTexCoord2f((j+1)*fx*texmagx,(i+1)*fy*texmagy); glVertex3f(((j+1)*fx)*sizex-sizexh, ((i+1)*fy)*sizey-sizeyh, k);
					glTexCoord2f( j   *fx*texmagx,(i+1)*fy*texmagy); glVertex3f( (j   *fx)*sizex-sizexh, ((i+1)*fy)*sizey-sizeyh, k);
					glEnd();
			}
		}
	}
	for (k=-sizeyh; k<=sizeyh; k+=sizey) {
		for    (i=0; i<nz; i++) { // top/bot
			for (j=0; j<nx; j++) {
				glBegin(GL_QUADS);
					if (k<0) glNormal3f(0,-1,0);
					else     glNormal3f(0, 1,0);
					glTexCoord2f( j   *fx*texmagx, i   *fz*texmagz); glVertex3f( (j   *fx)*sizex-sizexh, k, ( i   *fz)*sizez-sizezh);
					glTexCoord2f((j+1)*fx*texmagx, i   *fz*texmagz); glVertex3f(((j+1)*fx)*sizex-sizexh, k, ( i   *fz)*sizez-sizezh);
					glTexCoord2f((j+1)*fx*texmagx,(i+1)*fz*texmagz); glVertex3f(((j+1)*fx)*sizex-sizexh, k, ((i+1)*fz)*sizez-sizezh);
					glTexCoord2f( j   *fx*texmagx,(i+1)*fz*texmagz); glVertex3f( (j   *fx)*sizex-sizexh, k, ((i+1)*fz)*sizez-sizezh);
					glEnd();
			}
		}
	}
	for (k=-sizexh; k<=sizexh; k+=sizex) {
		for    (i=0; i<ny; i++) { // left/right
			for (j=0; j<nz; j++) {
				glBegin(GL_QUADS);
					if (k<0) glNormal3f(-1,0,0);
					else     glNormal3f( 1,0,0);
					glTexCoord2f( j   *fx*texmagx, i   *fz*texmagz); glVertex3f(k, (i   *fy)*sizey-sizeyh,  ( j   *fz)*sizez-sizezh);
					glTexCoord2f((j+1)*fx*texmagx, i   *fz*texmagz); glVertex3f(k,((i+1)*fy)*sizey-sizeyh,  ( j   *fz)*sizez-sizezh);
					glTexCoord2f((j+1)*fx*texmagx,(i+1)*fz*texmagz); glVertex3f(k,((i+1)*fy)*sizey-sizeyh,  ((j+1)*fz)*sizez-sizezh);
					glTexCoord2f( j   *fx*texmagx,(i+1)*fz*texmagz); glVertex3f(k, (i   *fy)*sizey-sizeyh,  ((j+1)*fz)*sizez-sizezh);
					glEnd();
			}
		}
	}
	Hw::get()->ntris+=nx*ny*2*2+nx*nz*2*2+ny*nz*2*2;
}
*/
//-------------------------------------------------------------

void Geom::slab(float sizex, float sizey, float sizez, int nx, int ny, int nz) {
	int i,j;
	float fx=1./nx, //bricksize
		   fy=1./ny, 
			fz=1./nz, x,y,z;
	float sizexh=.5*sizex, 
		   sizeyh=.5*sizey, 
			sizezh=.5*sizez, k;

	for (k=-sizexh; k<=sizexh; k+=sizex) {
		if (k<0) glNormal3f(-1,0,0);
		else     glNormal3f( 1,0,0);
		for    (j=0; j<ny; j++) { // left/right
			for (i=0; i<nz; i++) {
				glBegin(GL_QUADS);
					x=k;
					y= (j   *fy)*sizey-sizeyh;
					z= (i   *fz)*sizez-sizezh;	glTexCoord2f(z*texmagx+texbasex,y*texmagy+texbasey); glVertex3f(x,y,z);
					z=((i+1)*fz)*sizez-sizezh;	glTexCoord2f(z*texmagx+texbasex,y*texmagy+texbasey); glVertex3f(x,y,z);
					y=((j+1)*fy)*sizey-sizeyh; glTexCoord2f(z*texmagx+texbasex,y*texmagy+texbasey); glVertex3f(x,y,z);
					z= (i   *fz)*sizez-sizezh; glTexCoord2f(z*texmagx+texbasex,y*texmagy+texbasey); glVertex3f(x,y,z);
					glEnd();
			}
		}
	}
	for (k=-sizezh; k<=sizezh; k+=sizez) {
		if (k<0) glNormal3f(0,0,-1);
		else     glNormal3f(0,0, 1);
		for    (j=0; j<ny; j++) { // back/front
			for (i=0; i<nx; i++) {
				glBegin(GL_QUADS);
					z=k;
					y= (j   *fy)*sizey-sizeyh;
					x= (i   *fx)*sizex-sizexh;	glTexCoord2f(x*texmagx+texbasex,y*texmagy+texbasey); glVertex3f(x,y,z);
					x=((i+1)*fx)*sizex-sizexh;	glTexCoord2f(x*texmagx+texbasex,y*texmagy+texbasey); glVertex3f(x,y,z);
					y=((j+1)*fy)*sizey-sizeyh; glTexCoord2f(x*texmagx+texbasex,y*texmagy+texbasey); glVertex3f(x,y,z);
					x= (i   *fx)*sizex-sizexh; glTexCoord2f(x*texmagx+texbasex,y*texmagy+texbasey); glVertex3f(x,y,z);
					glEnd();
			}
		}
	}
	for (k=-sizeyh; k<=sizeyh; k+=sizey) {
		if (k<0) glNormal3f(0,-1,0);
		else     glNormal3f(0, 1,0);
		for    (j=0; j<nz; j++) { // bot/top
			for (i=0; i<nx; i++) {
				glBegin(GL_QUADS);
					y=k;
					z= (j   *fz)*sizez-sizezh;
					x= (i   *fx)*sizex-sizexh;	glTexCoord2f(x*texmagx+texbasex,z*texmagy+texbasey); glVertex3f(x,y,z);
					x=((i+1)*fx)*sizex-sizexh;	glTexCoord2f(x*texmagx+texbasex,z*texmagy+texbasey); glVertex3f(x,y,z);
					z=((j+1)*fz)*sizez-sizezh; glTexCoord2f(x*texmagx+texbasex,z*texmagy+texbasey); glVertex3f(x,y,z);
					x= (i   *fx)*sizex-sizexh; glTexCoord2f(x*texmagx+texbasex,z*texmagy+texbasey); glVertex3f(x,y,z);
					glEnd();
			}
		}
	}
	Hw::get()->ntris+=nx*ny*2*2+nx*nz*2*2+ny*nz*2*2;
}

//-------------------------------------------------------------


