//---------------------------------------------------------------------------------
//	File:		lightning.c
//	Comments:   by George Ziniewicz
//---------------------------------------------------------------------------------

#include "lightning.h"

//---------------------------------------------------------------------------------

Lightning::Lightning() {
	n=40;
	lastFlashTime=5;
	nextFlashTime=10;
	flashDuration=2;//1.5;
	flashInterval=2;
	play=0;
	flashOn=0;
	nextSound=10;
	width=.075;
	ballDuration=.3;
	ballsize=100;
	ballr0=0;
	ballg0=0;
	ballb0=1;
	ballr1=0;
	ballg1=0;
	ballb1=1;
	x=y=z=xi=yi=0;
	zi=3;
	bottom=0;
	top=100;
	crag=10;
	arcT=0;
	sphereT=0;
};

Lightning::~Lightning() {};

//---------------------------------------------------------------------------------

void Lightning::init(void) {
	log_message("Init Lightning...");

	thunderS=Hw::get()->sound.load_wav("sounds/ethunder.wav");
	zi=5*Hw::get()->meter;
}

//------------------------------------------------------------------------------

void Lightning::deinit(void) {
	Hw::get()->sound.free_sample(thunderS);
}

//------------------------------------------------------------------------------

void Lightning::new_strike(void) {
	int i, tilt=(rndn(.25)-.125)*(top-bottom);
		
	x=(rndn(1)-.5)*Hw::get()->worldSize*.5;
	z=(rndn(1)-.5)*Hw::get()->worldSize*.5;
	n=rndn(10)+10;
	flashDuration=rndn(1)+.15;//1

	for (i=0; i<n; i++) {
		xs[i]=rndn(crag)+((float)i/n)*tilt;
		ys[i]=rndn(crag)+((float)i/n)*(top-bottom)+bottom;
		zs[i]=rndn(crag)+((float)i/n)*tilt;
	}
	ys[0  ]=bottom;
	ys[n-1]=top;
	flashOn=1;
	lastFlashTime=Hw::get()->time.time;
}

//------------------------------------------------------------------------------

void Lightning::draw(void) {
	Hw *hw=Hw::get();
	int i;
	float texoffy=rndn(1), age,agef,agef1, size;

	if (flashOn) {
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(GL_GREATER,0);
		glEnable(GL_BLEND);
		set_blend(1,1);
		glEnable(GL_DEPTH_TEST);
		glDepthMask(GL_FALSE);
		glEnable(GL_TEXTURE_2D);
		glPushMatrix();
			glTranslatef(x,y,z);
			if ((hw->time.time-lastFlashTime)<ballDuration) { // still on?
				age=hw->time.time-lastFlashTime; // balls
				agef=age/ballDuration;
				agef1=1-agef;
				size=ballsize*agef1;

				sphereT->bind();

				glPushMatrix(); // bot ball
					glColor3f(lin_interp(1,ballr0,agef)*agef1,
						       lin_interp(1,ballg0,agef)*agef1,
							    lin_interp(1,ballb0,agef)*agef1);
					glTranslatef(xs[0],ys[0],zs[0]);
					billboard(size,size);
					glPopMatrix();

				glPushMatrix(); // top ball
					glColor3f(lin_interp(1,ballr1,agef)*agef1,
						       lin_interp(1,ballg1,agef)*agef1,
							    lin_interp(1,ballb1,agef)*agef1);
					glTranslatef(xs[n-1],ys[n-1],zs[n-1]);
					billboard(size,size);
					glPopMatrix();

				hw->ntris+=4;
			}

			if (rndn(1)<.5) { // arc
				set_blend(1,1);
				glColor3f(.9,.9,1);
				arcT->bind();
				glBegin(GL_QUADS);
				for (i=0; i<n-1; i++) {
					glTexCoord2f(0,  texoffy); glVertex3f(xs[i],        ys[i],  zs[i]);
					glTexCoord2f(1,  texoffy); glVertex3f(xs[i]  +width,ys[i],  zs[i]);
					glTexCoord2f(1,1+texoffy); glVertex3f(xs[i+1]+width,ys[i+1],zs[i+1]);
					glTexCoord2f(0,1+texoffy); glVertex3f(xs[i+1],      ys[i+1],zs[i+1]);
				}
				glEnd();
				hw->ntris+=n*2;
			}
		glPopMatrix();
	glDisable(GL_ALPHA_TEST);
	glDisable(GL_BLEND);
	glDisable(GL_DEPTH_TEST);
	glDepthMask(GL_TRUE);
	glDisable(GL_TEXTURE_2D);
	}
}

//------------------------------------------------------------------------------

void Lightning::update(void) {
	Hw *hw=Hw::get();
	if (flashOn==0) {
		if (hw->time.time>nextFlashTime) { // new flash?
			new_strike();
			play=1;
			distance=fabs(hw->screen.viewport->camera.place.pos.z-z);
			nextSound=hw->time.time+.3;//distance*.001;
		}
	} else { // flash is on
		if (hw->pause==0) z+=zi*hw->time.dt; // crawl
		if ((hw->time.time-lastFlashTime)>flashDuration) { // turn off?
			nextFlashTime=hw->time.time+flashInterval+rndn(flashInterval);
			flashOn=0;
		}
	}
	if (play==1) {
		if (hw->time.time>nextSound) {
			play=0;
			hw->sound.play(thunderS,x,y,z);
			//hw->sound->set_volume(1);//-.5*distance);
			hw->sound.set_frequency(12000+rndn(8000));
		}
	}
	//if ((rndn(1)<.1*hw->time.dt)&&(hw->time.>10)) {// intermittent rumbling
	//	hw->sound.play(thunderS,rndn(2)-1,rndn(2)-1,rndn(2)-1);
	//	hw->sound.set_frequency((rndn(4)+3)*1000);
	//}
}

//--------------------------------------------------------------------------

void Lightning::billboard(float sizex, float sizey) {
	Camera *camera=&Hw::get()->screen.viewport->camera;
	float rightxs,rightys,rightzs, upxs,upys,upzs;

	rightxs=camera->rightx*sizex;
	rightys=camera->righty*sizex;
	rightzs=camera->rightz*sizex;
	upxs=camera->upx*sizey;
	upys=camera->upy*sizey;
	upzs=camera->upz*sizey;
	glBegin(GL_QUADS);
		glTexCoord2f(0,0); glVertex3f(-rightxs-upxs,-rightys-upys,-rightzs-upzs);
		glTexCoord2f(1,0); glVertex3f( rightxs-upxs, rightys-upys, rightzs-upzs);
		glTexCoord2f(1,1); glVertex3f( rightxs+upxs, rightys+upys, rightzs+upzs);
		glTexCoord2f(0,1); glVertex3f(-rightxs+upxs,-rightys+upys,-rightzs+upzs);
		glEnd();
}
