//-------------------------------------------------------------------
//	File:		   particle.c
//	Comments:   by George Ziniewicz
//-------------------------------------------------------------------

#include "particle.h"
#include "main.h"
#include "base.h"
#include "hardware.h"

//--------------------------------------------------------------------------------------

Part::Part() {};
Part::~Part() {};

//--------------------------------------------------------------------------------------

Particle::Particle() {
	maxparticles=100;	lifeTime=1;	interval=.1; lifeTimeRnd=0;
	size0=1; r0=1; g0=1; b0=1; alpha0=1; colorRnd=0;
	size1=1; r1=1; g1=1; b1=1; alpha1=1; sizeRnd=0; length=0;
	dispersex=0; windx=0; sizex=1; xi=0;
	dispersey=0; windy=0; sizey=1; yi=0;
	dispersez=0; windz=0; sizez=1; zi=0; rollspeed=0;
	alphatest=1; alphafunc=0; depthtest=1; depthwrite=0;
	blend=1; blendFunc0=2; blendFunc1=1;
	tx0=0; ty0=0;
	tx1=1; ty1=1; tex=0;	kill(); init=0;
};

Particle::~Particle() {};

//--------------------------------------------------------------------------------------

int Particle::create(float x, float y, float z) {
	static int current=0;
	int i, i2, found=-1;
	float r,g,b;

	if ((interval)&&(Hw::get()->time.time<(lastTime+interval))) return found; // too early for another one

	for (i=current; i<maxparticles+current; i++) {// scan from a circular buffer for slot
		i2=i%maxparticles;
		if (p[i2].on==0) {
			found=i2;
			current=(i2+1)%maxparticles;
			break;
		}
	}
	if (found<0) return found; // full buffer?
	i=found;
	lastTime=Hw::get()->time.time;
	p[i].on=1;
	p[i].set_xyz(x,y,z);
	//p[i].lastx=x;	//p[i].lasty=y;	//p[i].lastz=z;
	p[i].xi=(rndn(2)-1)*dispersex+xi;
	p[i].yi=(rndn(2)-1)*dispersey+yi;
	p[i].zi=(rndn(2)-1)*dispersez+zi;
	p[i].pt=0;	p[i].pti=0;
	p[i].hd=0;	p[i].hdi=0;
	p[i].roll=(rollspeed)?rndn(360):0;
	p[i].rolli=(rndn(2)-1)*rollspeed;

	p[i].size0=size0+(rndn(2)-1)*sizeRnd;
	p[i].size1=size1+(rndn(2)-1)*sizeRnd;
	r=r0+(rndn(1)-.5)*colorRnd; p[i].r0=(r<0)?0:(r>1)?1:r;
	g=g0+(rndn(1)-.5)*colorRnd; p[i].g0=(g<0)?0:(g>1)?1:g;
	b=b0+(rndn(1)-.5)*colorRnd; p[i].b0=(b<0)?0:(b>1)?1:b;
	r=r1+(rndn(1)-.5)*colorRnd; p[i].r1=(r<0)?0:(r>1)?1:r;
	g=g1+(rndn(1)-.5)*colorRnd; p[i].g1=(g<0)?0:(g>1)?1:g;
	b=b1+(rndn(1)-.5)*colorRnd; p[i].b1=(b<0)?0:(b>1)?1:b;
	p[i].set_color(p[i].r0,p[i].g0,p[i].b0,alpha0);

	p[i].birthTime=Hw::get()->time.time;
	p[i].lifeTime=lifeTime+(rndn(1)-.5)*lifeTimeRnd*lifeTime;
	p[i].deathTime=Hw::get()->time.time+p[i].lifeTime;
	p[i].slickness=1;
	return found;
}

//-----------------------------------------------------------------------

void Particle::update(void) {
	int i;
	float inc=Hw::get()->time.dt, time=Hw::get()->time.time, agef=0;

	for (i=0; i<maxparticles; i++) {
		if (p[i].on==0) continue; // alive?
		if (p[i].lifeTime>0) { // living?
			if (time>=p[i].deathTime) {p[i].on=0; continue;} // died?
			p[i].age=time-p[i].birthTime;
			agef=p[i].age/p[i].lifeTime; // normalized age
		}
		p[i].x   +=p[i].xi   *inc;
		p[i].y   +=p[i].yi   *inc;
		p[i].z   +=p[i].zi   *inc;
		p[i].pt  +=p[i].pti  *inc;
		p[i].hd  +=p[i].hdi  *inc;
		p[i].roll+=p[i].rolli*inc;

		p[i].xi+=(windx*inc)*p[i].slickness; // forces and friction
		p[i].yi+=(windy*inc)*p[i].slickness;
		p[i].zi+=(windz*inc)*p[i].slickness;

		if (p[i].lifeTime) {
			float size=lin_interp(p[i].size0,p[i].size1,agef); // interpolations
			p[i].sizex=size*sizex; // shape
			p[i].sizey=size*sizey; 
			p[i].sizez=size*sizez;
			p[i].set_color(lin_interp(p[i].r0,p[i].r1,agef),
				            lin_interp(p[i].g0,p[i].g1,agef),
								lin_interp(p[i].b0,p[i].b1,agef));
		}
		p[i].a=lin_interp(alpha0,alpha1,agef);
	}
	return;
}

//---------------------------------------------------------------------

void Particle::draw(void) {
	Hw *hw=Hw::get();
	Camera &camera=hw->screen.viewport->camera;

	int i;
	float hd,pt,rx,ry,rz,ux,uy,uz;
	set();
	if (tex) { // textured quad
		if (rollspeed==0) {
			rx=camera.rightx;
			ry=camera.righty;
			rz=camera.rightz;
			ux=camera.upx;
			uy=camera.upy;
			uz=camera.upz;
		} else {
			hd=camera.place.orient.y;
			pt=camera.place.orient.x;
		}
		for (i=0; i<maxparticles; i++) {
			float rightxs,rightys,rightzs, upxs,upys,upzs, psizex,psizey,psizez;

			if (!p[i].on) continue;
			hw->ntris+=2;
			psizex=p[i].sizex;
			psizey=p[i].sizey;
			psizez=p[i].sizez;
			glColor4f(p[i].r,p[i].g,p[i].b,p[i].a);

			if (rollspeed==0) { // billboardable?
				rightxs=rx*psizex;
				rightys=ry*psizey;
				rightzs=rz*psizez;
				upxs=ux*psizex;
				upys=uy*psizey;
				upzs=uz*psizez;
				glPushMatrix();
					glTranslatef(p[i].x,p[i].y,p[i].z);
					glBegin(GL_QUADS);
						glTexCoord2f(tx0,ty0); glVertex3f(-rightxs-upxs,-rightys-upys,-rightzs-upzs);
						glTexCoord2f(tx1,ty0); glVertex3f( rightxs-upxs, rightys-upys, rightzs-upzs);
						glTexCoord2f(tx1,ty1); glVertex3f( rightxs+upxs, rightys+upys, rightzs+upzs);
						glTexCoord2f(tx0,ty1); glVertex3f(-rightxs+upxs,-rightys+upys,-rightzs+upzs);
						glEnd();
					glPopMatrix();
			} else {
				glPushMatrix();
					glTranslatef(p[i].x,p[i].y,p[i].z);
					glRotatef(hd-p[i].hd,   0,1,0);
					glRotatef(pt-p[i].pt,   1,0,0);
					glRotatef( 0-p[i].roll, 0,0,1);
					glBegin(GL_QUADS);
						glTexCoord2f(tx0,ty0); glVertex2f(-psizex,-psizey);
						glTexCoord2f(tx1,ty0); glVertex2f( psizex,-psizey);
						glTexCoord2f(tx1,ty1); glVertex2f( psizex, psizey);
						glTexCoord2f(tx0,ty1); glVertex2f(-psizex, psizey);
						glEnd();
					glPopMatrix();
			}
		}
	}
	else { // glpoint
		for (i=0; i<maxparticles; i++) {
			if (!p[i].on) continue;
			hw->ntris++;
			glColor4f(p[i].r,p[i].g,p[i].b,p[i].a);
			if (length==0) {
				glPointSize(p[i].size);
				glBegin(GL_POINTS);
					glVertex3f(p[i].x,p[i].y,p[i].z);
					glEnd();
			} else {
				glLineWidth(p[i].size);
				glBegin(GL_LINES);
					float len;

					len=sqrt(p[i].xi*p[i].xi+
							   p[i].yi*p[i].yi+
							   p[i].zi*p[i].zi);
					if (len) len=length / len;
					else len=.0001;
					glVertex3f(p[i].x,p[i].y,p[i].z);
					glVertex3f(p[i].x+p[i].xi*len,
							     p[i].y+p[i].yi*len,
							     p[i].z+p[i].zi*len);
					glEnd();
			}
		}
	}
	unset();
	glLineWidth(2);
	return;
}

//---------------------------------------------------------------------

void Particle::kill(void) {
	for (int i=0; i<maxparticles; i++) p[i].on=0;
	init=0;
	return;
}

//---------------------------------------------------------------------

int Particle::count(void) {
	int i, n=0;
	for (i=0; i<maxparticles; i++) if (p[i].on) n++;
	return n;
}

//---------------------------------------------------------------------

int Particle::collision(float x, float y, float z, float distance) { // cubic test
	int i, n=0;

	for (i=0; i<maxparticles; i++) {
		if (p[i].on==0) continue;
		if ( (fabs(p[i].x-x)<distance)&&
			  (fabs(p[i].y-y)<distance)&&
			  (fabs(p[i].z-z)<distance) ) {p[i].on=0; n++;}
	}
	return n;
}

//----------------------------------------------------------------------

void Particle::set(void) {
	if (alphatest) {
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(GL_GREATER,alphafunc);
	}
	if (blend) {
		set_blend(blendFunc0,blendFunc1);
		glEnable(GL_BLEND);
	}
	glColor4f(r,g,b,a);
	if (depthtest)   glEnable(GL_DEPTH_TEST);
	if (!depthwrite) glDepthMask(GL_FALSE);
	if ((tex)&&(Hw::get()->pref.textures)) {
		glEnable(GL_TEXTURE_2D);
		tex->bind();
	}
}

//-------------------------------------------------------------------

void Particle::unset(void) {
	if (alphatest)   glDisable(GL_ALPHA_TEST);
	if (blend)       glDisable(GL_BLEND);
	if (depthtest)   glDisable(GL_DEPTH_TEST);
	if (!depthwrite) glDepthMask(GL_TRUE);
	if ((tex)&&(Hw::get()->pref.textures)) glDisable(GL_TEXTURE_2D);
}

//-------------------------------------------------------------------

