//
#include "place.h"
#include "physics.h"
#include "hardware.h"

//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------

Physics::Physics() {
	gravity.   set_xyz(0,0,0);
	movedrag.  set_xyz(0,0,0);
	rotatedrag.set_xyz(0,0,0);
	friction.  set_xyz(0,0,0);
	spdLimit=1000;
	gravityScaler=1;
	dragScaler=1;
};
Physics::~Physics(){};

//----------------------------------------------------------------------

void Physics::limit_spd(Place *place){
	if (place->vel.length()>spdLimit) {
		 place->vel*=spdLimit / place->vel.length();
	}
}

//----------------------------------------------------------------------

void Physics::damp(Place *place){
	float spd=Hw::get()->time.dt*Hw::get()->pref.dragScaler;
	place->vel.x  *= 1-(  movedrag.x*spd);
	place->vel.y  *= 1-(  movedrag.y*spd);
	place->vel.z  *= 1-(  movedrag.z*spd);
	place->spin.y *= 1-(rotatedrag.x*spd);
	place->spin.x *= 1-(rotatedrag.y*spd);
}

//----------------------------------------------------------------------

void Physics::apply_gravity(Place *place){
	//place->acc+=gravity*Hw::get()->time.dt*Hw::get()->pref.gravityScaler;
	//if (Hw::get()->dialogs==0) 
		place->acc+=gravity*Hw::get()->pref.gravityScaler;
}

//----------------------------------------------------------------------

int Physics::ltFriction(Vector *v){
	if ((fabs(v->x)<friction.x)&&
		 (fabs(v->y)<friction.y)&&
		 (fabs(v->z)<friction.z)) return 1;
	else return 0;
}

//----------------------------------------------------------------------

void Physics::bounce(Place* place1, Place *place2){
	Vector contactVelV, contactPosV;
	float impactFactor;

	contactVelV=place2->vel-
		         place1->vel;
	contactPosV=place2->pos-
		         place1->pos;
	contactPosV.normalize();
	impactFactor=contactPosV.dot_product(contactVelV);

	contactPosV*=(impactFactor / (place1->mass*
		                           place2->mass));

	place1->vel += contactPosV*2*(place2->mass / place1->mass);
	place2->vel -= contactPosV*2*(place1->mass / place2->mass);
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------

