//       position/orientation controller w/
//          5 modes: hold
//                   manual: mouseLook
//                           camera-oriented arrowKey/Click thrust in x/z
//                           <Control>+arrows switches thrust axis to x/y
//                            <Shift> +arrows increase thrust 10x
//                           Double-Click mouse -> 160 x spurt forward
//                   random: adj "orbit" origin, range, speed
//                           adj spin origin, range, speed
//                   lookAtObject: random orbit position as above
//                                 orient is "to" location
//                   lookFromObject: position taken from another controller
//                                   orientation via manual control


#include "place.h"
#include "hardware.h"
#include "physics.h"
#include "perlin.h"

//------------------------------------------------------------------------

Place::Place() {
	moveRange.set_xyz(10,10,10);
	moveCenter.set_xyz(0,0,0);
	spinRange. set_xyz(20,720,0);
	spinCenter.set_xyz(0,0,0);
	to.set_xyz(0,0,0);
	moverate=1;
	spinrate=1;
	mass=1;
	physics=lastPhysics=0;
	otherPlace=0;
	reset();
	shakefreqx=40;
	shakefreqy=50;
	shakefreqz=60;
	wraporient=0;
};
Place::~Place() {};

//-------------------------------------------------------------------

void Place::set(void) {
	glTranslatef(pos.x,pos.y,pos.z);         // move to object
	if (orient.y) glRotatef(orient.y,0,1,0); // orient object
	if (orient.x) glRotatef(orient.x,1,0,0);
	if (orient.z) glRotatef(orient.z,0,0,1);
}

//-------------------------------------------------------------------

void Place::reset(void) {
	pos.set_xyz(0,0,0);
	lastpos=pos;
	vel.set_xyz(0,0,0);
	acc.set_xyz(0,0,0);
	momentum=0;
	orient.set_xyz(0,0,0);
	spin.set_xyz(0,0,0);
	shake.set_xyz(0,0,0);
	to.set_xyz(0,0,0);
	movemode=MANUAL;
}

//----------------------------------------------------------------

void Place::move(void) {
	//vel+=acc*.5*pow(Hw::get()->time.dt,2);
	vel+=acc*Hw::get()->time.dt;
	pos+=vel*Hw::get()->time.dt;
	acc.set_xyz(0,0,0);
	//momentum=vel.length()*mass;
}

//-------------------------------------------------------------------

void Place::rotate(void) {
	float speed=Hw::get()->time.dt;

	orient+=spin*speed;
	if (wraporient) {
		orient.x=wrap(orient.x,0,360);
		orient.y=wrap(orient.y,0,360);
	}
}

//-------------------------------------------------------------------

void Place::apply_shake(void) { // camera/object shake is a time-damped sin(time) value
	float t=Hw::get()->time.time;

	if (shakeampl!=0) {
		shakeampl*=(1-3*Hw::get()->time.dt);
		if (shakeampl<.01) {
			shakeampl=0;
			shakefreqx=rndn(50)+25;
			shakefreqy=rndn(50)+25;
			shakefreqz=rndn(50)+25;
		} else {
			shake.x=shakeampl*sinf(t*shakefreqx);
			shake.y=shakeampl*sinf(t*shakefreqy);
			shake.z=shakeampl*sinf(t*shakefreqz);
		}
	}
	return;
}

//-------------------------------------------------------------------

void Place::update_pos(int manualena) {
	float spd;
	static int init=0;
	static float rnd1,rnd2,rnd3,rnd4,rnd5;
	Hw *hw=Hw::get();
	static Perlin perlin;

	if (init==0) {
		srand(hw->time.msTime);
		rnd1=rndn(1000);
		rnd2=rndn(1000);
		rnd3=rndn(1000);
		rnd4=rndn(1000);
		rnd5=rndn(1000);
		init=1;
	}
	rnd1+=rndn(.0001);
	rnd2+=rndn(.0001);
	rnd3+=rndn(.0001);
	rnd4+=rndn(.0001);
	rnd5+=rndn(.0001);
	switch(movemode){
	case HOLD: break;

	case LOOKAT: // rnd/orbiting position, orient to "to"
		pos.x=sinf(hw->time.spd1*.1 *moverate+rnd1)*moveRange.x+moveCenter.x; // orbit position
		pos.y=cosf(hw->time.spd1*.12*moverate+rnd2)*moveRange.y+moveCenter.y;
		pos.z=cosf(hw->time.spd1*.14*moverate+rnd3)*moveRange.z+moveCenter.z;
		//pos.x=perlin.get(0,hw->time.spd1*.1 )*moveRange.x+moveCenter.x; // orbit position
		//pos.y=perlin.get(0,hw->time.spd1*.11)*moveRange.y+moveCenter.y; // orbit position
		//pos.z=perlin.get(0,hw->time.spd1*.09)*moveRange.z+moveCenter.z; // orbit position
		zLookAt(&pos,&to,&orient.x,&orient.y); // orient to
		break;

	case ORBIT:
		pos.x=sinf(hw->time.spd2*.2*moverate+rnd1)*moveRange.x+moveCenter.x; // orbiting position
		pos.y=sinf(hw->time.spd2*.2*moverate+rnd2)*moveRange.y+moveCenter.y;
		pos.z=sinf(hw->time.spd2*.2*moverate+rnd3)*moveRange.z+moveCenter.z;
		orient.x=sinf(hw->time.spd1*.04*spinrate+rnd4)*spinRange.x+spinCenter.x; // "orbiting" orientation
		orient.y=sinf(hw->time.spd1*.04*spinrate+rnd5)*spinRange.y+spinCenter.y;
		break;

	case MANUAL: // manual mouseLook, arrowKey/Click thrust
		if (manualena) {
			if (hw->mouse.moved) { // mouseLook
				spd=1.75*hw->pref.mouseSens;
				if (hw->mouse.yi!=0) spin.x-=hw->mouse.yi*spd*hw->mouse.ydir;
				if (hw->mouse.xi!=0) {
					int xdir=wrap(orient.x,0,360); // adjust x axis direction for upside down flite
					if ((xdir>90)&&(xdir<270)) xdir=-1;
					else                       xdir= 1;
					spin.y-=hw->mouse.xi*spd*xdir;
				}
				hw->mouse.warp();
			}
			spd=20*hw->meter*hw->pref.moveSens*((9*hw->key.shift)+1); // ArrowKey move
			if (hw->key.arrow[0])    cam_adj_x(-spd);
			if (hw->key.arrow[1])    cam_adj_x( spd);

			if (hw->key.arrow[2])    {
				if (hw->key.control) cam_adj_y(-spd);
				else                 cam_adj_z( spd);
			}
			if (hw->key.arrow[3])    {
				if (hw->key.control) cam_adj_y( spd);
				else                 cam_adj_z(-spd);
			}
			if ((hw->mouse.button[0]==1)&&(hw->mouse.lastdowndt[0]<hw->pref.doubleClickInterval)) spd*=hw->pref.spurt; // Click moves
			if (hw->mouse.button[0]) cam_adj_z(-spd);

			if ((hw->mouse.button[1]==1)&&(hw->mouse.lastdowndt[1]<hw->pref.doubleClickInterval)) spd*=hw->pref.spurt; // Click moves
			if (hw->mouse.button[1]) cam_adj_z( spd);
			if (physics!=0) physics->limit_spd(this); // velocities
			if ((hw->help==0)&&(hw->dialogs==0)) move();
			rotate();
		}
		apply_shake();
		break;

	case LOOKFROM: if (otherPlace!=0) pos=otherPlace->pos; // lookFrom
		if (manualena) {
			if (hw->mouse.moved) { // look?
				spd=1.8*hw->pref.mouseSens;
				if (hw->mouse.yi) spin.x-=hw->mouse.yi*spd*hw->mouse.ydir;
				if (hw->mouse.xi) {
					int xdir=wrap(orient.x,0,360); // adjust x axis direction for upside down flite
					if ((xdir>90)&&(xdir<270)) xdir=-1;
					else                       xdir= 1;
					spin.y-=hw->mouse.xi*spd*xdir;
				}
				hw->mouse.warp();
			}
			rotate();
		}
		break;
	}
}

//---------------------------------------------------------------------

void Place::cam_adj_x(float in) { // move relative to camera orientation
   acc.x+=Hw::get()->screen.viewport->camera.rightx*in;
   acc.y+=Hw::get()->screen.viewport->camera.righty*in;
   acc.z+=Hw::get()->screen.viewport->camera.rightz*in;
}

//-------------------------------------------------------------------

void Place::cam_adj_y(float in) {
	acc.x+=Hw::get()->screen.viewport->camera.upx*in;
	acc.y+=Hw::get()->screen.viewport->camera.upy*in;
	acc.z+=Hw::get()->screen.viewport->camera.upz*in;
}

//------------------------------------------------------------------------

void Place::cam_adj_z(float in) {
	acc.x+=Hw::get()->screen.viewport->camera.forwardx*in;
	acc.y+=Hw::get()->screen.viewport->camera.forwardy*in;
	acc.z+=Hw::get()->screen.viewport->camera.forwardz*in;
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
//-------------------------------------------------------------------
/*
void Vector::aaFromEuler(float rx, float ry, float rz) { 
                 Vector ax(1, 0, 0, rx); 
                 Vector ay(0, 1, 0, ry); 
                 Vector az(0, 0, 1, rz); 
                 aaMul(ax, ay); 
                 aaMul(*this, az);
            };

void Vector::aaMul(vector & aa1, vector &  aa2) { 
                 float c, s, t, x2, y2, z2; 

                  c = cosf(aa1.w); 
                  s = sinf(aa1.w); 
                  t = 1.0f - c; 

                  x2 = aa1.x * aa1.x; 
                  y2 = aa1.y * aa1.y; 
                  z2 = aa1.z * aa1.z; 

                  // Rotate the axis of aa2 around  
                  // the axis of aa1 by the rotation   // angle in aa1   // 
                  x  = (aa2.x * ((t * x2) + c)); 
                  x += (aa2.y * ((t * aa1.x * aa1.y)  + (s * aa1.y))); 
                  x += (aa2.z * ((t * aa1.x * aa1.z)  - (s  * aa1.y))); 

                  y  = (aa2.x * (t * aa1.x * aa1.y)  - (s * aa1.z)); 
                  y += (aa2.y * ((t * y2) + c)); 
                  y += (aa2.z * ((t * aa1.y * aa1.z) + (s * aa1.x))); 

                  z  = (aa2.x * ((t * aa1.x * aa1.z) + (s * aa1.y))); 
                  z += (aa2.y * ((y * aa1.y * aa1.z) - (s * aa1.x))); 
                  z += (aa2.z * ((t * z2) + c)); 

                  //  
                  // Simply set the new angle of rotation from aa2   // 
                  w = aa2.w;

            }
				*/