//
#include "screen.h"
#include "hardware.h"

//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------

Screen::Screen(){
	reset();
};
Screen::~Screen(){};

//------------------------------------------------------------------------

void Screen::reset(void) {
	windowID=-1; capture=0;
	width=500; height=500; windowAspect=1; fullscreen=1; 
	windowWidth=width; windowHeight=height;
	letterbox=0; letterboxBottom=0; letterboxHeight=windowHeight;
	nviewports=1; viewportset=1; currentViewport=0; viewport=&viewports[currentViewport];
	nearz=.1; farz=10; needblank=0;
	r=g=b=a=0;
	dimr=dimg=dimb=dima=0; dimdir=0; dimtime=0; dimduration=0;
}

//------------------------------------------------------------------------

void Screen::bound_cameras(float mx, float my) {
	Place *place;
	for (int viewportn=0; viewportn<nviewports; viewportn++) {
		place=&viewports[viewportn].camera.place;
		if (place->pos.x < -mx) {place->pos.x=-mx; place->vel.x*=.2;}
		if (place->pos.z < -my) {place->pos.z=-my; place->vel.z*=.2;}
		if (place->pos.x >  mx) {place->pos.x= mx; place->vel.x*=.2;}
		if (place->pos.z >  my) {place->pos.z= my; place->vel.z*=.2;}
	}
}

//-------------------------------------------------------------------

void Screen::clear(void){
	Hw *hw=Hw::get();
	int cleared=0;

	if (hw->pref.blank | needblank ) { // fullscreen clear? | (capture>=1)
		if (capture==2) glClearColor(0,0,0,1);
		glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
		if (capture==2) glClearColor(r,g,b,1);
		cleared=1;
	} else glClear(GL_DEPTH_BUFFER_BIT);
	if ((hw->pref.c2>.01)&&   // valid fade value?
		 (cleared==0)) { // and not fullscreen clearing?

		//glViewport(0, letterboxBottom, width, letterboxHeight);
		glViewport(0, 0, width, height);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		//glOrtho(0, width, letterboxBottom, letterboxBottom+letterboxHeight, -1,1); // l-r-b-t-n-f
		glOrtho(0, width, 0, height, -1,1); // l-r-b-t-n-f
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		if (capture==2) glColor4f(0,0,0,hw->pref.c2*hw->time.dt*40);
		else            glColor4f(r,g,b,hw->pref.c2*hw->time.dt*40);
		fade();
	}
	if (capture==1)  { // clear corner
		glColor4f(0,0,0,0);
		glRectf(0,0,256,256);
	}
	if ((letterbox!=0)&&(capture!=2))  {
		glViewport(0, 0, width, height);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0, width, 0, height, -1,1); // l-r-b-t-n-f
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		clear_letterbox();
	}
	needblank=maxi(0,needblank-1); // ensures both buffers cleared
}

//------------------------------------------------------------------------

void Screen::clear_letterbox(void) {
	if (letterbox==0) return;
	//if (capture==2) glColor3f(r,g,b);
	//else
		glColor3f(0,0,0);
	glPushAttrib(GL_POLYGON_MODE);
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		glRectf(0,                              0, width,letterboxBottom);
		glRectf(0,letterboxBottom+letterboxHeight, width,         height);
		glPopAttrib();
	return;
}

//---------------------------------------------------------------------------

void Screen::doc_letterbox(void) {
	if      (fabs(((float)Hw::get()->screen.width / (Hw::get()->screen.height*(1-2*Hw::get()->pref.letterbox))-1.77777) )<.03)
		Hw::get()->ann->hold("Letterbox: %3.0f %% (16:9 Widescreen)",Hw::get()->screen.letterbox*100);
	else if (fabs(((float)Hw::get()->screen.width / (Hw::get()->screen.height*(1-2*Hw::get()->pref.letterbox))-2.35) 	)< .1)
		Hw::get()->ann->hold("Letterbox: %3.0f %% (2.35:1 Theatre)", Hw::get()->screen.letterbox*100);
	else if (fabs(((float)Hw::get()->screen.width / (Hw::get()->screen.height*(1-2*Hw::get()->pref.letterbox))-1.33333) )<.03)
		Hw::get()->ann->hold("Letterbox: %3.0f %% (4:3 TV)",         Hw::get()->screen.letterbox*100);
	else
		Hw::get()->ann->hold("Letterbox: %3.0f %%",                  Hw::get()->screen.letterbox*100);
}

//-----------------------------------------------------------------------------

void Screen::draw_dim(void) {
	static float volume=0;
	float inten;
	Hw *hw=Hw::get();

	if (volume==0) volume=hw->pref.ambientvolume;
	if (hw->time.time>(dimtime+dimduration)) return;// if not dimming return;
	switch((int)dimdir) { // shape timed sawtooth
	case 0: case 1: // 0->1 ON  or 1->0 OFF
		inten=cos_interp(1-dimdir,dimdir,         (hw->time.time-dimtime) / dimduration); break;
	case 2: case 3: // v shape 0->1->0 A   or 1->0->1 V
		inten=cos_interp(3-dimdir,dimdir-2, (fabs((hw->time.time-dimtime) / (dimduration*.5))-1)); break;
	}
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	//if (hw->screen.capture==2)	glColor4f(0,0,0,0);
	//else
		glColor4f(dimr*inten,dimg*inten,dimb*inten,dima*inten);
	//rect();

	glDepthMask(GL_FALSE); 
	glPushAttrib(GL_POLYGON_MODE);
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		glBegin(GL_QUADS);
			if (hw->screen.capture!=2) {
				glTexCoord2f(0,0); glVertex2f(    0,letterboxBottom);
				glTexCoord2f(1,0); glVertex2f(width,letterboxBottom);
				glTexCoord2f(1,1); glVertex2f(width,letterboxBottom+letterboxHeight);
				glTexCoord2f(0,1); glVertex2f(    0,letterboxBottom+letterboxHeight);
			} else {
				glTexCoord2f(0,0); glVertex2f(    0,0);
				glTexCoord2f(1,0); glVertex2f(width,0);
				glTexCoord2f(1,1); glVertex2f(width,height);
				glTexCoord2f(0,1); glVertex2f(    0,height);
			}
			glEnd();
		glPopAttrib();
	glDepthMask(GL_TRUE);
	Hw::get()->ntris+=2;

	glDisable(GL_BLEND);
	if (hw->ex) hw->sound.set_master_volume(1-inten);
	else        hw->pref.ambientvolume=(1-inten)*volume;
}

//-----------------------------------------------------------------------------

void Screen::fade(void) { // variable screen fade F11/F12 control
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	//glColor4f(r,g,b,alpha);
	rect();
	glDisable(GL_BLEND);
	return;
}

//---------------------------------------------------------------------------

void Screen::init() {
	width =glutGet(GLUT_SCREEN_WIDTH);
	height=glutGet(GLUT_SCREEN_HEIGHT);
	log_message("Init Splinter...");
	if (splinter.init()) log_message("Cant make splinter texture **********");
}

//---------------------------------------------------------------------------

void Screen::rect(void) {
	glDepthMask(GL_FALSE); 
	glPushAttrib(GL_POLYGON_MODE);
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		glBegin(GL_QUADS);
			//glTexCoord2f(0,0); glVertex2f(    0,letterboxBottom);
			//glTexCoord2f(1,0); glVertex2f(width,letterboxBottom);
			//glTexCoord2f(1,1); glVertex2f(width,letterboxBottom+letterboxHeight);
			//glTexCoord2f(0,1); glVertex2f(    0,letterboxBottom+letterboxHeight);

			glTexCoord2f(0,0); glVertex2f(    0,0);
			glTexCoord2f(1,0); glVertex2f(width,0);
			glTexCoord2f(1,1); glVertex2f(width,height);
			glTexCoord2f(0,1); glVertex2f(    0,height);
			glEnd();
		glPopAttrib();
	glDepthMask(GL_TRUE);
	Hw::get()->ntris+=2;
}

//-----------------------------------------------------------------------------

void Screen::set_color(float inr,float ing,float inb) {
	r=inr;
	g=ing;
	b=inb;
}

//-------------------------------------------------------------------

void Screen::set_color(float inr,float ing,float inb, float ina) {
	r=inr;
	g=ing;
	b=inb;
	a=ina;
}

//-------------------------------------------------------------------

void Screen::set_dim(float indir, float induration, float inr, float ing, float inb, float ina) {
	dimtime=Hw::get()->time.time;
	dimdir=indir;
	dimduration=induration;
	dimr=inr;
	dimg=ing;
	dimb=inb;
	dima=ina;
}

//--------------------------------------------------------------------

void Screen::set_letterbox(void) {
	letterboxBottom=letterbox*windowHeight; // calc letterbox
	letterboxHeight=windowHeight-2*letterboxBottom;
}

//---------------------------------------------------------------------

void Screen::setup(void) {
	viewport->size_viewport(windowWidth,letterboxHeight,letterboxBottom);
	if (capture==1) viewport->useAspect=windowAspect*viewport->viewportAspect;
	else            viewport->useAspect=             viewport->viewportAspect;
}

//---------------------------------------------------------------------

void Screen::set_ortho(void) {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, viewport->iwidth, 0, viewport->iheight, -1,1); // l-r-b-t-n-f
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

//-----------------------------------------------------------------------------

void Screen::set_perspective(void){
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(viewport->camera.fov, viewport->useAspect, nearz, farz);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

//------------------------------------------------------------------------

void Screen::set_screen(void){
	if (fullscreen) {
		glutFullScreen();
		glutSetCursor(GLUT_CURSOR_NONE);
	} else {
		glutReshapeWindow(Hw::get()->pref.windowWidth,Hw::get()->pref.windowHeight);
		glutSetCursor(GLUT_CURSOR_LEFT_ARROW);
	}
}

//------------------------------------------------------------------------

void Screen::set_nviewports(int n) {
	viewportset=n;
	switch(n) { // viewports left bot right top
		case 1: {float vps[1][4]={0,   0,   1,   1}; load_viewport(vps, 1);}	break;
		case 2: {float vps[2][4]={0,   0,   1,  .5,
	                             0,  .5,   1,   1}; load_viewport(vps, 2);}	break;
		case 3: {float vps[2][4]={ 0,   0,  .5,  1,
	                              .5,  0,  1,   1}; load_viewport(vps, 2);}	break;
		case 4: {float vps[3][4]={0,   0,   1,  .6,
	                             0,  .6,  .5,   1,
								       .5,  .6,   1,   1}; load_viewport(vps, 3);} break;
		case 5: {float vps[3][4]={0,   0,  .5,   1,
	                             .5,  0,   1,  .5,
								        .5, .5,   1,   1}; load_viewport(vps, 3);} break;
		case 6: {float vps[4][4]={0,   0,   1,.666,
	                             0,.666,.333,   1,
	                          .333,.666,.666,   1,
	                          .666,.666,   1,   1}; load_viewport(vps, 4);}	break;
	}
}

//-------------------------------------------------------------------

void Screen::load_viewport(float array[][4], int n) {
	nviewports=n;
	currentViewport=0;
	viewport=&viewports[currentViewport];

	for (int i=0; i<nviewports; i++)
		viewports[i].set_view_pct(array[i][0],array[i][1],array[i][2],array[i][3]);
	viewports[nviewports].set_view_pct(0,0,1,1);
}

//-----------------------------------------------------------------------------

