//
#include "splinter.h"
#include "hardware.h"

//---------------------------------------------------------------------------------------

Splinter::Splinter() {
	skip=0;
	skipDir=1;
	type=128;
	nx=1; ny=1;
	ampl=1; speed=1;
	nextType=0;
	nextSize=0;
	c2=.2;
};

Splinter::~Splinter() {};

//-------------------------------------------------------------------

int Splinter::init(void) {
	return tex.make_tex(BLANK, 256,256, GL_RGBA, 0);
}

//-------------------------------------------------------------------

void Splinter::set_window(void) {
	Hw::get()->screen.windowWidth =tex.sizex;
	Hw::get()->screen.windowHeight=tex.sizey;
}

//--------------------------------------------------------------------

void Splinter::playback(void) {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(90,Hw::get()->screen.windowAspect,.1,100);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0,0,.5, 0,0,0,  0,1,0);
	draw();
}

//-------------------------------------------------------------------

void Splinter::start(void) {
	int x,y;
	float xh,yh;
	float width=Hw::get()->screen.windowAspect, height=1;
	float r1=rndn(1);

	tex.bind(); // grab the image
	//glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0,0,     0,0, tex.sizex,tex.sizey);
	     glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 0,0, tex.sizex,tex.sizey, 0);
	glColor3f(0,0,0);
	glRecti(0,0,tex.sizex,tex.sizey);
	if (Hw::get()->pref.fullscreen) {
		Hw::get()->screen.windowWidth =Hw::get()->screen.width;
		Hw::get()->screen.windowHeight=Hw::get()->screen.height;
	} else {
		Hw::get()->screen.windowWidth =Hw::get()->pref.windowWidth;
		Hw::get()->screen.windowHeight=Hw::get()->pref.windowHeight;
	}

	speed=rndn(1)+.5;
	ampl=1;
	if (skip) return;
	if (nextSize) {nx=nextSize;ny=nextSize;nextSize=0;}
	else {                                         // pick a splinter size/shape:
		if      (r1<.01) {nx=1; ny=1;}                  // 1% big block
		else if (r1<.3) {nx=nshatter-1;ny=nshatter-1;} // 30% tiny
		else if (r1<.6) {nx=rndn(nshatter-3)+2; ny=nx;}// 30% squares
		else            {nx=rndn(nshatter-3)+2;
		                 ny=rndn(nshatter-3)+2;}       //40% rects
	}
	xinc=width  / nx;
	yinc=height / ny;
	xsize=.5*xinc;
	ysize=.5*yinc;
	xh=.5*width;
	yh=.5*height;

	if (nextType) {type=nextType; nextType=0;}// pick a movement type
	else {
		type=0;
		if ((nx==1)&&(ny==1)) {            // big blok
			type+=rndi(2);                  // move ~x
			type+=rndi(2)*2;                // move ~y
			type+=rndi(2)*4;                // move ~z
			if (type==0) type=3;
			type+=pow(2,3+rndi(2));         // +=xdir
			type+=pow(2,5+rndi(2));         // +-ydir
			type+=pow(2,7+rndi(2));         // +-zdir
			type+=512+1024+2048;            //rotate all 3 axis
		} else {
			if (rndn(1)<.75) type=rndi(8191)+1;// most time pick a random type
			else {                             // otherwise always explode
				type=1+2+4;                     // move x/y/z
				type+=pow(2,3+rndi(2));         // +=xdir
				type+=pow(2,5+rndi(2));         // +-ydir
				type+=pow(2,7+rndi(2));         // +-zdir
				type+=512+1024+2048;            //rotate all 3 axis
			}
		}
	}
 	for    (x=0; x<nx; x++) {
		for (y=0; y<ny; y++) {
			xs[x][y]=x*xinc+xsize-xh;
			ys[x][y]=y*yinc+ysize-yh;
			zs[x][y]=0;
			xis[x][y]=sinf(rndn(pi2))*(rndn(.5)+.5)*.2;
			yis[x][y]=sinf(rndn(pi2))*(rndn(.5)+.5)*.2;
			zis[x][y]=sinf(rndn(pi2))*(rndn(.5)+.5)*.1;
			p[x][y]=0;
			h[x][y]=0;
			r[x][y]=0;
			pi[x][y]=rndn(200)-100;
			hi[x][y]=rndn(200)-100;
			ri[x][y]=rndn(200)-100;
		}
	}
	return;
}

//-----------------------------------------------------------------------------

void Splinter::draw(void) {
	int x,y;
	float fx,fy, tincx, tincy, spd;
	float xsizeampl, ysizeampl, width=Hw::get()->screen.windowAspect, height=1;

	glColor4f(1,1,1,1);
	tex.bind();
	if (skip) {do_skip();return;}
	glEnable(GL_TEXTURE_2D);
	spd=Hw::get()->time.dt*speed;
	tincx=1. / nx;
	tincy=1. / ny;
	if (type&4096) { // shrink
		ampl-=Hw::get()->time.dt*.25;
		if (ampl<0) ampl=0;
	}
	xsizeampl=xsize*ampl;
	ysizeampl=ysize*ampl;
	for    (x=0; x<nx; x++) { 
			fx=x*tincx;
		for (y=0; y<ny; y++) {
			fy=y*tincy;
			if (type&1)    xs[x][y] +=xis[x][y]*spd; // move
			if (type&2)    ys[x][y] +=yis[x][y]*spd;
			if (type&4)    zs[x][y] +=zis[x][y]*spd;
			if (type&8)   xis[x][y]-=spd; // deccelerate
			if (type&16)  xis[x][y]+=spd; // accelerate
			if (type&32)  yis[x][y]-=spd;
			if (type&64)  yis[x][y]+=spd;
			if (type&128) zis[x][y]-=spd*.5;
			if (type&256) zis[x][y]+=spd*.5;
			if (type&512)   p[x][y]+=pi[x][y]*spd; // rotate
			if (type&1024)  h[x][y]+=hi[x][y]*spd;
			if (type&2048)  r[x][y]+=ri[x][y]*spd;
			glPushMatrix();
				glTranslatef(xs[x][y],ys[x][y],zs[x][y]);
				glRotatef(p[x][y],1,0,0);
				glRotatef(h[x][y],0,1,0);
				glRotatef(r[x][y],0,0,1);
				glBegin(GL_QUADS);
					glTexCoord2f(fx      ,fy);       glVertex2f(-xsizeampl,-ysizeampl);
					glTexCoord2f(fx+tincx,fy);       glVertex2f(+xsizeampl,-ysizeampl);
					glTexCoord2f(fx+tincx,fy+tincy); glVertex2f(+xsizeampl,+ysizeampl);
					glTexCoord2f(fx      ,fy+tincy); glVertex2f(-xsizeampl,+ysizeampl);
					glEnd();
				glPopMatrix();
		}
	}
	glDisable(GL_TEXTURE_2D);
	Hw::get()->ntris+=nx*ny*2;
}

//-----------------------------------------------------------------------------

void Splinter::do_skip(void) {
	float spd=Hw::get()->time.dt*speed, skipd;
	float width=Hw::get()->screen.windowAspect, height=1;
	glEnable(GL_TEXTURE_2D);
	if (skip<1) {
		skip+=1.75*Hw::get()->time.dt;
		if (skip>=1) {
			if (rndn(1)<.5) {
				skip=0;
				Hw::get()->screen.capture=0;
			} else skip=.01;
		}
	} else {
		skip+=1.75*Hw::get()->time.dt;
		if (skip>=2) {
			if (rndn(1)<.5) {
				skip=0;
				Hw::get()->screen.capture=0;
			} else skip=1.01;
		}
	}
	skipd=skip*skipDir;
	glPushMatrix();
		glTranslatef(-.5*width,-.5*height,0);
		if (skip>1) {
			glBegin(GL_QUADS);
				glTexCoord2f(  skipd,0);  glVertex2f(0,0);
				glTexCoord2f(1+skipd,0);  glVertex2f(width,0);
				glTexCoord2f(1+skipd,1);  glVertex2f(width,height);
				glTexCoord2f(  skipd,1);  glVertex2f(0,height);
				glEnd();
		} else {
			glBegin(GL_QUADS);
				glTexCoord2f(0,  skipd); glVertex2f(0,0);
				glTexCoord2f(1,  skipd); glVertex2f(width,0);
				glTexCoord2f(1,1+skipd); glVertex2f(width,height);
				glTexCoord2f(0,1+skipd); glVertex2f(0,height);
				glEnd();
		}
		glPopMatrix();
	glDisable(GL_TEXTURE_2D);
	Hw::get()->ntris+=2;
}

//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------

