//
//---- standard framework objects, code and callbacks that rarely change
//

Hw *hw;            // hardware pointer

Config config;     // load file and update prefs
Dialog dialogObject, *dialog=&dialogObject; // old popup menu system
Fog fog;
Light light;       // gl light0
Listbox listbox;   // info.txt file scroller
Randoms randoms;   // manager for random splinter, lightmap, blank... effects
Scroller scroller; // one-line text/hint scroller
Skybox skybox;
Starfield  starfield;  // far flat star glpoints
Starfield2 starfield2; // near particle textured quads
Text ann, clock, note, title, splash, dialogText; // feedback annotation, fps and other config'd text objects
Texture fontT, controlsT, sphereT, sunT; // starT, smokeT, 

//-------------------------------------------------------------------

void bail(char *s) { log_message(s); glut_shutdown(); }

//--------------------------------------------------------------------

void glut_init(int argc, char** argv) {
	log_message("Initing glut...");
	glutInit(&argc,argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
	glutInitWindowSize(400,400);
	glutInitWindowPosition(0,0); 
	hw->screen.windowID=glutCreateWindow(hw->title);
	if (hw->pref.fullscreen) {
		glutFullScreen();
		glutSetCursor(GLUT_CURSOR_NONE);
	}
	glutDisplayFunc(display); 
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(specialdown);
	glutSpecialUpFunc(specialup);
	glutMotionFunc(motion);
	glutPassiveMotionFunc(motion);
	glutMouseFunc(mouseCB);
	//glutIdleFunc(idle);
	
	init(); // framework init

	glutTimerFunc(10, timer, hw->timern); // prep for display in 10.9.8...
	log_message("Ready.\n");
	glutMainLoop();
}

//--------------------------------------------------------------------

void glut_shutdown(void) {
	log_message(" ");
	log_message("Shutting down...");
	close_this();

	if (hw->sound.inited) {
		log_message("Closing sound...");
		hw->sound.stop_sound();
		hw->sound.close();
	}
	log_message("Deleting textures...");
	Texture::close(); // close all ID's

	log_message("Shutting down glut..."); // minimize leaks and crashes
	glutDisplayFunc(noop); 
	glutIdleFunc(NULL);
	glutReshapeFunc(NULL);
	glutTimerFunc(0, NULL, 0);
	glutKeyboardFunc(NULL);
	glutSpecialFunc(NULL);
	glutSpecialUpFunc(NULL);
	glutMotionFunc(NULL);
	glutPassiveMotionFunc(NULL);
	glutMouseFunc(NULL);
	log_message("Destroying window...");
	glutDestroyWindow(hw->screen.windowID);
	hw->screen.windowID=-1;
	log_message("Fin.                             zintel.com");
	log_message("                                        ");
	if (hw!=0) delete hw;
	exit(0);
}

//--------------------------------------------------------------------

void init(void) { // 1 time framework init
	log_message("Init...");

	Text::tex=&fontT; // bind text class w/ font texture

	init_multitexture();
	hw->screen.init();

	ann.size=hw->screen.width*.0117;//12; // text object setup
	ann.duration=hw->pref.annotateDuration;
	clock.size=hw->screen.width*.01;//11;
	note.size=hw->screen.width*.0087;//9;
	splash.size=15; splash.autoset=0;
	scroller.size=hw->screen.width*.012; //14
	listbox.load();

	log_message(".                                              ");
	log_message("    ||||||||   ||||||   ||||  ||||             ");
	log_message("  ------------------------------------         ");
	log_message("   -----//-------||------||\\--||----      ~   ");
	log_message("    ---//--------||------||-\\-||---     {*l*} ");
	log_message("   ---//---------||------||--\\||----     )-(  ");
	log_message("  ------------------------------------         ");
	log_message("    ||||||||   ||||||   ||||  ||||             ");
	log_message(".                                              ");
	log_message(".                           Just a moment...   ");
	log_message(".                                              ");
// abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ 1234567890 !@#$%^&*()-_=+`~[{]}\|;:'",<.>/?
	 // ---- sound -------------------------

	hw->sound.set_rolloff(1);
	hw->sound.mindistance=10;
	hw->sound.maxdistance=1000;
	int err=hw->sound.init();
	if (!err) hw->sound.set_master_volume(0);
	else log_message("Cannot init sound **********");

	log_message("Init fullscreen lightmap...");
	if (hw->screen.lightmap.tex.make_tex(SPHERE,128,128,GL_LUMINANCE_ALPHA,0))
		log_message("Cant make texture lightmap **********");

	randoms.lm=&hw->screen.lightmap; // bind lightmap and splinter to randomfx class
	randoms.splinter=&hw->screen.splinter;

	init_this(); // local init
	hw->mouse.warp();

	// init various opengl attributes
	glClearColor(hw->screen.r,hw->screen.g,hw->screen.b,hw->screen.a);
	fog.init();
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);  
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);

	glDisable(GL_DEPTH_TEST);
	glDepthMask(GL_TRUE);
	glDisable(GL_DITHER);
	glShadeModel(GL_SMOOTH);
	glLineWidth(2);
	glPointSize(2);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	hw->screen.set_dim(DIMOFF, 8, 0,0,0,1); // fade full->0, over n sec, in black
	srand(hw->time.msTime);

	hw->sound.set_master_volume(0);
}
		//hw->sound.open_geom();
		//hw->sound.set_reverb(13);
		//hw->sound.mod=sound.load_song("cochlea.xm");
		//hw->sound.play_music(sound.mod);
		//hw->sound.set_music_volume(sound.mod,1);
		//hw->sound.stop_music();
		//hw->sound.free_song();

//--------------------------------------------------------------------
//------ callbacks ---------------------------------------------------
//--------------------------------------------------------------------

void keyboard(unsigned char inkey, int x, int y) {
	if ((hw->anyinput==0)&&(inkey!=27)) hw->anyinput=1;
	else hw->key.set_key(inkey,glutGetModifiers());
	srand(hw->time.msTime);
}

//--------------------------------------------------------------------

void specialdown(int inkey, int x, int y){
	if (hw->anyinput==0) hw->anyinput=1;
	else 	hw->key.set_skey(inkey,glutGetModifiers());
	srand(hw->time.msTime);
}

//---------------------------------------------------------------------

void specialup(int inkey, int x, int y) {
	hw->key.unset_skey(inkey,0);
}

//----------------------------------------------------------------------------------

void motion(int glutx, int gluty){
	hw->mouse.set_pos(glutx,gluty);
	srand(hw->time.msTime);
	//hw->timern=1-hw->timern;
	//glutPostRedisplay();
 }
//gluty=hw->screen.windowHeight-gluty-1; //flipy

//---------------------------------------------------------------------------

void mouseCB(int inbutton, int state, int glutx, int gluty){
	srand(hw->time.msTime);
	if (hw->anyinput==0) {
		hw->anyinput=1;
		return;
	}
	hw->mouse.set_pos(glutx,gluty);
	hw->mouse.set_button(inbutton,state,glutGetModifiers());
	if ((hw->mouse.button[2]==1)&&(hw->skipSplash))  {
		if (hw->dialogs==0) {
			dialog->on();
			hw->mouse.button[0]=0;
			hw->mouse.button[2]=0;
		} else dialog->off();
	}
	//hw->timern=1-hw->timern;
	//glutPostRedisplay();
}

//---------------------------------------------------------------

void reshape(int width, int height) {
	if (!hw->pref.fullscreen) {
		hw->pref.windowWidth=width;
		hw->pref.windowHeight=height;
	}
	hw->screen.windowWidth =width;
	hw->screen.windowHeight=height;
	hw->screen.windowAspect=(float)hw->screen.windowWidth / hw->screen.windowHeight;
}

//--------------------------------------------------------------------

void timer(int value) {
	//glutPostRedisplay();//glutTimerFunc(TIMER, timer, 0); // set min delay
	//if (value==hw->timern) 
		glutPostRedisplay();
}

//--------------------------------------------------------------------

void idle(void) {
	//if (rndn(1)<.5)  // experiment
		//glutPostRedisplay();
}

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

// the display loops implements the following functions:
//
//  start the frame (all non-graphic functions like update timing, process input) 
//  clear screen
//  set letterbox size
//  for all viewports:
//     setup viewport & 3d perspective mode
//     place camera
//     render skybox/starfield/particles, and all 3d effect stuff
//     setup 2d ortho
//     render all 2d effect stuff
//   set fullscreen 2d ortho
//   render optional splinter and lightmap/mask
//   end frame (screen annot/fps/notes/gui menu/dimming, sequence quad-state inputs)
//   update screen buffer
//

 
void display(void) {
	Screen *screen=&hw->screen;
	static int frame=-1;

	if      (frame==-1) frame=0;
	else if (frame== 0) {
		hw->sound.set_pause(FALSE);
		hw->time.offset=-(hw->time.realTime-.30); // reduce to retard
		frame=1;
		//lastscenetime=0;///////////////////
	}

	screen->viewport=&screen->viewports[screen->currentViewport];

	start_frame();
	if (screen->capture==1) hw->screen.splinter.set_window(); // if capturing, set small window
	glClearColor(hw->screen.r,hw->screen.g,hw->screen.b,hw->screen.a);
	screen->set_letterbox();
	screen->clear();

	//if ((screen->capture<2)||(hw->screen.splinter.plus))       // draw all viewports
	{	for(screen->viewportn=0; screen->viewportn<screen->nviewports; screen->viewportn++) {
			screen->viewport=&screen->viewports[screen->viewportn];

			screen->setup(); // letterbox/viewport
			screen->set_perspective(); // setup 3d pass -------

			if (camscript.play) {
				camscript.update(hw->pref.spd2);
				camscript.script2place(&screen->viewport->camera.place);
			}

			screen->viewport->camera.orient();// orient cam
			if ((screen->capture>1)&&(hw->screen.splinter.plus==0)) continue;

			if (hw->pref.universe>3) {
				skybox.size=hw->worldSize*skybox.gap;
				if (hw->pref.universe&4) {
					skybox.set_xyz(screen->viewport->camera.place.pos.x+screen->viewport->camera.eyex,
										screen->viewport->camera.place.pos.y+screen->viewport->camera.eyey,
										screen->viewport->camera.place.pos.z);//+screen->viewport->camera.eyez);
				} else {skybox.x=0;skybox.y=0;skybox.z=0;}
				skybox.draw();
			}

			if (hw->pref.universe&1) starfield.draw();
			draw_this(); // main draw in 3d
			if (hw->pref.universe&2) starfield2.draw();

			screen->set_ortho(); // setup 2d pass ------------------------------------
			if ((screen->viewportn==screen->currentViewport)&&
				 (screen->nviewports>1)&&(hw->pref.demosequence==0)) screen->viewport->frame();
			annot_this(); // main draw in 2d
		}
	}
	//if (hw->ex) hw->screen.letterbox=0;
	screen->viewport=&hw->screen.viewports[screen->nviewports]; // setup fullscreen 2d
	screen->viewport->size_viewport(screen->windowWidth,screen->windowHeight,0);
	screen->set_ortho();

	if (screen->capture==1) hw->screen.splinter.start(); // capture it
	else if (screen->capture==2) {
		hw->screen.splinter.playback();
		screen->set_ortho();
	}
	hw->screen.draw_dim(); // check dimming
	if (hw->screen.lightmap.on) {
		hw->screen.lightmap.update();
		hw->screen.lightmap.draw();
	}
	end_frame();
	glutTimerFunc(4, timer, hw->timern); // set min delay, give other funcs a chance
}

//-----------------------------------------------------------------------

void start_frame(void) {
	hw->time.sequence();

	if (hw->time.time<10) hw->sound.set_master_volume(hw->time.time / 10.); // fade in musicsound
	//if (hw->ex) hw->sound.set_master_volume(1-((hw->time.time-hw->ex) / hw->shutdownduration)); // fade out sound

	key(); // process all keys & buttons

	if ((hw->skipSplash==0)||(hw->help==1)) listbox.update();

	if (hw->pref.fog) fog.set_distance(hw->pref.b1,hw->pref.b2); // update vars
	if (hw->screen.fullscreen==0) hw->mouse.moved=0; // disable mouselook in window
	hw->screen.letterbox=hw->pref.letterbox;
	//hw->screen.lightmap.set_color(1,1,1,1);
	set_polymode(&hw->pref);
	ann.set_xyz(  .02*hw->screen.windowWidth,.97*hw->screen.windowHeight);// update screen pos
	clock.set_xyz(hw->screen.windowWidth-100,.97*hw->screen.windowHeight);
	note.set_xyz( .42*hw->screen.windowWidth,.93*hw->screen.windowHeight);//.02 .93
	//note.set_xyz(.02*hw->screen.windowWidth,    .93*hw->screen.windowHeight);
	//note.set_xyz(.01*hw->screen.windowWidth,    .01*hw->screen.windowHeight);
	hw->ntris=0;
	if (hw->pref.universe&2) starfield2.update();
	start_this();
	if (hw->pref.viewportset!=hw->screen.viewportset) hw->screen.set_nviewports(hw->pref.viewportset);
}

//--------------------------------------------------------------------

void end_frame(void) {
	Screen *screen=&hw->screen;
	float speed=hw->screen.viewports[hw->screen.currentViewport].camera.place.vel.length()*2.24657f;

	for(screen->viewportn=0; screen->viewportn<screen->nviewports; screen->viewportn++) {
		screen->viewport=&screen->viewports[screen->viewportn];
		screen->viewport->camera.place.update_pos( // update camera pos
			(screen->viewportn==screen->currentViewport) && // if in active viewport ?////////
			(hw->dialogs==0));                         // and no dialogs
	}

	if ((hw->screen.capture!=1)&&(hw->ex==0)) 
	{
		ann.print_hold(); // print any held text
		scroller.print();

		if ((hw->skipSplash==0)||(hw->help==1)) splash_this();	
		else
		{
			if (hw->pref.clock) { // fps clock
				clock.hold("FPS %d",hw->time.fps);
				clock.print_hold();
			}
			if (hw->pref.n) {  // info text
				float f;
				switch(hw->pref.n) {
					case 1: note.hold("# Tris: %d",hw->ntris); break;
					case 2: note.hold("MegaTris/Sec: %.2f",hw->ntris*hw->time.fps*.000001f); break;
					case 3: note.hold( "Cam X: %.2f", hw->screen.viewports[hw->screen.currentViewport].camera.place.pos.x);  break;
					case 4: note.hold( "Cam Y: %.2f", hw->screen.viewports[hw->screen.currentViewport].camera.place.pos.y);  break;
					case 5: note.hold( "Cam Z: %.2f", hw->screen.viewports[hw->screen.currentViewport].camera.place.pos.z);  break;
					case 6:
						f=wrap(hw->screen.viewports[hw->screen.currentViewport].camera.place.orient.x, 0,360);
						if (f>180) f-=360;
						note.hold(  "Pitch %.2f deg.", f); break;
					case 7:
						f=wrap(hw->screen.viewports[hw->screen.currentViewport].camera.place.orient.y, 0,360);
						if (f>180) f-=360;
						note.hold("Heading %.2f deg.", f); break;
					case 8: note.hold("MouseX: %d", hw->mouse.x); break;
					case 9: note.hold("MouseY: %d", hw->mouse.y); break;
					case 10: note.hold("Time: %.3f",    hw->time.time); break;
					case 11: note.hold("Time-dt: %.3f", hw->time.dt); break;
					case 12:	note.hold("# playing channels: %d", hw->sound.get_channels_playing()); break;

#if defined haseffect
					case 13: // speedometer & physics indicator
						{
							void *n=hw->screen.viewports[hw->screen.currentViewport].camera.place.physics;
							int x=5, y=12, size=9;
							static char s1[10]={0}, s2[10]={0};
							static float lastTime=0;

							if (hw->time.realTime>(lastTime+.25)) {
								lastTime=hw->time.realTime;
								if      (n==&effect.spacePH)  strcpy(s1,"Space");
								else if (n==&effect.airPH)    strcpy(s1,"Air");
								else if (n==&effect.groundPH) strcpy(s1,"Ground");
							}
							sprintf(s2,"%d mph",(int)speed);
							guage(s1,s2,90,5,speed,200);
						}
						break;
#endif

					}
				if (note.s[0]!=0) note.print_hold();
			}
		}
		if ((hw->skipSplash)&&(hw->screen.capture!=1)) {
			if (hw->dialogs)       dialog->run();
			else if (dialog->fade) dialog->show();
		}
	}

	hw->screen.viewport=&hw->screen.viewports[hw->screen.currentViewport];
	end_this();

	if (dialog->refresh) dialog_init(dialog->ID,dialog->refresh-1); // update/replace dialog
	hw->key.sequence(); // step states
	hw->mouse.sequence();
	if (hw->screen.capture==1) hw->screen.capture=2;
	glutSwapBuffers();
	after_swap();
	hw->sound.update_listener();
	if (hw->ex) {
		if (hw->time.time>(hw->ex+hw->shutdownduration)) glut_shutdown();
	}
	int err=glGetError();
	if (err!=GL_NO_ERROR) printf("Error %x\n", err);
}

//--------------------------------------------------------------------

void key(void){
	if ((hw->key.alt==0)&&(hw->key.control==0)&&(hw->key.key!=0)) {
		switch(hw->key.key){
		case 27:
			//hw->sound.stop_sound();
			//hw->sound.play2d("sounds/.wav");
			//hw->sound.set_volume(.75);
			hw->ex=hw->time.time;
			hw->screen.set_dim(DIMON, hw->shutdownduration*1.1, 0,0,0,1); // fade 0->full, over 2 sec, in black
			hw->screen.capture=1; hw->pref.blank=0; hw->pref.c2=.2;
			hw->pref.user=1;
			//hw->sound.pause=1;
			//hw->sound.on=0;
			//hw->sound.stop_music(sound.mod);
			break;
		case 'q': glut_shutdown(); break;
		
		case '1': case '2': case '3': case '4':  case '5': case '6': 
			hw->pref.viewportset=hw->key.key-'0';
			hw->screen.set_nviewports(hw->pref.viewportset); break;

		case 'b': hw->pref.blendFunc0=(hw->pref.blendFunc0==8)?0:hw->pref.blendFunc0+1;
			ann.hold(doc_blend()); break;
		case 'B': hw->pref.blendFunc1=(hw->pref.blendFunc1==7)?0:hw->pref.blendFunc1+1;
			ann.hold(doc_blend()); break;

		case 'c': hw->pref.clock=!hw->pref.clock;
			ann.hold_multi(hw->pref.clock,"Clock Off","FPS Clock On");
			break;

		case 'C': hw->pref.colorMode=(hw->pref.colorMode==4)?0:hw->pref.colorMode+1;
			switch(hw->pref.colorMode) {
				case 0: ann.hold("RGB"); break;
				case 1: ann.hold("Sepia"); break;
				case 2: ann.hold("BlueGreen"); break;
				case 3: 
					ann.hold("NightVision");
					hw->screen.lightmap.on=1;
					hw->screen.lightmap.ampl=50;
					hw->screen.lightmap.type=36;
					hw->screen.lightmap.texmagx=.5;
					hw->screen.lightmap.texmagy=2;
					break;
				case 4: ann.hold("InverseColor"); break;
				case 5: ann.hold("B&W"); break;
			}
			if (hw->pref.colorMode==3) {
				hw->screen.lightmap.on=1;
				hw->screen.lightmap.ampl=50;
				hw->screen.lightmap.type=36;
				hw->screen.lightmap.texmagx=.5;
				hw->screen.lightmap.texmagy=2;
			} else hw->screen.lightmap.on=0;

			if (hw->pref.colorMode==5) Texture::globalBlendMode=GL_BLEND;
			else                       Texture::globalBlendMode=0;
			break;

		case 'd': hw->pref.fog=!hw->pref.fog;
			ann.hold_multi(hw->pref.fog,"Fog Off","Fog On");
			break;

		case 'f': hw->pref.fullscreen=!hw->pref.fullscreen;
			if (hw->pref.fullscreen) {
				hw->mouse.warp();
				ann.hold("FullScreen");
			} else ann.hold("Window"); 
			break;

		case 'i': 
			hw->pref.image=(hw->pref.image==(Texture::ntextures-1))?0:hw->pref.image+1;
			//ann.hold("Image: %d",hw->pref.image); break;
			if (maximage) {
				sprintf(s,"Image: %d: %s",hw->pref.image, &Texture::names[hw->pref.image][0]);
				ann.hold(s);
			}
				break;

		case 'I': hw->pref.scrollInfo=!hw->pref.scrollInfo;
			ann.hold_multi(hw->pref.scrollInfo,"Scrolling Help Info Off","Scrolling Help Info On");
			if (!hw->pref.scrollInfo) scroller.reset();
			else scroller.lastscrolltime=0;
			break;
		
		case 'l': {
				static letterboxi=1;
				hw->pref.letterbox+=.01*letterboxi;
				if ((hw->pref.letterbox<0)||(hw->pref.letterbox>.4)) {
					letterboxi=-letterboxi;
					hw->pref.letterbox+=.01*letterboxi;
				}
				hw->screen.letterbox=hw->pref.letterbox;
				if (letterboxi<0) hw->screen.needblank=2;
				hw->screen.doc_letterbox();
			}
			break;

		case 'L': hw->screen.lightmap.on=!hw->screen.lightmap.on; ann.hold_multi(hw->screen.lightmap.on,"Lightmap Off","Lightmap On");break;

		case 'n': case 'N':
			if (hw->key.key=='n') hw->pref.n=(hw->pref.n==maxn)?   0:hw->pref.n+1;
			else                  hw->pref.n=(hw->pref.n==   0)?maxn:hw->pref.n-1;
			if (hw->pref.n==0) note.hold("");
			break;

		case 'o': hw->pref.option=(hw->pref.option==maxoption)?0:hw->pref.option+1; ann.hold("Option: %d", hw->pref.option); break;
		
		case 'p': hw->pause=1-hw->pause;
			ann.hold_multi(hw->pause,"Continue","Pause");
			break;
		
		case 'r':
			hw->pref.reset();
			reset_this();
			ann.hold("Reset to default prefs");
			dialog->off();
			break;

		case 's': hw->pref.lights=!hw->pref.lights;
			ann.hold_multi(hw->pref.lights,"Lights Off","Lights On");
			break;

		case 't': hw->pref.textures=!hw->pref.textures;
			ann.hold_multi(hw->pref.textures,"Textures Off","Textures On");
			break;

		case 'u': hw->pref.universe=(hw->pref.universe==maxuniverse)?0:hw->pref.universe+1;
			if (hw->pref.universe==0) ann.hold("No Universe");
			else ann.hold(doc_universe());
			break;

		case 'U': hw->pref.user=!hw->pref.user;
			ann.hold_multi(hw->pref.user,"User Mode Off","UserMode On");
			break;

		case 'v':
			switch(hw->screen.viewport->camera.place.movemode) {
				case MANUAL:
					hw->screen.viewport->camera.place.movemode=HOLD;
					hw->screen.viewport->camera.lastmanpos   =hw->screen.viewport->camera.place.pos;
					hw->screen.viewport->camera.lastmanorient=hw->screen.viewport->camera.place.orient;
					ann.hold("Hold");
					break;

				case HOLD:	hw->screen.viewport->camera.place.movemode=ORBIT;  ann.hold("Random/Orbit Pos & Orient"); break;

				case ORBIT:	hw->screen.viewport->camera.place.movemode=LOOKAT; ann.hold("Look at..."); break;

				case LOOKAT:
					if (hw->screen.viewport->camera.place.otherPlace!=0) {
						hw->screen.viewport->camera.place.movemode=LOOKFROM;
						//hw->screen.viewport->camera.lastmanpos   =hw->screen.viewport->camera.place.pos;
						//hw->screen.viewport->camera.lastmanorient=hw->screen.viewport->camera.place.orient;
						ann.hold("Look From");
					} else {
						hw->screen.viewport->camera.place.movemode=MANUAL;
						hw->screen.viewport->camera.place.pos   =hw->screen.viewport->camera.lastmanpos;
						hw->screen.viewport->camera.place.orient=hw->screen.viewport->camera.lastmanorient;
						ann.hold("Manual ArrowKey/Mouse Control");
					}
					break;

				case LOOKFROM:
					hw->screen.viewport->camera.place.movemode=MANUAL;
					hw->screen.viewport->camera.place.pos   =hw->screen.viewport->camera.lastmanpos;
					hw->screen.viewport->camera.place.orient=hw->screen.viewport->camera.lastmanorient;
					ann.hold("Manual ArrowKey/Mouse Control");
					break;
			}
			break;

		case 'V':
			hw->screen.currentViewport=(hw->screen.currentViewport==(hw->screen.nviewports-1))?0:hw->screen.currentViewport+1;
			hw->screen.viewport=&hw->screen.viewports[hw->screen.currentViewport]; break;

		case 'w': case 'W': hw->pref.wireframe=(hw->pref.wireframe==2)?0:hw->pref.wireframe+1;
			ann.hold_multi(hw->pref.wireframe,"Filled Polys","WireFrame","Points");
			break;

		case 'x':
			if (hw->screen.capture==0) {hw->screen.capture=1; hw->pref.blank=0;hw->pref.c2=hw->screen.splinter.c2;}
			else                       {hw->screen.capture=0; hw->pref.blank=1;hw->pref.c2=.5;}
			break;

		case 'X': hw->screen.splinter.skip=.01+rndi(2); hw->screen.capture=1; hw->screen.splinter.skipDir=rndi(2)*2-1; break;

		case '/': case '?':
			hw->help=!hw->help; 
			if (hw->help) {
				listbox.curline=listbox.keyline;
				dialog->off();
			}
			break;

		case '<': case '>':
			if (hw->key.key=='<') {hw->pref.tess=maxi(mintess,hw->pref.tess-1); hw->pref.tessf*=0.95   ;}
			else                  {hw->pref.tess=mini(maxtess,hw->pref.tess+1); hw->pref.tessf*=1.05263;}
			//ann.hold("Tesselation: %d", hw->pref.tess);
			break;

		case '-': hw->pref.spd1*=0.9    ; ann.hold("Rotate Spd: %.3f", hw->pref.spd1); break;
		case '=': hw->pref.spd1*=1.11111; ann.hold("Rotate Spd: %.3f", hw->pref.spd1); break;
		case '_': hw->pref.spd2*=0.9    ; ann.hold("Move Spd: %.3f", hw->pref.spd2); break;
		case '+': hw->pref.spd2*=1.11111; ann.hold("Move Spd: %.3f", hw->pref.spd2); break;

		case '[': hw->pref.bracket*=0.9    ; ann.hold("Size: %.3f", hw->pref.bracket); break;
		case ']': hw->pref.bracket*=1.11111; ann.hold("Size: %.3f", hw->pref.bracket); break;
		
		case '(': hw->pref.paren*=0.9    ; ann.hold("Paren: %.3f", hw->pref.paren); break;
		case ')': hw->pref.paren*=1.11111; ann.hold("Paren: %.3f", hw->pref.paren); break;

		case '{': case '}':
			if (hw->key.key=='{') hw->pref.brace-=.01*hw->worldSize;
			else                  hw->pref.brace+=.01*hw->worldSize;
			hw->screen.viewport->camera.eyez=hw->pref.brace;
			ann.hold("Eye Z: %.3f", hw->pref.brace); break;
		
		case ',': case '.':
			if (hw->key.key==',') hw->screen.viewport->camera.fov=clampi(hw->screen.viewport->camera.fov+1,1,179);
			else                  hw->screen.viewport->camera.fov=clampi(hw->screen.viewport->camera.fov-1,1,179);
			ann.hold("Fov: %.0f", hw->screen.viewport->camera.fov); break;
		
		case ' ': hw->pref.blank=!hw->pref.blank;
			ann.hold_multi(hw->pref.blank,"Blanking Off","Blanking On");
			break;

		case '`':
			if (hw->skipSplash) {
				if (hw->dialogs==0) dialog->on();
				else                dialog->off();
			}
			break;

		case '!':
			hw->key.shift=0;
			if (hw->pref.save(hw->title)) ann.hold("Could NOT Save Prefs");
			else                          ann.hold("Saved Prefs");
			break;

		case '@': {
				hw->key.shift=0;
				if (hw->pref.load(hw->title))
					ann.hold("Could NOT Load Prefs");
				else {
					ann.hold("Loaded Prefs");
					dialog->off();
					hw->screen.needblank=2;
					if (hw->pref.viewportset!=hw->screen.viewportset) hw->screen.set_nviewports(hw->pref.viewportset);
				}
			}
			break;
		}
	}
	if ((hw->key.alt==0)&&(hw->key.control==0)&&(hw->key.shift==0)&&(hw->key.skey!=0)) { // unmod'd special key
		static float lastinserttime=0;
		switch(hw->key.skey){
			case GLUT_KEY_INSERT: // 127
				if ((hw->time.time-lastinserttime)>.666) {
					lastinserttime=hw->time.time;
					hw->sound.play2d("sounds/hit.wav");
					hw->sound.set_volume(1);
					hw->sound.set_frequency(hw->pref.home*10000);
				}
				break;
			//case GLUT_KEY_F1: hw->pref.a1*=0.95   ; ann.hold("TexMagX: %.3f", hw->pref.a1); break;
			//case GLUT_KEY_F2: hw->pref.a1*=1.05263; ann.hold("TexMagX: %.3f", hw->pref.a1); break;
			//case GLUT_KEY_F3: hw->pref.a2*=0.95   ; ann.hold("TexMagY: %.3f", hw->pref.a2); break;
			//case GLUT_KEY_F4: hw->pref.a2*=1.05263; ann.hold("TexMagY: %.3f", hw->pref.a2); break;
			case GLUT_KEY_F5: case GLUT_KEY_F6:
				if (hw->key.skey==GLUT_KEY_F5) hw->pref.b1-=0.01*hw->worldSize;
				else                           hw->pref.b1+=0.01*hw->worldSize; 
				fog.adjust(1,&hw->pref.b1,&hw->pref.b2); hw->pref.fog=1;
				ann.hold("Fog Near: %.3f", hw->pref.b1); break;

			case GLUT_KEY_F7: case GLUT_KEY_F8:
				if (hw->key.skey==GLUT_KEY_F7) hw->pref.b2-=0.01*hw->worldSize;
				else                           hw->pref.b2+=0.01*hw->worldSize; 
				fog.adjust(2,&hw->pref.b1,&hw->pref.b2); hw->pref.fog=1; 
				ann.hold( "Fog Far: %.3f", hw->pref.b2); break;

			case GLUT_KEY_F9:  hw->pref.c1*=0.95   ; ann.hold("c1: %.3f", hw->pref.c1); break;
			case GLUT_KEY_F10: hw->pref.c1*=1.05263; ann.hold("c1: %.3f", hw->pref.c1); break;
			case GLUT_KEY_F11: hw->pref.c2*=0.95   ; ann.hold("Fade: %.3f", hw->pref.c2);
				if (hw->pref.c2<1) hw->pref.blank=0;
				break;
			case GLUT_KEY_F12: hw->pref.c2*=1.05263; ann.hold("c2: %.3f", hw->pref.c2);
				if (hw->pref.c2>=1) {
					hw->pref.c2=1;
					hw->pref.blank=1;
				}
				break;
			case GLUT_KEY_PAGE_UP:   hw->pref.page*=1.05263; break;
			case GLUT_KEY_PAGE_DOWN: hw->pref.page*=0.95   ; break;
			case GLUT_KEY_HOME:      hw->pref.home*=1.05263; break;
			case GLUT_KEY_END:       hw->pref.home*=0.95   ; break;
		}
	}
	if ((hw->key.alt==0)&&(hw->key.control==0)&&(hw->key.shift==1)) { // shifted - lightmap bank
		switch(hw->key.skey){
			case GLUT_KEY_F1: hw->screen.lightmap.texmagx*=0.95   ; ann.hold("Lightmap TexMagX: %.2f", hw->screen.lightmap.texmagx); break;
			case GLUT_KEY_F2: hw->screen.lightmap.texmagx*=1.05263; ann.hold("Lightmap TexMagX: %.2f", hw->screen.lightmap.texmagx); break;
			case GLUT_KEY_F3: hw->screen.lightmap.texmagy*=0.95   ; ann.hold("Lightmap TexMagY: %.2f", hw->screen.lightmap.texmagy); break;
			case GLUT_KEY_F4: hw->screen.lightmap.texmagy*=1.05263; ann.hold("Lightmap TexMagY: %.2f", hw->screen.lightmap.texmagy); break;

			case GLUT_KEY_F5: case GLUT_KEY_F6:
				if (hw->key.skey==GLUT_KEY_F5) hw->screen.lightmap.updateInterval=maxf(0,hw->screen.lightmap.updateInterval-.01);
				else                           hw->screen.lightmap.updateInterval=minf(1,hw->screen.lightmap.updateInterval+.01);
				ann.hold("Lightmap UpdateInterval: %.2f", hw->screen.lightmap.updateInterval); break;

			case GLUT_KEY_F7: case GLUT_KEY_F8:
				if (hw->key.skey==GLUT_KEY_F7) hw->screen.lightmap.type=maxi(0,hw->screen.lightmap.type-1);
				else hw->screen.lightmap.type=hw->screen.lightmap.type+1;
				//ann.hold("Lightmap %d",hw->screen.lightmap.type); hw->screen.lightmap.on=1; 
				ann.hold(hw->screen.lightmap.doc());
				break;

			case GLUT_KEY_F9:  hw->screen.lightmap.ampl*=0.95   ;  ann.hold("Lightmap Amplitude: %.2f", hw->screen.lightmap.ampl); break;
			case GLUT_KEY_F10: hw->screen.lightmap.ampl*=1.05263;  ann.hold("Lightmap Amplitude: %.2f", hw->screen.lightmap.ampl); break;
			case GLUT_KEY_F11: hw->screen.lightmap.size*=0.95   ;  ann.hold("Lightmap Size: %.2f", hw->screen.lightmap.size); break;
			case GLUT_KEY_F12: hw->screen.lightmap.size*=1.05263;  ann.hold("Lightmap Size: %.2f", hw->screen.lightmap.size); break;
		}
	}

	if ((hw->key.alt==1)&&(hw->key.control==0)&&(hw->key.shift==0)) { // alt keys/camera script
		switch(hw->key.key){
			case 'r':
				camscript.record=!camscript.record;
				ann.hold_multi(camscript.record,"Record Camera Off","Record Camera On");
				if (camscript.record==1) {
					camscript.nitems=0;
					camscript.play=0;
				}
				break;
			case 'p':
				camscript.play=!camscript.play;
				hw->screen.viewport->camera.place.movemode=MANUAL;
				ann.hold_multi(camscript.play,"Playback Camera Off","Playback Camera On");
				if (camscript.play==1) camscript.record=0;
				break;
			case 's': camscript.save(hw->title); ann.hold("Saved camera script"); break;
			case 'l': camscript.load(hw->title); ann.hold("Loaded camera script");break;
			case 'm':
				if (camscript.type==BSPLINE) {camscript.type=CATMULL; ann.hold("CatmullRom interpolation");}
				else                         {camscript.type=BSPLINE; ann.hold("Bspline interpolation");}
				break;
		}
	}

	if (hw->screen.fullscreen!=hw->pref.fullscreen) {
		 hw->screen.fullscreen =hw->pref.fullscreen;
		 hw->screen.set_screen();
	}
	key_this();
}

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------


