//       texture attribute class w/
//          smart state setter/unsetter (non-sorting)

#include "texattrib.h"
#include "hardware.h"

extern PFNGLMULTITEXCOORD2FARBPROC     glMultiTexCoord2fARB;
extern PFNGLACTIVETEXTUREARBPROC       glActiveTextureARB;
//extern PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB;

//-------------------------------------------------------------------
//-------------------------------------------------------------------
//-------------------------------------------------------------------

TexAttrib::TexAttrib(){
	textures=1;
	texmagx=texmagy=1;
	texoffx=texoffy=0;
	image=0;
	layerunits[0]=0;
	layerunits[1]=0;
	layerunits[2]=0;
	layerunits[3]=0;
	layerunits[4]=0;
	function=GL_ADD;
	blend=1;
	blendFunc0=1;
	blendFunc1=1;
};
TexAttrib::~TexAttrib(){};

int TexAttrib::gtextureson[] ={0,0,0,0,0};
int TexAttrib::layerunits[] ={0,0,0,0,0};

//------------------------------------------------------------------------

void TexAttrib::set(int unitn) { // state manager for no redundant calls
	set_unit(unitn);
	switch(textures) {
		case -1:
			if (gtextureson[unitn]==0) glEnable(GL_TEXTURE_2D);
			if (gtextureson[unitn]!=-1) tex_gen(1, 1);
			if (Texture::textures[image]!=0) Texture::textures[image]->bind();
			gtextureson[unitn]=-1;
			break;
		case 0:
			if (gtextureson[unitn]!=0) glDisable(GL_TEXTURE_2D);
			if (gtextureson[unitn]<0) tex_gen(-gtextureson[unitn], 0);
			gtextureson[unitn]=0;
			break;
		case 1:
			if (gtextureson[unitn]==0) glEnable(GL_TEXTURE_2D);
			if (gtextureson[unitn]<0) tex_gen(-gtextureson[unitn], 0);
			if (Texture::textures[image]!=0) Texture::textures[image]->bind();
			gtextureson[unitn]=1;
			break;
	}
	if (unitn!=0) { // setup multitexture (A func B [X])
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);

		glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_PREVIOUS_EXT); // set A  GL_PRIMARY_COLOR_EXT);
		glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT,GL_SRC_COLOR);

		glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, function);        // set func

		glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE1_RGB_EXT, GL_TEXTURE);      // set B
		glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND1_RGB_EXT,GL_SRC_COLOR);    //         wt smj

		if (function==GL_INTERPOLATE_EXT) {
			glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE2_RGB_EXT, GL_TEXTURE);   // setup interpolate indeX texture
			glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND2_RGB_EXT,GL_ONE_MINUS_SRC_ALPHA); // wt smj
		}
	}
	set_matrix();
	set_unit(0);
}
		//glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_ADD);
		//glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_INTERPOLATE_EXT);
		//glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_MODULATE);
		//glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND2_RGB_EXT,GL_ONE_MINUS_SRC_COLOR); // wt smj
		//glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND2_RGB_EXT,GL_SRC_COLOR); // blk smj
		//glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND2_RGB_EXT,GL_SRC_ALPHA); // blk smj


    //glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE2_RGB_EXT, GL_PRIMARY_COLOR_ARB); // source 2
    //glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND2_RGB_EXT, GL_ONE_MINUS_SRC_ALPHA); // op 2

//-------------------------------------------------------------------------------

void TexAttrib::reset(void) {
	for (int i=0; i<Hw::get()->nTextureUnits; i++) {
		gtextureson[i]=0;
	}
}

//-------------------------------------------------------------------------------

void TexAttrib::unset(int unitn) {
	set_unit(unitn);
	if (gtextureson[unitn]) {
		if (gtextureson[unitn]<0) tex_gen(-gtextureson[unitn], 0); // turn off gen
		glDisable(GL_TEXTURE_2D);
		gtextureson[unitn]=0;
	}
	if (unitn!=0) set_unit(0);
}

//------------------------------------------------------------------------

void TexAttrib::set_matrix(void) {
	//set_unit(unitn);
	if ((texmagx!=1)||(texmagy!=1)||(texoffx!=0)||(texoffy!=0)) {
		glMatrixMode(GL_TEXTURE);
		glPushMatrix();
			if ((texmagx!=1)||(texmagy!=1)) glScalef(texmagx, texmagy, 1);
			if ((texoffx!=0)||(texoffy!=0)) 
				glTranslatef(Hw::get()->time.time*texoffx, // texture scrolling
							    Hw::get()->time.time*texoffy,0);
			glMatrixMode(GL_MODELVIEW);
	}
}

//-------------------------------------------------------------------------

void TexAttrib::unset_matrix(int unitn) {
	set_unit(unitn);
	if ((texmagx!=1)||(texmagy!=1)||(texoffx!=0)||(texoffy!=0)) {
		glMatrixMode(GL_TEXTURE);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
	}
}

//--------------------------------------------------------------------------

void set_unit(int unitn) {
	int units[]={GL_TEXTURE0_ARB,
		          GL_TEXTURE1_ARB,
					 GL_TEXTURE2_ARB,
					 GL_TEXTURE3_ARB};
	
	glActiveTextureARB(units[unitn]);
}

//--------------------------------------------------------------------------




