//
#include "torus.h"
#include "hardware.h"

extern PFNGLMULTITEXCOORD2FARBPROC     glMultiTexCoord2fARB;
extern PFNGLACTIVETEXTUREARBPROC       glActiveTextureARB;

//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------

Torus::Torus(){
	size=0; // if 0 use draw(size)
	thickness=.1;
	tessx=tessy=10;
	tessdistmin=50;  mindisttess=40; // close up
	tessdistmax=200; maxdisttess=10; // faraway
	tessdist=0;
	innerclip=-1; outerpoint=1;
	outermag=1;
	texmagx=1;
	texmagy=1;
};
Torus::~Torus(){};

//----------------------------------------------------------------------------

void Torus::draw(float insize){
	int tess=tessx;
	float f, fsize;

	if (size==0) fsize=insize;
	else         fsize=size;

	if (tessdist!=0) {
		f=1-((tessdist-tessdistmin) / (tessdistmax-tessdistmin)); // inverse dist
		f=clampf(f,0,1);                                        // clip 1->0
		f=pow(f,2);                                             // exp curve 
		tess=lin_interp(maxdisttess,mindisttess,f);             // interp tess
	}
	solid_torus(tess, fsize, thickness);
}

//----------------------------------------------------------------------------

int Torus::collide(Place *place, Physics *physicson){return 0;};

//----------------------------------------------------------------------------

void Torus::init(void){};

//------------------------------------------------------------------------------

void Torus::solid_torus(int tes, float rad1, float rad2) {
	int thicki, ringi, ring2, tesh=maxi(3,tes*.5), ix; 
	float ringrad, thickrad, x,y,z, sinring, cosring, sinthick,costhick;
	float ringinc=pi2 / tes, thickinc=pi2 / tesh, ty;
	 
	glPushMatrix();
		glRotatef(-90, 1,0,0);
		for       (ringi=0;  ringi <tes;    ringi++) { // around ring
			glBegin(GL_QUAD_STRIP);
				for (thicki=0; thicki<=tesh; thicki++) { // around thickness
					thickrad=thicki*thickinc;
					sinthick=sinf(thickrad);
					costhick=cosf(thickrad);
					if (costhick<innerclip)  costhick =innerclip; // shape inside
					if (costhick>outerpoint) costhick=(costhick-outerpoint)*outermag+outerpoint; // shape outside
					ty=((float)thicki / tesh)*texmagy;
					for (ix=0; ix<2; ix++) { // 2 vertices
						ring2=ringi+ix;
						ringrad=ring2*ringinc;
						sinring=sinf(ringrad);
						cosring=cosf(ringrad);
						x=cosring*(rad1+rad2*costhick);
						y=sinring*(rad1+rad2*costhick);
						z=              rad2*sinthick;
						switch(Hw::get()->pref.option) {
							case 0: glNormal3f(costhick,sinthick,sinring); break; // nice
							case 1: glNormal3f(cosring,sinring,sinthick); break; // normal
							case 2: glNormal3f(sinthick,cosring,sinring); break;
							case 3: glNormal3f(sinring,sinthick,cosring); break;
							case 4: glNormal3f(sinring,cosring,costhick); break;
							case 5: glNormal3f(sinring,cosring,sinthick); break;
							case 6: glNormal3f(cosring,sinring,costhick); break;
							case 7: glNormal3f(sinthick,sinthick,cosring); break;
						}
						if (Hw::get()->nTextureUnits>1) {
							int us[]={GL_TEXTURE0_ARB, GL_TEXTURE1_ARB};
							glMultiTexCoord2fARB(us[TexAttrib::layerunits[1]], ((float)ring2 / tes)*texmagx, ty );
						}
						else    glTexCoord2f(                 ((float)ring2 / tes)*texmagx, ty);
						glVertex3f(x,y,z);
					}
				}
				glEnd();
		}
		glPopMatrix();
	Hw::get()->ntris+=tes*tesh*2;
}

//------------------------------------------------------------------------------


